with Text_Io;

with Symbol_Table;

package Ada_Parameterization is
------------------------------------------------------------------------------
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.
--
--                  All Rights Reserved.
--
-- Permission to use, copy, modify, and distribute this software and its
-- documentation for any purpose and without fee is hereby granted,
-- provided that the above copyright notice(s) appear in all copies and that
-- both that copyright notice(s) and this permission notice appear in
-- supporting documentation, and that the name of Rational not be used in
-- advertising or publicity pertaining to distribution of the software
-- without specific, written prior permission.
--
-- Rational disclaims all warranties with regard to this software, including
-- all implied warranties of merchantability and fitness, in no event shall
-- Rational be liable for any special, indirect or consequential damages or
-- any damages whatsoever resulting from loss of use, data or profits, whether
-- in an action of contract, negligence or other tortious action, arising out
-- of or in connection with the use or performance of this software.
------------------------------------------------------------------------------

    Ada_Parameterization_Errors : exception;
------------------------------------------------------------------------------
-- Raised if we encounter errors during processing
------------------------------------------------------------------------------

    procedure Parameter_Definition (Additional_Parameters : String)
        renames Symbol_Table.Parameter_Definition;
------------------------------------------------------------------------------
--  Additional_Parameters - Specifies additional/different parameter values
--      than the default set
--
-- Called as an initialization procedure.  Sets up all of the default
-- parameter values and additionally adds or modifies any parameters that
-- are specified by the argument.
--
-- Specify parameter values with strings with these forms:
--
--      "Sun=>TRUE"                     -- any number of blanks are optional
--      "Sun => TRUE"                   -- any number of blanks are optional
--      "Sun"                           -- same as "Sun=>TRUE"
--      "Sun|Unix=>FALSE"               -- set several to one value
--      "Sun=>TRUE,Unix=>TRUE"          -- set several to several values
--
-- Blanks are allowed anywhere and are optional.  Each parameter must be
-- mentioned by name.
------------------------------------------------------------------------------

    procedure Process_File (Input     : String;  
                            Output    : String;  
                            Normalize : Boolean := False);
    procedure Process_File (Input     : Text_Io.File_Type;  
                            Output    : Text_Io.File_Type;  
                            Normalize : Boolean := False);
------------------------------------------------------------------------------
--  Input       - Specifies the file to read
--  Output      - Specifies the file to write
--  Normalize   - Specifies True to override all parameters and to remove,
--                unconditionally, all --//'s from the input file.
--
-- Called after Parameter_Definition.
--
-- Reads the input file and produces the output file.  Raises various Text_Io
-- exceptions if files cannot be read/written.  Raises
-- Ada_Parmeterization_Errors if any errors are encountered during processing.
--
-- With String parameter, Process_File opens and closes the files.  With
-- File_Type parameters, Process_File merely performs I/O.  The caller must
-- open and then close the files.
------------------------------------------------------------------------------

end Ada_Parameterization;
