with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  

package Tile_Data_Bamboo is  
------------------------------------------------------------------------------  
-- Dragon - a version of Mah-Jongg for X Windows  
--  
-- Author: Gary E. Barnes        March 1989  
--  
-- tile_data_bamboo - Mah-Jongg tile bitmap data.  
------------------------------------------------------------------------------  
-- 05/30/90 GEB  - Translate to Ada  
------------------------------------------------------------------------------  

-- *****************************************************************************  
-- Tile Sizes  
--  
-- For each size of playing surface (there are 5 sizes) we have a different  
-- size for tiles and for what goes on them.  The tile sizes are in the Tile  
-- column.  
--  
-- Many tiles (eg. Summer) are a single image.  However, some times (eg. Crak's)  
-- are made up of composite images.  These image pieces have sizes as shown  
-- in the Objs column.  
--  
-- Many tiles (eg. Summer) have writing upon them.  The sizes of lettering  
-- on these tiles is given in the Char column.  The lettering on these tiles  
-- is positioned upwards from the bottom of the tile by the number of pixels  
-- indicated in the Up column.  The letters are spaced apart by the number of  
-- pixels indicated in the Apart column.  
--  
--       Tile    Objs    Char    Up      Apart  
--  
--       28x32   7x8     4x6     1       1  
--       40x48   10x12   5x7     1       1  
--       56x64   14x16   7x11    7       1  
--       68x80   17x20   9x13    11      2  
--       80x96   20x24   9x13    16      3  
--  
-- ****************************************************************************  
--
------------------------------------------------------------------------------  
----FLOWERS  
------------------------------------------------------------------------------  

    Bamboo_28x32_Width  : constant := 28;  
    Bamboo_28x32_Height : constant := 32;  

    Bamboo_28x32_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#80#, 16#00#, 16#00#,  
        16#0B#, 16#41#, 16#00#, 16#00#, 16#1E#, 16#42#, 16#80#, 16#00#,  
        16#1A#, 16#E6#, 16#E0#, 16#00#, 16#0F#, 16#A9#, 16#90#, 16#00#,  
        16#39#, 16#CA#, 16#A6#, 16#00#, 16#23#, 16#8F#, 16#CA#, 16#00#,  
        16#3E#, 16#C3#, 16#1A#, 16#00#, 16#00#, 16#C3#, 16#14#, 16#00#,  
        16#1C#, 16#63#, 16#1C#, 16#00#, 16#12#, 16#67#, 16#30#, 16#00#,  
        16#11#, 16#66#, 16#70#, 16#00#, 16#0A#, 16#E6#, 16#60#, 16#00#,  
        16#0A#, 16#76#, 16#CF#, 16#00#, 16#09#, 16#36#, 16#D0#, 16#80#,  
        16#04#, 16#B6#, 16#EE#, 16#40#, 16#03#, 16#76#, 16#F1#, 16#C0#,  
        16#00#, 16#BF#, 16#C0#, 16#00#, 16#00#, 16#FF#, 16#C0#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#63#, 16#2B#, 16#18#, 16#C0#,  
        16#54#, 16#BA#, 16#A5#, 16#20#, 16#64#, 16#BB#, 16#25#, 16#20#,  
        16#57#, 16#AA#, 16#A5#, 16#20#, 16#54#, 16#AA#, 16#A5#, 16#20#,  
        16#64#, 16#AB#, 16#18#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Bamboo_40x48_Width  : constant := 40;  
    Bamboo_40x48_Height : constant := 48;  

    Bamboo_40x48_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#24#,  
        16#00#, 16#00#, 16#00#, 16#03#, 16#44#, 16#06#, 16#00#, 16#00#,  
        16#04#, 16#C4#, 16#09#, 16#00#, 16#00#, 16#04#, 16#5F#, 16#09#,  
        16#00#, 16#00#, 16#05#, 16#71#, 16#7B#, 16#E0#, 16#00#, 16#03#,  
        16#F2#, 16#8E#, 16#10#, 16#00#, 16#0E#, 16#3C#, 16#8E#, 16#11#,  
        16#80#, 16#10#, 16#38#, 16#7F#, 16#E7#, 16#80#, 16#10#, 16#58#,  
        16#0C#, 16#0C#, 16#80#, 16#1F#, 16#9C#, 16#0C#, 16#1B#, 16#00#,  
        16#00#, 16#1C#, 16#0C#, 16#15#, 16#00#, 16#00#, 16#0E#, 16#0C#,  
        16#1D#, 16#00#, 16#06#, 16#0E#, 16#0C#, 16#1E#, 16#00#, 16#07#,  
        16#0E#, 16#0C#, 16#38#, 16#00#, 16#04#, 16#C7#, 16#1C#, 16#30#,  
        16#00#, 16#06#, 16#47#, 16#18#, 16#70#, 16#00#, 16#02#, 16#27#,  
        16#18#, 16#E0#, 16#00#, 16#02#, 16#B7#, 16#18#, 16#C0#, 16#00#,  
        16#02#, 16#57#, 16#19#, 16#9F#, 16#C0#, 16#02#, 16#4B#, 16#99#,  
        16#AF#, 16#60#, 16#01#, 16#2B#, 16#BB#, 16#C0#, 16#30#, 16#00#,  
        16#9F#, 16#BB#, 16#8E#, 16#10#, 16#00#, 16#47#, 16#BB#, 16#B1#,  
        16#F8#, 16#00#, 16#27#, 16#BF#, 16#40#, 16#00#, 16#00#, 16#13#,  
        16#FF#, 16#40#, 16#00#, 16#00#, 16#0B#, 16#FF#, 16#C0#, 16#00#,  
        16#00#, 16#0F#, 16#FF#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#3C#, 16#72#, 16#2F#, 16#1C#, 16#70#,  
        16#22#, 16#8B#, 16#68#, 16#A2#, 16#88#, 16#22#, 16#8B#, 16#68#,  
        16#A2#, 16#88#, 16#3C#, 16#8B#, 16#6F#, 16#22#, 16#88#, 16#22#,  
        16#FA#, 16#A8#, 16#A2#, 16#88#, 16#22#, 16#8A#, 16#A8#, 16#A2#,  
        16#88#, 16#22#, 16#8A#, 16#A8#, 16#A2#, 16#88#, 16#3C#, 16#8A#,  
        16#2F#, 16#1C#, 16#70#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Bamboo_56x64_Width  : constant := 56;  
    Bamboo_56x64_Height : constant := 64;  

    Bamboo_56x64_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#0C#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#12#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#A2#, 16#03#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#,  
        16#62#, 16#04#, 16#80#, 16#00#, 16#00#, 16#00#, 16#02#, 16#2F#,  
        16#84#, 16#80#, 16#00#, 16#00#, 16#00#, 16#02#, 16#B8#, 16#BD#,  
        16#F0#, 16#00#, 16#00#, 16#00#, 16#01#, 16#F9#, 16#47#, 16#08#,  
        16#00#, 16#00#, 16#00#, 16#07#, 16#1E#, 16#47#, 16#08#, 16#C0#,  
        16#00#, 16#00#, 16#08#, 16#1C#, 16#3F#, 16#F3#, 16#C0#, 16#00#,  
        16#00#, 16#08#, 16#2C#, 16#06#, 16#06#, 16#40#, 16#00#, 16#00#,  
        16#0F#, 16#CE#, 16#06#, 16#0D#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#0E#, 16#06#, 16#0A#, 16#80#, 16#00#, 16#00#, 16#00#, 16#07#,  
        16#06#, 16#0E#, 16#80#, 16#00#, 16#00#, 16#03#, 16#07#, 16#06#,  
        16#0F#, 16#00#, 16#00#, 16#00#, 16#03#, 16#87#, 16#06#, 16#1C#,  
        16#00#, 16#00#, 16#00#, 16#02#, 16#63#, 16#8E#, 16#18#, 16#00#,  
        16#00#, 16#00#, 16#03#, 16#23#, 16#8C#, 16#38#, 16#00#, 16#00#,  
        16#00#, 16#01#, 16#13#, 16#8C#, 16#70#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#5B#, 16#8C#, 16#60#, 16#00#, 16#00#, 16#00#, 16#01#,  
        16#2B#, 16#8C#, 16#CF#, 16#E0#, 16#00#, 16#00#, 16#01#, 16#25#,  
        16#CC#, 16#D7#, 16#B0#, 16#00#, 16#00#, 16#00#, 16#95#, 16#DD#,  
        16#E0#, 16#18#, 16#00#, 16#00#, 16#00#, 16#4F#, 16#DD#, 16#C7#,  
        16#08#, 16#00#, 16#00#, 16#00#, 16#23#, 16#DD#, 16#D8#, 16#FC#,  
        16#00#, 16#00#, 16#00#, 16#13#, 16#DF#, 16#A0#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#09#, 16#FF#, 16#A0#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#05#, 16#FF#, 16#E0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#07#, 16#FF#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#0F#, 16#83#, 16#88#, 16#2F#, 16#83#, 16#83#,  
        16#80#, 16#08#, 16#44#, 16#4C#, 16#68#, 16#44#, 16#44#, 16#40#,  
        16#08#, 16#28#, 16#2C#, 16#68#, 16#28#, 16#28#, 16#20#, 16#08#,  
        16#28#, 16#2C#, 16#68#, 16#28#, 16#28#, 16#20#, 16#08#, 16#48#,  
        16#2A#, 16#A8#, 16#48#, 16#28#, 16#20#, 16#0F#, 16#8F#, 16#EA#,  
        16#AF#, 16#88#, 16#28#, 16#20#, 16#08#, 16#48#, 16#2A#, 16#A8#,  
        16#48#, 16#28#, 16#20#, 16#08#, 16#28#, 16#29#, 16#28#, 16#28#,  
        16#28#, 16#20#, 16#08#, 16#28#, 16#29#, 16#28#, 16#28#, 16#28#,  
        16#20#, 16#08#, 16#48#, 16#29#, 16#28#, 16#44#, 16#44#, 16#40#,  
        16#0F#, 16#88#, 16#28#, 16#2F#, 16#83#, 16#83#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Bamboo_68x80_Width  : constant := 68;  
    Bamboo_68x80_Height : constant := 80;  

    Bamboo_68x80_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#60#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#90#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#0D#, 16#10#, 16#18#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#13#, 16#10#, 16#24#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#11#, 16#7C#, 16#24#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#15#, 16#C5#, 16#EF#,  
        16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0F#, 16#CA#,  
        16#38#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#38#,  
        16#F2#, 16#38#, 16#46#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#40#, 16#E1#, 16#FF#, 16#9E#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#41#, 16#60#, 16#30#, 16#32#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#7E#, 16#70#, 16#30#, 16#6C#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#70#, 16#30#, 16#54#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#38#, 16#30#, 16#74#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#18#, 16#38#, 16#30#,  
        16#78#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#1C#, 16#38#,  
        16#30#, 16#E0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#13#,  
        16#1C#, 16#70#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#19#, 16#1C#, 16#61#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#08#, 16#9C#, 16#63#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#0A#, 16#DC#, 16#63#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#09#, 16#5C#, 16#66#, 16#7F#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#09#, 16#2E#, 16#66#, 16#BD#,  
        16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#04#, 16#AE#, 16#EF#,  
        16#00#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#7E#,  
        16#EE#, 16#38#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#,  
        16#1E#, 16#EE#, 16#C7#, 16#E0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#9E#, 16#FD#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#4F#, 16#FD#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#2F#, 16#FF#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#3F#, 16#FE#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#3F#, 16#81#, 16#F0#, 16#80#, 16#9F#, 16#C0#, 16#F8#, 16#1F#,  
        16#00#, 16#20#, 16#42#, 16#08#, 16#C1#, 16#90#, 16#21#, 16#04#,  
        16#20#, 16#80#, 16#20#, 16#24#, 16#04#, 16#C1#, 16#90#, 16#12#,  
        16#02#, 16#40#, 16#40#, 16#20#, 16#24#, 16#04#, 16#C1#, 16#90#,  
        16#12#, 16#02#, 16#40#, 16#40#, 16#20#, 16#24#, 16#04#, 16#A2#,  
        16#90#, 16#12#, 16#02#, 16#40#, 16#40#, 16#20#, 16#44#, 16#04#,  
        16#A2#, 16#90#, 16#22#, 16#02#, 16#40#, 16#40#, 16#3F#, 16#87#,  
        16#FC#, 16#A2#, 16#9F#, 16#C2#, 16#02#, 16#40#, 16#40#, 16#20#,  
        16#44#, 16#04#, 16#94#, 16#90#, 16#22#, 16#02#, 16#40#, 16#40#,  
        16#20#, 16#24#, 16#04#, 16#94#, 16#90#, 16#12#, 16#02#, 16#40#,  
        16#40#, 16#20#, 16#24#, 16#04#, 16#94#, 16#90#, 16#12#, 16#02#,  
        16#40#, 16#40#, 16#20#, 16#24#, 16#04#, 16#88#, 16#90#, 16#12#,  
        16#02#, 16#40#, 16#40#, 16#20#, 16#44#, 16#04#, 16#88#, 16#90#,  
        16#21#, 16#04#, 16#20#, 16#80#, 16#3F#, 16#84#, 16#04#, 16#88#,  
        16#9F#, 16#C0#, 16#F8#, 16#1F#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Bamboo_80x96_Width  : constant := 80;  
    Bamboo_80x96_Height : constant := 96;  

    Bamboo_80x96_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#,  
        16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#34#, 16#40#, 16#60#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#4C#, 16#40#, 16#90#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#45#, 16#F0#, 16#90#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#57#,  
        16#17#, 16#BE#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#3F#, 16#28#, 16#E1#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#E3#, 16#C8#, 16#E1#, 16#18#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#03#, 16#87#, 16#FE#,  
        16#78#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#05#,  
        16#80#, 16#C0#, 16#C8#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#F9#, 16#C0#, 16#C1#, 16#B0#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#01#, 16#C0#, 16#C1#, 16#50#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#E0#, 16#C1#,  
        16#D0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#60#,  
        16#E0#, 16#C1#, 16#E0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#70#, 16#E0#, 16#C3#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#4C#, 16#71#, 16#C3#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#64#, 16#71#, 16#87#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#22#,  
        16#71#, 16#8E#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#2B#, 16#71#, 16#8C#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#25#, 16#71#, 16#99#, 16#FC#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#24#, 16#B9#, 16#9A#,  
        16#F6#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#12#,  
        16#BB#, 16#BC#, 16#03#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#09#, 16#FB#, 16#B8#, 16#E1#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#04#, 16#7B#, 16#BB#, 16#1F#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#7B#, 16#F4#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#,  
        16#3F#, 16#F4#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#BF#, 16#FC#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#FF#, 16#F8#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#07#, 16#F0#,  
        16#1F#, 16#04#, 16#04#, 16#7F#, 16#01#, 16#F0#, 16#1F#, 16#00#,  
        16#04#, 16#08#, 16#20#, 16#86#, 16#0C#, 16#40#, 16#82#, 16#08#,  
        16#20#, 16#80#, 16#04#, 16#04#, 16#40#, 16#46#, 16#0C#, 16#40#,  
        16#44#, 16#04#, 16#40#, 16#40#, 16#04#, 16#04#, 16#40#, 16#46#,  
        16#0C#, 16#40#, 16#44#, 16#04#, 16#40#, 16#40#, 16#04#, 16#04#,  
        16#40#, 16#45#, 16#14#, 16#40#, 16#44#, 16#04#, 16#40#, 16#40#,  
        16#04#, 16#08#, 16#40#, 16#45#, 16#14#, 16#40#, 16#84#, 16#04#,  
        16#40#, 16#40#, 16#07#, 16#F0#, 16#7F#, 16#C5#, 16#14#, 16#7F#,  
        16#04#, 16#04#, 16#40#, 16#40#, 16#04#, 16#08#, 16#40#, 16#44#,  
        16#A4#, 16#40#, 16#84#, 16#04#, 16#40#, 16#40#, 16#04#, 16#04#,  
        16#40#, 16#44#, 16#A4#, 16#40#, 16#44#, 16#04#, 16#40#, 16#40#,  
        16#04#, 16#04#, 16#40#, 16#44#, 16#A4#, 16#40#, 16#44#, 16#04#,  
        16#40#, 16#40#, 16#04#, 16#04#, 16#40#, 16#44#, 16#44#, 16#40#,  
        16#44#, 16#04#, 16#40#, 16#40#, 16#04#, 16#08#, 16#40#, 16#44#,  
        16#44#, 16#40#, 16#82#, 16#08#, 16#20#, 16#80#, 16#07#, 16#F0#,  
        16#40#, 16#44#, 16#44#, 16#7F#, 16#01#, 16#F0#, 16#1F#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

end Tile_Data_Bamboo;  

