with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  

package Tile_Data_Gdragon is  
------------------------------------------------------------------------------  
-- Dragon - a version of Mah-Jongg for X Windows  
--  
-- Author: Gary E. Barnes        March 1989  
--  
-- tile_data_gdragon - Mah-Jongg tile bitmap data.  
------------------------------------------------------------------------------  
-- 05/30/90 GEB  - Translate to Ada  
------------------------------------------------------------------------------  

-- *****************************************************************************  
-- Tile Sizes  
--  
-- For each size of playing surface (there are 5 sizes) we have a different  
-- size for tiles and for what goes on them.  The tile sizes are in the Tile  
-- column.  
--  
-- Many tiles (eg. Summer) are a single image.  However, some times (eg. Crak's)  
-- are made up of composite images.  These image pieces have sizes as shown  
-- in the Objs column.  
--  
-- Many tiles (eg. Summer) have writing upon them.  The sizes of lettering  
-- on these tiles is given in the Char column.  The lettering on these tiles  
-- is positioned upwards from the bottom of the tile by the number of pixels  
-- indicated in the Up column.  The letters are spaced apart by the number of  
-- pixels indicated in the Apart column.  
--  
--       Tile    Objs    Char    Up      Apart  
--  
--       28x32   7x8     4x6     1       1  
--       40x48   10x12   5x7     1       1  
--       56x64   14x16   7x11    7       1  
--       68x80   17x20   9x13    11      2  
--       80x96   20x24   9x13    16      3  
--  
-- ****************************************************************************  
--
------------------------------------------------------------------------------  
----DRAGONS  
------------------------------------------------------------------------------  

    Gdragon_28x32_Width  : constant := 28;  
    Gdragon_28x32_Height : constant := 32;  

    Gdragon_28x32_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#C0#,  
        16#00#, 16#00#, 16#02#, 16#20#, 16#00#, 16#00#, 16#02#, 16#00#,  
        16#00#, 16#00#, 16#02#, 16#60#, 16#00#, 16#00#, 16#02#, 16#20#,  
        16#00#, 16#00#, 16#01#, 16#C0#, 16#00#, 16#1F#, 16#C0#, 16#00#,  
        16#00#, 16#60#, 16#30#, 16#00#, 16#01#, 16#80#, 16#0C#, 16#00#,  
        16#02#, 16#0F#, 16#82#, 16#00#, 16#04#, 16#30#, 16#61#, 16#00#,  
        16#04#, 16#C0#, 16#19#, 16#00#, 16#08#, 16#8F#, 16#88#, 16#80#,  
        16#09#, 16#10#, 16#44#, 16#80#, 16#11#, 16#2A#, 16#A4#, 16#40#,  
        16#12#, 16#57#, 16#52#, 16#40#, 16#12#, 16#48#, 16#92#, 16#40#,  
        16#12#, 16#53#, 16#52#, 16#40#, 16#12#, 16#48#, 16#52#, 16#40#,  
        16#12#, 16#47#, 16#92#, 16#40#, 16#11#, 16#20#, 16#24#, 16#40#,  
        16#09#, 16#10#, 16#44#, 16#80#, 16#08#, 16#8F#, 16#88#, 16#80#,  
        16#04#, 16#C0#, 16#19#, 16#00#, 16#04#, 16#30#, 16#61#, 16#00#,  
        16#02#, 16#0F#, 16#82#, 16#00#, 16#01#, 16#80#, 16#0C#, 16#00#,  
        16#00#, 16#60#, 16#30#, 16#00#, 16#00#, 16#1F#, 16#C0#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Gdragon_40x48_Width  : constant := 40;  
    Gdragon_40x48_Height : constant := 48;  

    Gdragon_40x48_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#38#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#44#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#4C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#44#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#38#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#01#, 16#FF#, 16#00#, 16#00#, 16#00#,  
        16#06#, 16#00#, 16#C0#, 16#00#, 16#00#, 16#18#, 16#00#, 16#30#,  
        16#00#, 16#00#, 16#60#, 16#FE#, 16#0C#, 16#00#, 16#00#, 16#87#,  
        16#01#, 16#C2#, 16#00#, 16#01#, 16#08#, 16#00#, 16#21#, 16#00#,  
        16#02#, 16#30#, 16#FE#, 16#18#, 16#80#, 16#02#, 16#43#, 16#01#,  
        16#84#, 16#80#, 16#04#, 16#44#, 16#00#, 16#44#, 16#40#, 16#04#,  
        16#88#, 16#7C#, 16#22#, 16#40#, 16#09#, 16#11#, 16#83#, 16#11#,  
        16#20#, 16#09#, 16#22#, 16#00#, 16#89#, 16#20#, 16#11#, 16#24#,  
        16#AA#, 16#49#, 16#10#, 16#12#, 16#45#, 16#7D#, 16#44#, 16#90#,  
        16#12#, 16#49#, 16#83#, 16#24#, 16#90#, 16#12#, 16#4A#, 16#1C#,  
        16#A4#, 16#90#, 16#12#, 16#4A#, 16#36#, 16#A4#, 16#90#, 16#12#,  
        16#49#, 16#1C#, 16#A4#, 16#90#, 16#12#, 16#48#, 16#C1#, 16#24#,  
        16#90#, 16#12#, 16#44#, 16#7E#, 16#44#, 16#90#, 16#11#, 16#24#,  
        16#00#, 16#49#, 16#10#, 16#09#, 16#22#, 16#00#, 16#89#, 16#20#,  
        16#09#, 16#11#, 16#83#, 16#11#, 16#20#, 16#04#, 16#88#, 16#7C#,  
        16#22#, 16#40#, 16#04#, 16#44#, 16#00#, 16#44#, 16#40#, 16#02#,  
        16#43#, 16#01#, 16#84#, 16#80#, 16#02#, 16#30#, 16#FE#, 16#18#,  
        16#80#, 16#01#, 16#08#, 16#00#, 16#21#, 16#00#, 16#00#, 16#87#,  
        16#01#, 16#C2#, 16#00#, 16#00#, 16#60#, 16#FE#, 16#0C#, 16#00#,  
        16#00#, 16#18#, 16#00#, 16#30#, 16#00#, 16#00#, 16#06#, 16#00#,  
        16#C0#, 16#00#, 16#00#, 16#01#, 16#FF#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Gdragon_56x64_Width  : constant := 56;  
    Gdragon_56x64_Height : constant := 64;  

    Gdragon_56x64_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#70#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#88#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#01#, 16#04#, 16#00#, 16#00#, 16#03#, 16#FF#, 16#80#,  
        16#01#, 16#00#, 16#00#, 16#00#, 16#1F#, 16#FF#, 16#F0#, 16#01#,  
        16#00#, 16#00#, 16#00#, 16#FC#, 16#00#, 16#7E#, 16#01#, 16#00#,  
        16#00#, 16#03#, 16#E0#, 16#00#, 16#0F#, 16#81#, 16#1C#, 16#00#,  
        16#07#, 16#00#, 16#00#, 16#01#, 16#C1#, 16#04#, 16#00#, 16#1E#,  
        16#01#, 16#FF#, 16#00#, 16#F1#, 16#04#, 16#00#, 16#38#, 16#0F#,  
        16#FF#, 16#E0#, 16#38#, 16#88#, 16#00#, 16#70#, 16#7E#, 16#00#,  
        16#FC#, 16#1C#, 16#70#, 16#00#, 16#C0#, 16#F0#, 16#00#, 16#1E#,  
        16#06#, 16#00#, 16#01#, 16#83#, 16#C0#, 16#00#, 16#07#, 16#83#,  
        16#00#, 16#03#, 16#87#, 16#01#, 16#FF#, 16#01#, 16#C3#, 16#80#,  
        16#03#, 16#0E#, 16#07#, 16#FF#, 16#C0#, 16#E1#, 16#80#, 16#06#,  
        16#1C#, 16#1F#, 16#01#, 16#F0#, 16#70#, 16#C0#, 16#0E#, 16#38#,  
        16#78#, 16#00#, 16#3C#, 16#38#, 16#E0#, 16#0C#, 16#30#, 16#E0#,  
        16#00#, 16#0E#, 16#18#, 16#60#, 16#18#, 16#61#, 16#C0#, 16#FE#,  
        16#07#, 16#0C#, 16#30#, 16#18#, 16#E3#, 16#83#, 16#FF#, 16#83#,  
        16#8E#, 16#30#, 16#18#, 16#C3#, 16#0F#, 16#01#, 16#E1#, 16#86#,  
        16#30#, 16#30#, 16#C6#, 16#1C#, 16#00#, 16#70#, 16#C6#, 16#18#,  
        16#31#, 16#86#, 16#38#, 16#00#, 16#38#, 16#C3#, 16#18#, 16#31#,  
        16#8C#, 16#30#, 16#7C#, 16#18#, 16#63#, 16#18#, 16#61#, 16#8C#,  
        16#61#, 16#FF#, 16#0C#, 16#63#, 16#0C#, 16#63#, 16#1C#, 16#63#,  
        16#83#, 16#8C#, 16#71#, 16#8C#, 16#63#, 16#18#, 16#C3#, 16#29#,  
        16#86#, 16#31#, 16#8C#, 16#63#, 16#18#, 16#C6#, 16#BA#, 16#C6#,  
        16#31#, 16#8C#, 16#63#, 16#18#, 16#C6#, 16#44#, 16#C6#, 16#31#,  
        16#8C#, 16#63#, 16#18#, 16#C6#, 16#9A#, 16#C6#, 16#31#, 16#8C#,  
        16#63#, 16#18#, 16#C6#, 16#42#, 16#C6#, 16#31#, 16#8C#, 16#63#,  
        16#18#, 16#C6#, 16#3C#, 16#C6#, 16#31#, 16#8C#, 16#63#, 16#18#,  
        16#C3#, 16#01#, 16#86#, 16#31#, 16#8C#, 16#63#, 16#1C#, 16#63#,  
        16#83#, 16#8C#, 16#71#, 16#8C#, 16#61#, 16#8C#, 16#61#, 16#FF#,  
        16#0C#, 16#63#, 16#0C#, 16#31#, 16#8C#, 16#30#, 16#7C#, 16#18#,  
        16#63#, 16#18#, 16#31#, 16#86#, 16#38#, 16#00#, 16#38#, 16#C3#,  
        16#18#, 16#30#, 16#C6#, 16#1C#, 16#00#, 16#70#, 16#C6#, 16#18#,  
        16#18#, 16#C3#, 16#0F#, 16#01#, 16#E1#, 16#86#, 16#30#, 16#18#,  
        16#E3#, 16#83#, 16#FF#, 16#83#, 16#8E#, 16#30#, 16#18#, 16#61#,  
        16#C0#, 16#FE#, 16#07#, 16#0C#, 16#30#, 16#0C#, 16#30#, 16#E0#,  
        16#00#, 16#0E#, 16#18#, 16#60#, 16#0E#, 16#38#, 16#78#, 16#00#,  
        16#3C#, 16#38#, 16#E0#, 16#06#, 16#1C#, 16#1F#, 16#01#, 16#F0#,  
        16#70#, 16#C0#, 16#03#, 16#0E#, 16#07#, 16#FF#, 16#C0#, 16#E1#,  
        16#80#, 16#03#, 16#87#, 16#01#, 16#FF#, 16#01#, 16#C3#, 16#80#,  
        16#01#, 16#83#, 16#C0#, 16#00#, 16#07#, 16#83#, 16#00#, 16#00#,  
        16#C0#, 16#F0#, 16#00#, 16#1E#, 16#06#, 16#00#, 16#00#, 16#70#,  
        16#7E#, 16#00#, 16#FC#, 16#1C#, 16#00#, 16#00#, 16#38#, 16#0F#,  
        16#FF#, 16#E0#, 16#38#, 16#00#, 16#00#, 16#1E#, 16#01#, 16#FF#,  
        16#00#, 16#F0#, 16#00#, 16#00#, 16#07#, 16#00#, 16#00#, 16#01#,  
        16#C0#, 16#00#, 16#00#, 16#03#, 16#E0#, 16#00#, 16#0F#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#FC#, 16#00#, 16#7E#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#1F#, 16#FF#, 16#F0#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#03#, 16#FF#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Gdragon_68x80_Width  : constant := 68;  
    Gdragon_68x80_Height : constant := 80;  

    Gdragon_68x80_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#1F#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#20#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#40#, 16#40#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#FF#, 16#80#, 16#00#,  
        16#43#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#3F#, 16#FF#, 16#F8#,  
        16#00#, 16#40#, 16#40#, 16#00#, 16#00#, 16#00#, 16#FC#, 16#00#,  
        16#7E#, 16#00#, 16#40#, 16#40#, 16#00#, 16#00#, 16#03#, 16#E0#,  
        16#00#, 16#0F#, 16#80#, 16#40#, 16#40#, 16#00#, 16#00#, 16#0F#,  
        16#01#, 16#FF#, 16#01#, 16#E0#, 16#20#, 16#80#, 16#00#, 16#00#,  
        16#1C#, 16#1F#, 16#FF#, 16#F0#, 16#70#, 16#1F#, 16#00#, 16#00#,  
        16#00#, 16#70#, 16#7E#, 16#00#, 16#FC#, 16#1C#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#E1#, 16#E0#, 16#00#, 16#0F#, 16#0E#, 16#00#,  
        16#00#, 16#00#, 16#01#, 16#C7#, 16#80#, 16#00#, 16#03#, 16#C7#,  
        16#00#, 16#00#, 16#00#, 16#03#, 16#8E#, 16#01#, 16#FF#, 16#00#,  
        16#E3#, 16#80#, 16#00#, 16#00#, 16#07#, 16#1C#, 16#0F#, 16#FF#,  
        16#E0#, 16#71#, 16#C0#, 16#00#, 16#00#, 16#06#, 16#30#, 16#3E#,  
        16#00#, 16#F8#, 16#18#, 16#C0#, 16#00#, 16#00#, 16#0C#, 16#60#,  
        16#F0#, 16#00#, 16#1E#, 16#0C#, 16#60#, 16#00#, 16#00#, 16#18#,  
        16#C1#, 16#C0#, 16#00#, 16#07#, 16#06#, 16#30#, 16#00#, 16#00#,  
        16#19#, 16#C3#, 16#80#, 16#FE#, 16#03#, 16#87#, 16#30#, 16#00#,  
        16#00#, 16#31#, 16#86#, 16#07#, 16#FF#, 16#C0#, 16#C3#, 16#18#,  
        16#00#, 16#00#, 16#33#, 16#0C#, 16#1F#, 16#01#, 16#F0#, 16#61#,  
        16#98#, 16#00#, 16#00#, 16#63#, 16#1C#, 16#38#, 16#00#, 16#38#,  
        16#71#, 16#8C#, 16#00#, 16#00#, 16#66#, 16#18#, 16#70#, 16#00#,  
        16#1C#, 16#30#, 16#CC#, 16#00#, 16#00#, 16#E6#, 16#30#, 16#C0#,  
        16#FE#, 16#07#, 16#18#, 16#CE#, 16#00#, 16#00#, 16#CC#, 16#31#,  
        16#C3#, 16#FF#, 16#87#, 16#18#, 16#66#, 16#00#, 16#00#, 16#CC#,  
        16#61#, 16#87#, 16#83#, 16#C3#, 16#0C#, 16#66#, 16#00#, 16#00#,  
        16#CC#, 16#63#, 16#0E#, 16#00#, 16#E1#, 16#8C#, 16#66#, 16#00#,  
        16#01#, 16#8C#, 16#63#, 16#1C#, 16#54#, 16#71#, 16#8C#, 16#63#,  
        16#00#, 16#01#, 16#98#, 16#C3#, 16#18#, 16#29#, 16#31#, 16#86#,  
        16#33#, 16#00#, 16#01#, 16#98#, 16#C6#, 16#39#, 16#7E#, 16#B8#,  
        16#C6#, 16#33#, 16#00#, 16#01#, 16#98#, 16#C6#, 16#30#, 16#83#,  
        16#18#, 16#C6#, 16#33#, 16#00#, 16#01#, 16#98#, 16#C6#, 16#35#,  
        16#01#, 16#18#, 16#C6#, 16#33#, 16#00#, 16#01#, 16#98#, 16#C6#,  
        16#32#, 16#18#, 16#98#, 16#C6#, 16#33#, 16#00#, 16#01#, 16#98#,  
        16#C6#, 16#36#, 16#18#, 16#98#, 16#C6#, 16#33#, 16#00#, 16#01#,  
        16#98#, 16#C6#, 16#33#, 16#01#, 16#18#, 16#C6#, 16#33#, 16#00#,  
        16#01#, 16#98#, 16#C6#, 16#38#, 16#C2#, 16#38#, 16#C6#, 16#33#,  
        16#00#, 16#01#, 16#98#, 16#C3#, 16#18#, 16#3C#, 16#31#, 16#86#,  
        16#33#, 16#00#, 16#01#, 16#8C#, 16#63#, 16#1C#, 16#00#, 16#71#,  
        16#8C#, 16#63#, 16#00#, 16#00#, 16#CC#, 16#63#, 16#0E#, 16#00#,  
        16#E1#, 16#8C#, 16#66#, 16#00#, 16#00#, 16#CC#, 16#61#, 16#87#,  
        16#83#, 16#C3#, 16#0C#, 16#66#, 16#00#, 16#00#, 16#CC#, 16#31#,  
        16#C3#, 16#FF#, 16#87#, 16#18#, 16#66#, 16#00#, 16#00#, 16#E6#,  
        16#30#, 16#C0#, 16#FE#, 16#06#, 16#18#, 16#CE#, 16#00#, 16#00#,  
        16#66#, 16#18#, 16#70#, 16#00#, 16#1C#, 16#30#, 16#CC#, 16#00#,  
        16#00#, 16#63#, 16#1C#, 16#38#, 16#00#, 16#38#, 16#71#, 16#8C#,  
        16#00#, 16#00#, 16#33#, 16#0C#, 16#1F#, 16#01#, 16#F0#, 16#61#,  
        16#98#, 16#00#, 16#00#, 16#31#, 16#86#, 16#07#, 16#FF#, 16#C0#,  
        16#C3#, 16#18#, 16#00#, 16#00#, 16#19#, 16#C3#, 16#80#, 16#FE#,  
        16#03#, 16#87#, 16#30#, 16#00#, 16#00#, 16#18#, 16#C1#, 16#C0#,  
        16#00#, 16#07#, 16#06#, 16#30#, 16#00#, 16#00#, 16#0C#, 16#60#,  
        16#F0#, 16#00#, 16#1E#, 16#0C#, 16#60#, 16#00#, 16#00#, 16#06#,  
        16#30#, 16#3E#, 16#00#, 16#F8#, 16#18#, 16#C0#, 16#00#, 16#00#,  
        16#07#, 16#1C#, 16#0F#, 16#FF#, 16#E0#, 16#71#, 16#C0#, 16#00#,  
        16#00#, 16#03#, 16#8E#, 16#01#, 16#FF#, 16#00#, 16#E3#, 16#80#,  
        16#00#, 16#00#, 16#01#, 16#C7#, 16#80#, 16#00#, 16#03#, 16#C7#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#E1#, 16#E0#, 16#00#, 16#0F#,  
        16#0E#, 16#00#, 16#00#, 16#00#, 16#00#, 16#70#, 16#7E#, 16#00#,  
        16#FC#, 16#1C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#1C#, 16#1F#,  
        16#FF#, 16#F0#, 16#70#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0F#,  
        16#01#, 16#FF#, 16#01#, 16#E0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#03#, 16#E0#, 16#00#, 16#0F#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#FC#, 16#00#, 16#7E#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#3F#, 16#FF#, 16#F8#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#FF#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Gdragon_80x96_Width  : constant := 80;  
    Gdragon_80x96_Height : constant := 96;  

    Gdragon_80x96_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#F0#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#08#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#04#, 16#04#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#04#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#04#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#04#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#04#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#04#, 16#3C#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#04#, 16#04#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#04#, 16#04#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#1F#, 16#FC#, 16#00#, 16#00#,  
        16#04#, 16#04#, 16#00#, 16#00#, 16#00#, 16#01#, 16#FF#, 16#FF#,  
        16#C0#, 16#00#, 16#02#, 16#08#, 16#00#, 16#00#, 16#00#, 16#0F#,  
        16#E0#, 16#03#, 16#F8#, 16#00#, 16#01#, 16#F0#, 16#00#, 16#00#,  
        16#00#, 16#3E#, 16#00#, 16#00#, 16#3E#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#F0#, 16#00#, 16#00#, 16#07#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#C0#, 16#1F#, 16#FC#,  
        16#01#, 16#E0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#07#, 16#01#,  
        16#FF#, 16#FF#, 16#C0#, 16#70#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#1C#, 16#07#, 16#E0#, 16#03#, 16#F0#, 16#1C#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#38#, 16#1F#, 16#00#, 16#00#, 16#7C#, 16#0E#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#70#, 16#78#, 16#00#, 16#00#,  
        16#0F#, 16#07#, 16#00#, 16#00#, 16#00#, 16#00#, 16#E0#, 16#E0#,  
        16#00#, 16#00#, 16#03#, 16#83#, 16#80#, 16#00#, 16#00#, 16#01#,  
        16#C3#, 16#80#, 16#0F#, 16#F8#, 16#00#, 16#E1#, 16#C0#, 16#00#,  
        16#00#, 16#03#, 16#87#, 16#00#, 16#FF#, 16#FF#, 16#80#, 16#70#,  
        16#E0#, 16#00#, 16#00#, 16#03#, 16#0E#, 16#03#, 16#F0#, 16#07#,  
        16#E0#, 16#38#, 16#60#, 16#00#, 16#00#, 16#06#, 16#1C#, 16#0F#,  
        16#80#, 16#00#, 16#F8#, 16#1C#, 16#30#, 16#00#, 16#00#, 16#0C#,  
        16#38#, 16#1C#, 16#00#, 16#00#, 16#1C#, 16#0E#, 16#18#, 16#00#,  
        16#00#, 16#0C#, 16#30#, 16#38#, 16#00#, 16#00#, 16#0E#, 16#06#,  
        16#18#, 16#00#, 16#00#, 16#18#, 16#60#, 16#60#, 16#0F#, 16#F8#,  
        16#03#, 16#03#, 16#0C#, 16#00#, 16#00#, 16#18#, 16#C0#, 16#C0#,  
        16#3F#, 16#FE#, 16#01#, 16#81#, 16#8C#, 16#00#, 16#00#, 16#30#,  
        16#C1#, 16#80#, 16#F8#, 16#0F#, 16#80#, 16#C1#, 16#86#, 16#00#,  
        16#00#, 16#31#, 16#83#, 16#03#, 16#C0#, 16#01#, 16#E0#, 16#60#,  
        16#C6#, 16#00#, 16#00#, 16#61#, 16#87#, 16#07#, 16#00#, 16#00#,  
        16#70#, 16#70#, 16#C3#, 16#00#, 16#00#, 16#63#, 16#06#, 16#0E#,  
        16#00#, 16#00#, 16#38#, 16#30#, 16#63#, 16#00#, 16#00#, 16#63#,  
        16#0C#, 16#1C#, 16#07#, 16#F0#, 16#1C#, 16#18#, 16#63#, 16#00#,  
        16#00#, 16#C7#, 16#0C#, 16#18#, 16#1F#, 16#FC#, 16#0C#, 16#18#,  
        16#71#, 16#80#, 16#00#, 16#C6#, 16#1C#, 16#30#, 16#38#, 16#0E#,  
        16#06#, 16#1C#, 16#31#, 16#80#, 16#00#, 16#C6#, 16#18#, 16#30#,  
        16#60#, 16#03#, 16#06#, 16#0C#, 16#31#, 16#80#, 16#00#, 16#C6#,  
        16#18#, 16#60#, 16#C1#, 16#11#, 16#83#, 16#0C#, 16#31#, 16#80#,  
        16#01#, 16#8C#, 16#18#, 16#61#, 16#84#, 16#A4#, 16#C3#, 16#0C#,  
        16#18#, 16#C0#, 16#01#, 16#8C#, 16#30#, 16#E1#, 16#92#, 16#F9#,  
        16#C3#, 16#86#, 16#18#, 16#C0#, 16#01#, 16#8C#, 16#30#, 16#C3#,  
        16#0B#, 16#0E#, 16#61#, 16#86#, 16#18#, 16#C0#, 16#01#, 16#8C#,  
        16#30#, 16#C3#, 16#2C#, 16#62#, 16#61#, 16#86#, 16#18#, 16#C0#,  
        16#01#, 16#8C#, 16#30#, 16#C3#, 16#30#, 16#92#, 16#61#, 16#86#,  
        16#18#, 16#C0#, 16#01#, 16#8C#, 16#30#, 16#C3#, 16#71#, 16#69#,  
        16#61#, 16#86#, 16#18#, 16#C0#, 16#01#, 16#8C#, 16#30#, 16#C3#,  
        16#30#, 16#91#, 16#61#, 16#86#, 16#18#, 16#C0#, 16#01#, 16#8C#,  
        16#30#, 16#C3#, 16#0C#, 16#63#, 16#61#, 16#86#, 16#18#, 16#C0#,  
        16#01#, 16#8C#, 16#30#, 16#C3#, 16#03#, 16#07#, 16#61#, 16#86#,  
        16#18#, 16#C0#, 16#01#, 16#8C#, 16#30#, 16#E1#, 16#80#, 16#F8#,  
        16#C3#, 16#86#, 16#18#, 16#C0#, 16#01#, 16#8C#, 16#18#, 16#61#,  
        16#80#, 16#00#, 16#C3#, 16#0C#, 16#18#, 16#C0#, 16#00#, 16#C6#,  
        16#18#, 16#60#, 16#C0#, 16#01#, 16#83#, 16#0C#, 16#31#, 16#80#,  
        16#00#, 16#C6#, 16#18#, 16#30#, 16#60#, 16#03#, 16#06#, 16#0C#,  
        16#31#, 16#80#, 16#00#, 16#C6#, 16#1C#, 16#30#, 16#38#, 16#0E#,  
        16#06#, 16#1C#, 16#31#, 16#80#, 16#00#, 16#C7#, 16#0C#, 16#18#,  
        16#1F#, 16#FC#, 16#0C#, 16#18#, 16#71#, 16#80#, 16#00#, 16#63#,  
        16#0C#, 16#1C#, 16#07#, 16#F0#, 16#1C#, 16#18#, 16#63#, 16#00#,  
        16#00#, 16#63#, 16#06#, 16#0E#, 16#00#, 16#00#, 16#38#, 16#30#,  
        16#63#, 16#00#, 16#00#, 16#61#, 16#87#, 16#07#, 16#00#, 16#00#,  
        16#70#, 16#70#, 16#C3#, 16#00#, 16#00#, 16#31#, 16#83#, 16#03#,  
        16#C0#, 16#01#, 16#E0#, 16#60#, 16#C6#, 16#00#, 16#00#, 16#30#,  
        16#C1#, 16#80#, 16#F8#, 16#0F#, 16#80#, 16#C1#, 16#86#, 16#00#,  
        16#00#, 16#18#, 16#C0#, 16#C0#, 16#3F#, 16#FE#, 16#01#, 16#81#,  
        16#8C#, 16#00#, 16#00#, 16#18#, 16#60#, 16#60#, 16#0F#, 16#F8#,  
        16#03#, 16#03#, 16#0C#, 16#00#, 16#00#, 16#0C#, 16#30#, 16#38#,  
        16#00#, 16#00#, 16#0E#, 16#06#, 16#18#, 16#00#, 16#00#, 16#0C#,  
        16#38#, 16#1C#, 16#00#, 16#00#, 16#1C#, 16#0E#, 16#18#, 16#00#,  
        16#00#, 16#06#, 16#1C#, 16#0F#, 16#80#, 16#00#, 16#F8#, 16#1C#,  
        16#30#, 16#00#, 16#00#, 16#03#, 16#0E#, 16#03#, 16#F0#, 16#07#,  
        16#E0#, 16#38#, 16#60#, 16#00#, 16#00#, 16#03#, 16#87#, 16#00#,  
        16#FF#, 16#FF#, 16#80#, 16#70#, 16#E0#, 16#00#, 16#00#, 16#01#,  
        16#C3#, 16#80#, 16#0F#, 16#F8#, 16#00#, 16#E1#, 16#C0#, 16#00#,  
        16#00#, 16#00#, 16#E0#, 16#E0#, 16#00#, 16#00#, 16#03#, 16#83#,  
        16#80#, 16#00#, 16#00#, 16#00#, 16#70#, 16#78#, 16#00#, 16#00#,  
        16#0F#, 16#07#, 16#00#, 16#00#, 16#00#, 16#00#, 16#38#, 16#1F#,  
        16#00#, 16#00#, 16#7C#, 16#0E#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#1C#, 16#07#, 16#E0#, 16#03#, 16#F0#, 16#1C#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#07#, 16#01#, 16#FF#, 16#FF#, 16#C0#, 16#70#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#C0#, 16#1F#, 16#FC#,  
        16#01#, 16#E0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#F0#,  
        16#00#, 16#00#, 16#07#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#3E#, 16#00#, 16#00#, 16#3E#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#0F#, 16#E0#, 16#03#, 16#F8#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#FF#, 16#FF#,  
        16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#1F#, 16#FC#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

end Tile_Data_Gdragon;  

