with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  

package Tile_Data_Orchid is  
------------------------------------------------------------------------------  
-- Dragon - a version of Mah-Jongg for X Windows  
--  
-- Author: Gary E. Barnes        March 1989  
--  
-- tile_data_orchid - Mah-Jongg tile bitmap data.  
------------------------------------------------------------------------------  
-- 05/30/90 GEB  - Translate to Ada  
------------------------------------------------------------------------------  

-- *****************************************************************************  
-- Tile Sizes  
--  
-- For each size of playing surface (there are 5 sizes) we have a different  
-- size for tiles and for what goes on them.  The tile sizes are in the Tile  
-- column.  
--  
-- Many tiles (eg. Summer) are a single image.  However, some times (eg. Crak's)  
-- are made up of composite images.  These image pieces have sizes as shown  
-- in the Objs column.  
--  
-- Many tiles (eg. Summer) have writing upon them.  The sizes of lettering  
-- on these tiles is given in the Char column.  The lettering on these tiles  
-- is positioned upwards from the bottom of the tile by the number of pixels  
-- indicated in the Up column.  The letters are spaced apart by the number of  
-- pixels indicated in the Apart column.  
--  
--       Tile    Objs    Char    Up      Apart  
--  
--       28x32   7x8     4x6     1       1  
--       40x48   10x12   5x7     1       1  
--       56x64   14x16   7x11    7       1  
--       68x80   17x20   9x13    11      2  
--       80x96   20x24   9x13    16      3  
--  
-- ****************************************************************************  
--
------------------------------------------------------------------------------  
----FLOWERS  
------------------------------------------------------------------------------  

    Orchid_28x32_Width  : constant := 28;  
    Orchid_28x32_Height : constant := 32;  

    Orchid_28x32_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#07#, 16#80#, 16#00#, 16#00#, 16#1A#, 16#C0#, 16#00#,  
        16#00#, 16#17#, 16#40#, 16#00#, 16#00#, 16#1F#, 16#60#, 16#00#,  
        16#0F#, 16#17#, 16#40#, 16#00#, 16#19#, 16#9B#, 16#67#, 16#80#,  
        16#12#, 16#48#, 16#C6#, 16#40#, 16#17#, 16#7D#, 16#8D#, 16#A0#,  
        16#17#, 16#00#, 16#F9#, 16#E0#, 16#17#, 16#3C#, 16#03#, 16#A0#,  
        16#10#, 16#DD#, 16#FC#, 16#20#, 16#1B#, 16#BD#, 16#F3#, 16#C0#,  
        16#0E#, 16#EC#, 16#9C#, 16#00#, 16#01#, 16#DF#, 16#EE#, 16#00#,  
        16#03#, 16#A5#, 16#AE#, 16#00#, 16#03#, 16#AD#, 16#AE#, 16#00#,  
        16#03#, 16#2F#, 16#27#, 16#00#, 16#02#, 16#20#, 16#61#, 16#00#,  
        16#00#, 16#1F#, 16#C0#, 16#00#, 16#00#, 16#02#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#33#, 16#8D#, 16#5D#, 16#C0#,  
        16#4A#, 16#51#, 16#49#, 16#20#, 16#4A#, 16#51#, 16#C9#, 16#20#,  
        16#4B#, 16#91#, 16#49#, 16#20#, 16#4A#, 16#91#, 16#49#, 16#20#,  
        16#32#, 16#4D#, 16#5D#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Orchid_40x48_Width  : constant := 40;  
    Orchid_40x48_Height : constant := 48;  

    Orchid_40x48_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#7F#, 16#00#, 16#00#, 16#00#, 16#01#,  
        16#81#, 16#80#, 16#00#, 16#00#, 16#01#, 16#00#, 16#80#, 16#00#,  
        16#00#, 16#01#, 16#76#, 16#80#, 16#00#, 16#00#, 16#01#, 16#7C#,  
        16#80#, 16#00#, 16#07#, 16#C1#, 16#08#, 16#80#, 16#00#, 16#0C#,  
        16#61#, 16#B2#, 16#80#, 16#00#, 16#08#, 16#10#, 16#9D#, 16#83#,  
        16#E0#, 16#08#, 16#08#, 16#81#, 16#06#, 16#30#, 16#09#, 16#88#,  
        16#42#, 16#04#, 16#08#, 16#0A#, 16#67#, 16#E4#, 16#0C#, 16#C8#,  
        16#12#, 16#E0#, 16#07#, 16#F8#, 16#E8#, 16#1A#, 16#60#, 16#00#,  
        16#00#, 16#A8#, 16#0A#, 16#83#, 16#C0#, 16#01#, 16#E8#, 16#0B#,  
        16#9E#, 16#E7#, 16#F0#, 16#88#, 16#08#, 16#33#, 16#C7#, 16#8C#,  
        16#08#, 16#08#, 16#DF#, 16#45#, 16#F3#, 16#F0#, 16#0F#, 16#BE#,  
        16#C2#, 16#FC#, 16#00#, 16#00#, 16#3D#, 16#83#, 16#7C#, 16#00#,  
        16#00#, 16#7D#, 16#79#, 16#7E#, 16#00#, 16#00#, 16#FA#, 16#6C#,  
        16#BF#, 16#00#, 16#00#, 16#F4#, 16#4E#, 16#5F#, 16#00#, 16#00#,  
        16#F4#, 16#C6#, 16#5F#, 16#00#, 16#01#, 16#C4#, 16#F4#, 16#4F#,  
        16#80#, 16#00#, 16#84#, 16#7C#, 16#47#, 16#80#, 16#00#, 16#06#,  
        16#00#, 16#80#, 16#00#, 16#00#, 16#03#, 16#83#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#7C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#1C#, 16#F0#, 16#E8#, 16#BE#, 16#F0#,  
        16#22#, 16#89#, 16#08#, 16#88#, 16#88#, 16#22#, 16#8A#, 16#08#,  
        16#88#, 16#88#, 16#22#, 16#8A#, 16#0F#, 16#88#, 16#88#, 16#22#,  
        16#F2#, 16#08#, 16#88#, 16#88#, 16#22#, 16#A2#, 16#08#, 16#88#,  
        16#88#, 16#22#, 16#91#, 16#08#, 16#88#, 16#88#, 16#1C#, 16#88#,  
        16#E8#, 16#BE#, 16#F0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Orchid_56x64_Width  : constant := 56;  
    Orchid_56x64_Height : constant := 64;  

    Orchid_56x64_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#7F#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#01#, 16#81#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#,  
        16#76#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#7C#,  
        16#80#, 16#00#, 16#00#, 16#00#, 16#07#, 16#C1#, 16#08#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#0C#, 16#61#, 16#B2#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#08#, 16#10#, 16#9D#, 16#83#, 16#E0#, 16#00#,  
        16#00#, 16#08#, 16#08#, 16#81#, 16#06#, 16#30#, 16#00#, 16#00#,  
        16#09#, 16#88#, 16#42#, 16#04#, 16#08#, 16#00#, 16#00#, 16#0A#,  
        16#67#, 16#E4#, 16#0C#, 16#C8#, 16#00#, 16#00#, 16#12#, 16#E0#,  
        16#07#, 16#F8#, 16#E8#, 16#00#, 16#00#, 16#1A#, 16#60#, 16#00#,  
        16#00#, 16#A8#, 16#00#, 16#00#, 16#0A#, 16#83#, 16#C0#, 16#01#,  
        16#E8#, 16#00#, 16#00#, 16#0B#, 16#9E#, 16#E7#, 16#F0#, 16#88#,  
        16#00#, 16#00#, 16#08#, 16#33#, 16#C7#, 16#8C#, 16#08#, 16#00#,  
        16#00#, 16#08#, 16#DF#, 16#45#, 16#F3#, 16#F0#, 16#00#, 16#00#,  
        16#0F#, 16#BE#, 16#C2#, 16#FC#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#3D#, 16#83#, 16#7C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#7D#,  
        16#79#, 16#7E#, 16#00#, 16#00#, 16#00#, 16#00#, 16#FA#, 16#6C#,  
        16#BF#, 16#00#, 16#00#, 16#00#, 16#00#, 16#F4#, 16#4E#, 16#5F#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#F4#, 16#C6#, 16#5F#, 16#00#,  
        16#00#, 16#00#, 16#01#, 16#C4#, 16#F4#, 16#4F#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#84#, 16#7C#, 16#47#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#06#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#03#, 16#83#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#7C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#03#, 16#87#, 16#83#, 16#E8#, 16#2F#, 16#EF#,  
        16#80#, 16#04#, 16#48#, 16#44#, 16#08#, 16#21#, 16#08#, 16#40#,  
        16#08#, 16#28#, 16#28#, 16#08#, 16#21#, 16#08#, 16#20#, 16#08#,  
        16#28#, 16#28#, 16#08#, 16#21#, 16#08#, 16#20#, 16#08#, 16#28#,  
        16#28#, 16#08#, 16#21#, 16#08#, 16#20#, 16#08#, 16#28#, 16#48#,  
        16#08#, 16#21#, 16#08#, 16#20#, 16#08#, 16#2F#, 16#88#, 16#0F#,  
        16#E1#, 16#08#, 16#20#, 16#08#, 16#28#, 16#88#, 16#08#, 16#21#,  
        16#08#, 16#20#, 16#08#, 16#28#, 16#48#, 16#08#, 16#21#, 16#08#,  
        16#20#, 16#04#, 16#48#, 16#44#, 16#08#, 16#21#, 16#08#, 16#40#,  
        16#03#, 16#88#, 16#23#, 16#E8#, 16#2F#, 16#EF#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Orchid_68x80_Width  : constant := 68;  
    Orchid_68x80_Height : constant := 80;  

    Orchid_68x80_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#FC#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#06#, 16#06#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#04#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#05#, 16#DA#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#05#, 16#F2#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#1F#, 16#04#, 16#22#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#31#, 16#86#, 16#CA#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#20#, 16#42#,  
        16#76#, 16#0F#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#20#,  
        16#22#, 16#04#, 16#18#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#26#, 16#21#, 16#08#, 16#10#, 16#20#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#29#, 16#9F#, 16#90#, 16#33#, 16#20#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#4B#, 16#80#, 16#1F#, 16#E3#, 16#A0#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#69#, 16#80#, 16#00#, 16#02#, 16#A0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#2A#, 16#0F#, 16#00#, 16#07#,  
        16#A0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#2E#, 16#7B#, 16#9F#,  
        16#C2#, 16#20#, 16#00#, 16#00#, 16#00#, 16#00#, 16#20#, 16#CF#,  
        16#1E#, 16#30#, 16#20#, 16#00#, 16#00#, 16#00#, 16#00#, 16#23#,  
        16#7D#, 16#17#, 16#CF#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#3E#, 16#FB#, 16#0B#, 16#F0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#F6#, 16#0D#, 16#F0#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#01#, 16#F5#, 16#E5#, 16#F8#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#03#, 16#E9#, 16#B2#, 16#FC#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#D1#, 16#39#, 16#7C#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#D3#, 16#19#,  
        16#7C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#07#, 16#13#,  
        16#D1#, 16#3E#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#,  
        16#11#, 16#F1#, 16#1E#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#18#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#0E#, 16#0C#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#01#, 16#F0#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#0F#, 16#83#, 16#F0#, 16#3F#, 16#90#, 16#13#, 16#FE#, 16#7F#,  
        16#00#, 16#10#, 16#44#, 16#08#, 16#40#, 16#10#, 16#10#, 16#20#,  
        16#40#, 16#80#, 16#20#, 16#24#, 16#04#, 16#80#, 16#10#, 16#10#,  
        16#20#, 16#40#, 16#40#, 16#20#, 16#24#, 16#04#, 16#80#, 16#10#,  
        16#10#, 16#20#, 16#40#, 16#40#, 16#20#, 16#24#, 16#04#, 16#80#,  
        16#10#, 16#10#, 16#20#, 16#40#, 16#40#, 16#20#, 16#24#, 16#08#,  
        16#80#, 16#10#, 16#10#, 16#20#, 16#40#, 16#40#, 16#20#, 16#27#,  
        16#F0#, 16#80#, 16#1F#, 16#F0#, 16#20#, 16#40#, 16#40#, 16#20#,  
        16#24#, 16#10#, 16#80#, 16#10#, 16#10#, 16#20#, 16#40#, 16#40#,  
        16#20#, 16#24#, 16#10#, 16#80#, 16#10#, 16#10#, 16#20#, 16#40#,  
        16#40#, 16#20#, 16#24#, 16#08#, 16#80#, 16#10#, 16#10#, 16#20#,  
        16#40#, 16#40#, 16#20#, 16#24#, 16#08#, 16#80#, 16#10#, 16#10#,  
        16#20#, 16#40#, 16#40#, 16#10#, 16#44#, 16#04#, 16#40#, 16#10#,  
        16#10#, 16#20#, 16#40#, 16#80#, 16#0F#, 16#84#, 16#04#, 16#3F#,  
        16#90#, 16#13#, 16#FE#, 16#7F#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Orchid_80x96_Width  : constant := 80;  
    Orchid_80x96_Height : constant := 96;  

    Orchid_80x96_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#07#, 16#F0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#18#, 16#18#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#10#, 16#08#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#17#, 16#68#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#17#, 16#C8#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#7C#, 16#10#, 16#88#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#C6#, 16#1B#, 16#28#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#81#,  
        16#09#, 16#58#, 16#3E#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#80#, 16#88#, 16#10#, 16#63#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#98#, 16#85#, 16#A0#, 16#40#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#A6#, 16#7E#, 16#40#,  
        16#CC#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#2E#,  
        16#00#, 16#7F#, 16#8E#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#A6#, 16#C1#, 16#00#, 16#0A#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#A8#, 16#3C#, 16#00#, 16#DE#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#B9#, 16#ED#, 16#7F#,  
        16#08#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#83#,  
        16#3D#, 16#78#, 16#C0#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#8D#, 16#F5#, 16#5F#, 16#3F#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#FB#, 16#EC#, 16#2F#, 16#C0#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#D9#, 16#37#,  
        16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#07#,  
        16#D7#, 16#97#, 16#E0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#0F#, 16#A6#, 16#CB#, 16#F0#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#0F#, 16#44#, 16#E5#, 16#F0#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0F#, 16#4C#, 16#65#,  
        16#F0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#1C#,  
        16#4F#, 16#44#, 16#F8#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#08#, 16#47#, 16#C4#, 16#78#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#60#, 16#08#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#38#, 16#30#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#07#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#7C#,  
        16#1F#, 16#81#, 16#FC#, 16#80#, 16#47#, 16#FC#, 16#FE#, 16#00#,  
        16#00#, 16#82#, 16#20#, 16#42#, 16#00#, 16#80#, 16#40#, 16#40#,  
        16#81#, 16#00#, 16#01#, 16#01#, 16#20#, 16#24#, 16#00#, 16#80#,  
        16#40#, 16#40#, 16#80#, 16#80#, 16#01#, 16#01#, 16#20#, 16#24#,  
        16#00#, 16#80#, 16#40#, 16#40#, 16#80#, 16#80#, 16#01#, 16#01#,  
        16#20#, 16#24#, 16#00#, 16#80#, 16#40#, 16#40#, 16#80#, 16#80#,  
        16#01#, 16#01#, 16#20#, 16#44#, 16#00#, 16#80#, 16#40#, 16#40#,  
        16#80#, 16#80#, 16#01#, 16#01#, 16#3F#, 16#84#, 16#00#, 16#FF#,  
        16#C0#, 16#40#, 16#80#, 16#80#, 16#01#, 16#01#, 16#20#, 16#84#,  
        16#00#, 16#80#, 16#40#, 16#40#, 16#80#, 16#80#, 16#01#, 16#01#,  
        16#20#, 16#84#, 16#00#, 16#80#, 16#40#, 16#40#, 16#80#, 16#80#,  
        16#01#, 16#01#, 16#20#, 16#44#, 16#00#, 16#80#, 16#40#, 16#40#,  
        16#80#, 16#80#, 16#01#, 16#01#, 16#20#, 16#44#, 16#00#, 16#80#,  
        16#40#, 16#40#, 16#80#, 16#80#, 16#00#, 16#82#, 16#20#, 16#22#,  
        16#00#, 16#80#, 16#40#, 16#40#, 16#81#, 16#00#, 16#00#, 16#7C#,  
        16#20#, 16#21#, 16#FC#, 16#80#, 16#47#, 16#FC#, 16#FE#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

end Tile_Data_Orchid;  

