with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  

package Tile_Data_Rdragon is  
------------------------------------------------------------------------------  
-- Dragon - a version of Mah-Jongg for X Windows  
--  
-- Author: Gary E. Barnes        March 1989  
--  
-- tile_data_rdragon - Mah-Jongg tile bitmap data.  
------------------------------------------------------------------------------  
-- 05/30/90 GEB  - Translate to Ada  
------------------------------------------------------------------------------  

-- *****************************************************************************  
-- Tile Sizes  
--  
-- For each size of playing surface (there are 5 sizes) we have a different  
-- size for tiles and for what goes on them.  The tile sizes are in the Tile  
-- column.  
--  
-- Many tiles (eg. Summer) are a single image.  However, some times (eg. Crak's)  
-- are made up of composite images.  These image pieces have sizes as shown  
-- in the Objs column.  
--  
-- Many tiles (eg. Summer) have writing upon them.  The sizes of lettering  
-- on these tiles is given in the Char column.  The lettering on these tiles  
-- is positioned upwards from the bottom of the tile by the number of pixels  
-- indicated in the Up column.  The letters are spaced apart by the number of  
-- pixels indicated in the Apart column.  
--  
--       Tile    Objs    Char    Up      Apart  
--  
--       28x32   7x8     4x6     1       1  
--       40x48   10x12   5x7     1       1  
--       56x64   14x16   7x11    7       1  
--       68x80   17x20   9x13    11      2  
--       80x96   20x24   9x13    16      3  
--  
-- ****************************************************************************  
--
------------------------------------------------------------------------------  
----DRAGONS  
------------------------------------------------------------------------------  

    Rdragon_28x32_Width  : constant := 28;  
    Rdragon_28x32_Height : constant := 32;  

    Rdragon_28x32_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#21#, 16#F1#, 16#00#,  
        16#00#, 16#67#, 16#FD#, 16#40#, 16#00#, 16#3E#, 16#1F#, 16#80#,  
        16#00#, 16#1C#, 16#07#, 16#80#, 16#00#, 16#3F#, 16#82#, 16#80#,  
        16#00#, 16#39#, 16#03#, 16#E0#, 16#00#, 16#70#, 16#01#, 16#60#,  
        16#03#, 16#FE#, 16#00#, 16#A0#, 16#01#, 16#74#, 16#00#, 16#40#,  
        16#00#, 16#70#, 16#00#, 16#00#, 16#00#, 16#74#, 16#00#, 16#00#,  
        16#00#, 16#7E#, 16#07#, 16#80#, 16#00#, 16#38#, 16#04#, 16#40#,  
        16#00#, 16#3E#, 16#04#, 16#40#, 16#00#, 16#3F#, 16#07#, 16#80#,  
        16#06#, 16#63#, 16#84#, 16#40#, 16#08#, 16#20#, 16#C4#, 16#40#,  
        16#10#, 16#00#, 16#E0#, 16#00#, 16#30#, 16#00#, 16#60#, 16#00#,  
        16#60#, 16#00#, 16#30#, 16#00#, 16#60#, 16#00#, 16#30#, 16#00#,  
        16#60#, 16#00#, 16#30#, 16#00#, 16#60#, 16#00#, 16#70#, 16#00#,  
        16#60#, 16#00#, 16#60#, 16#00#, 16#30#, 16#00#, 16#E0#, 16#00#,  
        16#38#, 16#01#, 16#C0#, 16#00#, 16#1E#, 16#07#, 16#80#, 16#00#,  
        16#0F#, 16#9F#, 16#00#, 16#00#, 16#03#, 16#FC#, 16#00#, 16#00#,  
        16#00#, 16#F0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Rdragon_40x48_Width  : constant := 40;  
    Rdragon_40x48_Height : constant := 48;  

    Rdragon_40x48_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#87#,  
        16#F8#, 16#10#, 16#00#, 16#01#, 16#3F#, 16#FF#, 16#34#, 16#00#,  
        16#03#, 16#FC#, 16#07#, 16#F8#, 16#00#, 16#00#, 16#F0#, 16#00#,  
        16#D0#, 16#00#, 16#01#, 16#FF#, 16#00#, 16#F0#, 16#00#, 16#03#,  
        16#C4#, 16#00#, 16#D0#, 16#00#, 16#03#, 16#CA#, 16#00#, 16#6C#,  
        16#00#, 16#07#, 16#80#, 16#00#, 16#16#, 16#00#, 16#07#, 16#80#,  
        16#00#, 16#0C#, 16#00#, 16#0F#, 16#C0#, 16#00#, 16#00#, 16#00#,  
        16#5F#, 16#F4#, 16#00#, 16#00#, 16#00#, 16#3F#, 16#18#, 16#00#,  
        16#00#, 16#00#, 16#47#, 16#24#, 16#00#, 16#00#, 16#00#, 16#27#,  
        16#00#, 16#03#, 16#C0#, 16#00#, 16#07#, 16#38#, 16#02#, 16#20#,  
        16#00#, 16#07#, 16#30#, 16#02#, 16#20#, 16#00#, 16#07#, 16#A8#,  
        16#03#, 16#C0#, 16#00#, 16#03#, 16#E0#, 16#02#, 16#40#, 16#00#,  
        16#03#, 16#F0#, 16#02#, 16#20#, 16#00#, 16#03#, 16#F8#, 16#02#,  
        16#20#, 16#00#, 16#01#, 16#FC#, 16#00#, 16#00#, 16#00#, 16#C1#,  
        16#3E#, 16#00#, 16#00#, 16#03#, 16#07#, 16#0F#, 16#00#, 16#00#,  
        16#06#, 16#02#, 16#83#, 16#80#, 16#00#, 16#0C#, 16#00#, 16#01#,  
        16#C0#, 16#00#, 16#18#, 16#00#, 16#00#, 16#E0#, 16#00#, 16#30#,  
        16#00#, 16#00#, 16#60#, 16#00#, 16#30#, 16#00#, 16#00#, 16#70#,  
        16#00#, 16#30#, 16#00#, 16#00#, 16#38#, 16#00#, 16#30#, 16#00#,  
        16#00#, 16#18#, 16#00#, 16#60#, 16#00#, 16#00#, 16#18#, 16#00#,  
        16#60#, 16#00#, 16#00#, 16#18#, 16#00#, 16#60#, 16#00#, 16#00#,  
        16#18#, 16#00#, 16#30#, 16#00#, 16#00#, 16#18#, 16#00#, 16#30#,  
        16#00#, 16#00#, 16#18#, 16#00#, 16#30#, 16#00#, 16#00#, 16#38#,  
        16#00#, 16#30#, 16#00#, 16#00#, 16#70#, 16#00#, 16#18#, 16#00#,  
        16#00#, 16#60#, 16#00#, 16#18#, 16#00#, 16#00#, 16#E0#, 16#00#,  
        16#0C#, 16#00#, 16#01#, 16#C0#, 16#00#, 16#0E#, 16#00#, 16#03#,  
        16#80#, 16#00#, 16#07#, 16#00#, 16#07#, 16#00#, 16#00#, 16#03#,  
        16#C0#, 16#1E#, 16#00#, 16#00#, 16#00#, 16#FC#, 16#38#, 16#00#,  
        16#00#, 16#00#, 16#3F#, 16#F0#, 16#00#, 16#00#, 16#00#, 16#07#,  
        16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Rdragon_56x64_Width  : constant := 56;  
    Rdragon_56x64_Height : constant := 64;  

    Rdragon_56x64_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#04#, 16#07#, 16#FF#, 16#C0#, 16#40#, 16#00#, 16#00#,  
        16#03#, 16#3F#, 16#FF#, 16#F8#, 16#80#, 16#00#, 16#00#, 16#07#,  
        16#FC#, 16#00#, 16#7F#, 16#20#, 16#00#, 16#00#, 16#00#, 16#F0#,  
        16#00#, 16#07#, 16#C0#, 16#00#, 16#00#, 16#03#, 16#F0#, 16#00#,  
        16#07#, 16#80#, 16#00#, 16#00#, 16#07#, 16#DF#, 16#00#, 16#06#,  
        16#C0#, 16#00#, 16#00#, 16#0F#, 16#8E#, 16#80#, 16#07#, 16#E0#,  
        16#00#, 16#00#, 16#0F#, 16#85#, 16#00#, 16#01#, 16#B0#, 16#00#,  
        16#00#, 16#0F#, 16#04#, 16#00#, 16#00#, 16#DC#, 16#00#, 16#00#,  
        16#1E#, 16#00#, 16#00#, 16#00#, 16#2C#, 16#00#, 16#00#, 16#3E#,  
        16#00#, 16#00#, 16#00#, 16#10#, 16#00#, 16#00#, 16#3E#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#05#, 16#FF#, 16#E0#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#03#, 16#7C#, 16#50#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#03#, 16#3C#, 16#E0#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#04#, 16#BC#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#3C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#3C#, 16#00#, 16#00#, 16#1F#, 16#00#, 16#00#, 16#00#, 16#3C#,  
        16#00#, 16#00#, 16#10#, 16#80#, 16#00#, 16#00#, 16#3C#, 16#28#,  
        16#00#, 16#10#, 16#40#, 16#00#, 16#00#, 16#3C#, 16#10#, 16#00#,  
        16#10#, 16#40#, 16#00#, 16#00#, 16#1C#, 16#30#, 16#00#, 16#10#,  
        16#80#, 16#00#, 16#00#, 16#1E#, 16#68#, 16#00#, 16#1F#, 16#00#,  
        16#00#, 16#00#, 16#0F#, 16#60#, 16#00#, 16#11#, 16#00#, 16#00#,  
        16#00#, 16#0F#, 16#E0#, 16#00#, 16#10#, 16#80#, 16#00#, 16#00#,  
        16#0F#, 16#F0#, 16#00#, 16#10#, 16#80#, 16#00#, 16#00#, 16#0F#,  
        16#FC#, 16#00#, 16#10#, 16#40#, 16#00#, 16#07#, 16#05#, 16#FE#,  
        16#00#, 16#10#, 16#40#, 16#00#, 16#18#, 16#06#, 16#FF#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#70#, 16#0C#, 16#1F#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#C0#, 16#0E#, 16#07#, 16#C0#, 16#00#, 16#00#,  
        16#01#, 16#00#, 16#08#, 16#01#, 16#E0#, 16#00#, 16#00#, 16#03#,  
        16#00#, 16#00#, 16#00#, 16#F0#, 16#00#, 16#00#, 16#0E#, 16#00#,  
        16#00#, 16#00#, 16#70#, 16#00#, 16#00#, 16#08#, 16#00#, 16#00#,  
        16#00#, 16#78#, 16#00#, 16#00#, 16#18#, 16#00#, 16#00#, 16#00#,  
        16#3C#, 16#00#, 16#00#, 16#10#, 16#00#, 16#00#, 16#00#, 16#1C#,  
        16#00#, 16#00#, 16#30#, 16#00#, 16#00#, 16#00#, 16#1C#, 16#00#,  
        16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#1E#, 16#00#, 16#00#,  
        16#60#, 16#00#, 16#00#, 16#00#, 16#0E#, 16#00#, 16#00#, 16#60#,  
        16#00#, 16#00#, 16#00#, 16#0E#, 16#00#, 16#00#, 16#60#, 16#00#,  
        16#00#, 16#00#, 16#0F#, 16#00#, 16#00#, 16#60#, 16#00#, 16#00#,  
        16#00#, 16#07#, 16#00#, 16#00#, 16#60#, 16#00#, 16#00#, 16#00#,  
        16#07#, 16#00#, 16#00#, 16#60#, 16#00#, 16#00#, 16#00#, 16#07#,  
        16#00#, 16#00#, 16#60#, 16#00#, 16#00#, 16#00#, 16#07#, 16#00#,  
        16#00#, 16#60#, 16#00#, 16#00#, 16#00#, 16#0E#, 16#00#, 16#00#,  
        16#60#, 16#00#, 16#00#, 16#00#, 16#0E#, 16#00#, 16#00#, 16#60#,  
        16#00#, 16#00#, 16#00#, 16#1E#, 16#00#, 16#00#, 16#70#, 16#00#,  
        16#00#, 16#00#, 16#1C#, 16#00#, 16#00#, 16#30#, 16#00#, 16#00#,  
        16#00#, 16#3C#, 16#00#, 16#00#, 16#18#, 16#00#, 16#00#, 16#00#,  
        16#30#, 16#00#, 16#00#, 16#18#, 16#00#, 16#00#, 16#00#, 16#30#,  
        16#00#, 16#00#, 16#0C#, 16#00#, 16#00#, 16#00#, 16#60#, 16#00#,  
        16#00#, 16#0E#, 16#00#, 16#00#, 16#00#, 16#E0#, 16#00#, 16#00#,  
        16#06#, 16#00#, 16#00#, 16#01#, 16#C0#, 16#00#, 16#00#, 16#03#,  
        16#80#, 16#00#, 16#03#, 16#00#, 16#00#, 16#00#, 16#01#, 16#C0#,  
        16#00#, 16#07#, 16#00#, 16#00#, 16#00#, 16#00#, 16#F0#, 16#00#,  
        16#1E#, 16#00#, 16#00#, 16#00#, 16#00#, 16#3E#, 16#00#, 16#38#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#0F#, 16#FF#, 16#E0#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#01#, 16#FF#, 16#C0#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Rdragon_68x80_Width  : constant := 68;  
    Rdragon_68x80_Height : constant := 80;  

    Rdragon_68x80_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#07#, 16#FF#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#20#, 16#3F#, 16#FF#,  
        16#F0#, 16#20#, 16#00#, 16#00#, 16#00#, 16#00#, 16#1E#, 16#FC#,  
        16#00#, 16#FE#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#33#,  
        16#E0#, 16#00#, 16#0F#, 16#C8#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#13#, 16#C0#, 16#00#, 16#03#, 16#F0#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#27#, 16#C0#, 16#00#, 16#01#, 16#E0#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#1F#, 16#60#, 16#00#, 16#01#, 16#B0#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#3C#, 16#3E#, 16#00#, 16#01#, 16#F8#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#78#, 16#19#, 16#00#, 16#01#,  
        16#FC#, 16#00#, 16#00#, 16#00#, 16#00#, 16#7C#, 16#14#, 16#00#,  
        16#00#, 16#7E#, 16#00#, 16#00#, 16#00#, 16#00#, 16#F8#, 16#10#,  
        16#00#, 16#00#, 16#3B#, 16#80#, 16#00#, 16#00#, 16#00#, 16#F0#,  
        16#00#, 16#00#, 16#00#, 16#0D#, 16#80#, 16#00#, 16#00#, 16#01#,  
        16#F0#, 16#00#, 16#00#, 16#00#, 16#06#, 16#00#, 16#00#, 16#00#,  
        16#03#, 16#E0#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#00#,  
        16#00#, 16#03#, 16#E0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#07#, 16#FE#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#1F#, 16#F3#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#A7#, 16#C2#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#63#, 16#C5#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#93#, 16#C0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#,  
        16#E0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#03#, 16#C0#, 16#00#, 16#00#, 16#07#, 16#F0#, 16#00#, 16#00#,  
        16#00#, 16#03#, 16#C0#, 16#00#, 16#00#, 16#04#, 16#08#, 16#00#,  
        16#00#, 16#00#, 16#03#, 16#C0#, 16#00#, 16#00#, 16#04#, 16#04#,  
        16#00#, 16#00#, 16#00#, 16#03#, 16#C0#, 16#00#, 16#00#, 16#04#,  
        16#04#, 16#00#, 16#00#, 16#00#, 16#03#, 16#E0#, 16#00#, 16#00#,  
        16#04#, 16#04#, 16#00#, 16#00#, 16#00#, 16#01#, 16#E0#, 16#40#,  
        16#00#, 16#04#, 16#08#, 16#00#, 16#00#, 16#00#, 16#01#, 16#E0#,  
        16#80#, 16#00#, 16#07#, 16#F0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#F1#, 16#E0#, 16#00#, 16#04#, 16#10#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#FB#, 16#40#, 16#00#, 16#04#, 16#10#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#7F#, 16#40#, 16#00#, 16#04#, 16#08#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#7F#, 16#80#, 16#00#, 16#04#, 16#08#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#7F#, 16#C0#, 16#00#, 16#04#,  
        16#04#, 16#00#, 16#00#, 16#00#, 16#00#, 16#7F#, 16#F0#, 16#00#,  
        16#04#, 16#04#, 16#00#, 16#00#, 16#00#, 16#E0#, 16#6F#, 16#FC#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#00#, 16#47#,  
        16#FE#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#1C#, 16#00#,  
        16#50#, 16#7F#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#70#,  
        16#00#, 16#60#, 16#3F#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#60#, 16#00#, 16#A0#, 16#0F#, 16#C0#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#C0#, 16#00#, 16#A0#, 16#03#, 16#C0#, 16#00#, 16#00#,  
        16#00#, 16#01#, 16#80#, 16#00#, 16#00#, 16#01#, 16#E0#, 16#00#,  
        16#00#, 16#00#, 16#03#, 16#00#, 16#00#, 16#00#, 16#00#, 16#F0#,  
        16#00#, 16#00#, 16#00#, 16#06#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#F0#, 16#00#, 16#00#, 16#00#, 16#0C#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#78#, 16#00#, 16#00#, 16#00#, 16#08#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#3C#, 16#00#, 16#00#, 16#00#, 16#18#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#3C#, 16#00#, 16#00#, 16#00#, 16#30#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#3E#, 16#00#, 16#00#, 16#00#,  
        16#30#, 16#00#, 16#00#, 16#00#, 16#00#, 16#3E#, 16#00#, 16#00#,  
        16#00#, 16#30#, 16#00#, 16#00#, 16#00#, 16#00#, 16#1E#, 16#00#,  
        16#00#, 16#00#, 16#30#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0F#,  
        16#00#, 16#00#, 16#00#, 16#30#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#0F#, 16#00#, 16#00#, 16#00#, 16#60#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#0F#, 16#00#, 16#00#, 16#00#, 16#60#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#0F#, 16#00#, 16#00#, 16#00#, 16#60#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#0F#, 16#00#, 16#00#, 16#00#, 16#60#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#0F#, 16#00#, 16#00#, 16#00#,  
        16#60#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0F#, 16#00#, 16#00#,  
        16#00#, 16#60#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0F#, 16#00#,  
        16#00#, 16#00#, 16#60#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0F#,  
        16#00#, 16#00#, 16#00#, 16#60#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#0E#, 16#00#, 16#00#, 16#00#, 16#60#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#0E#, 16#00#, 16#00#, 16#00#, 16#30#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#0E#, 16#00#, 16#00#, 16#00#, 16#30#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#1C#, 16#00#, 16#00#, 16#00#, 16#30#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#1C#, 16#00#, 16#00#, 16#00#,  
        16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#3C#, 16#00#, 16#00#,  
        16#00#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#38#, 16#00#,  
        16#00#, 16#00#, 16#0C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#70#,  
        16#00#, 16#00#, 16#00#, 16#0C#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#E0#, 16#00#, 16#00#, 16#00#, 16#06#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#06#, 16#00#, 16#00#,  
        16#00#, 16#01#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#03#, 16#00#,  
        16#00#, 16#00#, 16#03#, 16#80#, 16#00#, 16#00#, 16#00#, 16#01#,  
        16#80#, 16#00#, 16#00#, 16#06#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#E0#, 16#00#, 16#00#, 16#0C#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#70#, 16#00#, 16#00#, 16#18#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#1C#, 16#00#, 16#00#, 16#70#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#0F#, 16#80#, 16#01#, 16#E0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#FE#, 16#1F#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#3F#,  
        16#FE#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#E0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Rdragon_80x96_Width  : constant := 80;  
    Rdragon_80x96_Height : constant := 96;  

    Rdragon_80x96_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#,  
        16#FF#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#1F#, 16#FF#, 16#F0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#03#, 16#00#, 16#FF#, 16#FF#, 16#FE#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#FB#, 16#F8#, 16#00#, 16#7F#,  
        16#81#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#9F#, 16#80#,  
        16#00#, 16#07#, 16#E2#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#9F#, 16#00#, 16#00#, 16#01#, 16#FC#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#03#, 16#3F#, 16#00#, 16#00#, 16#00#, 16#79#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#7F#, 16#80#, 16#00#, 16#00#,  
        16#7E#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#FC#, 16#C0#,  
        16#00#, 16#00#, 16#76#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#,  
        16#F0#, 16#7C#, 16#00#, 16#00#, 16#77#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#03#, 16#E0#, 16#72#, 16#00#, 16#00#, 16#7F#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#07#, 16#E0#, 16#48#, 16#00#, 16#00#,  
        16#1F#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#0F#, 16#C0#, 16#48#,  
        16#00#, 16#00#, 16#0E#, 16#F0#, 16#00#, 16#00#, 16#00#, 16#0F#,  
        16#80#, 16#00#, 16#00#, 16#00#, 16#07#, 16#70#, 16#00#, 16#00#,  
        16#00#, 16#1F#, 16#80#, 16#00#, 16#00#, 16#00#, 16#01#, 16#B0#,  
        16#00#, 16#00#, 16#00#, 16#1F#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#50#, 16#00#, 16#00#, 16#00#, 16#3F#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#3F#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#3F#, 16#F0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#01#, 16#FF#, 16#D0#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#FF#, 16#8E#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#14#, 16#7E#,  
        16#09#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#0C#, 16#3E#, 16#14#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#12#, 16#3E#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#3E#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#3E#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#3E#, 16#00#, 16#00#, 16#00#, 16#03#, 16#F8#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#3E#, 16#00#, 16#00#, 16#00#, 16#02#,  
        16#04#, 16#00#, 16#00#, 16#00#, 16#00#, 16#3E#, 16#00#, 16#00#,  
        16#00#, 16#02#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#3E#,  
        16#00#, 16#00#, 16#00#, 16#02#, 16#02#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#3E#, 16#00#, 16#00#, 16#00#, 16#02#, 16#02#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#1F#, 16#00#, 16#00#, 16#00#, 16#02#,  
        16#04#, 16#00#, 16#00#, 16#00#, 16#00#, 16#1F#, 16#01#, 16#00#,  
        16#00#, 16#03#, 16#F8#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0F#,  
        16#02#, 16#00#, 16#00#, 16#02#, 16#08#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#0F#, 16#8F#, 16#80#, 16#00#, 16#02#, 16#08#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#0F#, 16#DA#, 16#00#, 16#00#, 16#02#,  
        16#04#, 16#00#, 16#00#, 16#00#, 16#00#, 16#07#, 16#F9#, 16#00#,  
        16#00#, 16#02#, 16#04#, 16#00#, 16#00#, 16#00#, 16#00#, 16#07#,  
        16#F8#, 16#00#, 16#00#, 16#02#, 16#02#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#03#, 16#FE#, 16#00#, 16#00#, 16#02#, 16#02#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#03#, 16#FF#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#07#, 16#FF#, 16#E0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#1C#, 16#06#,  
        16#7F#, 16#F0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#E0#, 16#04#, 16#3F#, 16#F8#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#03#, 16#00#, 16#06#, 16#8F#, 16#FC#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#04#, 16#00#, 16#03#, 16#01#, 16#FE#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#18#, 16#00#, 16#04#,  
        16#80#, 16#7F#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#30#,  
        16#00#, 16#00#, 16#00#, 16#3F#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#70#, 16#00#, 16#00#, 16#00#, 16#0F#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#E0#, 16#00#, 16#00#, 16#00#, 16#07#,  
        16#C0#, 16#00#, 16#00#, 16#00#, 16#01#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#03#, 16#E0#, 16#00#, 16#00#, 16#00#, 16#03#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#01#, 16#F0#, 16#00#, 16#00#, 16#00#,  
        16#03#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#F0#, 16#00#,  
        16#00#, 16#00#, 16#06#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#F8#, 16#00#, 16#00#, 16#00#, 16#0C#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#7C#, 16#00#, 16#00#, 16#00#, 16#0C#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#7E#, 16#00#, 16#00#, 16#00#,  
        16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#3E#, 16#00#,  
        16#00#, 16#00#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#3F#, 16#00#, 16#00#, 16#00#, 16#18#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#3F#, 16#00#, 16#00#, 16#00#, 16#30#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#1F#, 16#00#, 16#00#, 16#00#,  
        16#30#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#1F#, 16#00#,  
        16#00#, 16#00#, 16#30#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#1F#, 16#00#, 16#00#, 16#00#, 16#60#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#0F#, 16#80#, 16#00#, 16#00#, 16#60#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#0F#, 16#80#, 16#00#, 16#00#,  
        16#60#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0F#, 16#80#,  
        16#00#, 16#00#, 16#60#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#0F#, 16#80#, 16#00#, 16#00#, 16#60#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#0F#, 16#80#, 16#00#, 16#00#, 16#60#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#0F#, 16#80#, 16#00#, 16#00#,  
        16#60#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0F#, 16#80#,  
        16#00#, 16#00#, 16#60#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#0F#, 16#00#, 16#00#, 16#00#, 16#60#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#0F#, 16#00#, 16#00#, 16#00#, 16#60#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#0F#, 16#00#, 16#00#, 16#00#,  
        16#60#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0F#, 16#00#,  
        16#00#, 16#00#, 16#30#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#1E#, 16#00#, 16#00#, 16#00#, 16#30#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#1E#, 16#00#, 16#00#, 16#00#, 16#30#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#1E#, 16#00#, 16#00#, 16#00#,  
        16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#3C#, 16#00#,  
        16#00#, 16#00#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#3C#, 16#00#, 16#00#, 16#00#, 16#18#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#38#, 16#00#, 16#00#, 16#00#, 16#0C#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#78#, 16#00#, 16#00#, 16#00#,  
        16#0C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#70#, 16#00#,  
        16#00#, 16#00#, 16#06#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#E0#, 16#00#, 16#00#, 16#00#, 16#03#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#01#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#03#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#01#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#80#, 16#00#, 16#00#, 16#00#, 16#03#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#06#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#60#, 16#00#, 16#00#,  
        16#00#, 16#0C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#38#,  
        16#00#, 16#00#, 16#00#, 16#38#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#1C#, 16#00#, 16#00#, 16#00#, 16#70#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#07#, 16#00#, 16#00#, 16#01#, 16#C0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#E0#, 16#00#,  
        16#0F#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#FC#, 16#00#, 16#7E#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#1F#, 16#FF#, 16#F0#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#FF#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

end Tile_Data_Rdragon;  

