with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  

package Tile_Data_Spring is  
------------------------------------------------------------------------------  
-- Dragon - a version of Mah-Jongg for X Windows  
--  
-- Author: Gary E. Barnes        March 1989  
--  
-- tile_data_spring - Mah-Jongg tile bitmap data.  
------------------------------------------------------------------------------  
-- 05/30/90 GEB  - Translate to Ada  
------------------------------------------------------------------------------  

-- *****************************************************************************  
-- Tile Sizes  
--  
-- For each size of playing surface (there are 5 sizes) we have a different  
-- size for tiles and for what goes on them.  The tile sizes are in the Tile  
-- column.  
--  
-- Many tiles (eg. Summer) are a single image.  However, some times (eg. Crak's)  
-- are made up of composite images.  These image pieces have sizes as shown  
-- in the Objs column.  
--  
-- Many tiles (eg. Summer) have writing upon them.  The sizes of lettering  
-- on these tiles is given in the Char column.  The lettering on these tiles  
-- is positioned upwards from the bottom of the tile by the number of pixels  
-- indicated in the Up column.  The letters are spaced apart by the number of  
-- pixels indicated in the Apart column.  
--  
--       Tile    Objs    Char    Up      Apart  
--  
--       28x32   7x8     4x6     1       1  
--       40x48   10x12   5x7     1       1  
--       56x64   14x16   7x11    7       1  
--       68x80   17x20   9x13    11      2  
--       80x96   20x24   9x13    16      3  
--  
-- ****************************************************************************  
--
------------------------------------------------------------------------------  
----SEASONS  
------------------------------------------------------------------------------  

    Spring_28x32_Width  : constant := 28;  
    Spring_28x32_Height : constant := 32;  

    Spring_28x32_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#E0#, 16#00#, 16#00#, 16#03#, 16#18#, 16#00#,  
        16#00#, 16#04#, 16#EC#, 16#00#, 16#00#, 16#0B#, 16#12#, 16#00#,  
        16#00#, 16#0A#, 16#EA#, 16#00#, 16#00#, 16#0E#, 16#EE#, 16#00#,  
        16#00#, 16#0A#, 16#EA#, 16#00#, 16#00#, 16#09#, 16#1A#, 16#00#,  
        16#00#, 16#16#, 16#F2#, 16#00#, 16#00#, 16#12#, 16#0C#, 16#00#,  
        16#00#, 16#21#, 16#F0#, 16#00#, 16#00#, 16#20#, 16#00#, 16#00#,  
        16#00#, 16#20#, 16#C0#, 16#00#, 16#00#, 16#21#, 16#20#, 16#00#,  
        16#00#, 16#11#, 16#A0#, 16#00#, 16#07#, 16#D1#, 16#A0#, 16#00#,  
        16#04#, 16#29#, 16#60#, 16#00#, 16#07#, 16#9F#, 16#20#, 16#00#,  
        16#02#, 16#66#, 16#C0#, 16#00#, 16#01#, 16#B3#, 16#C0#, 16#00#,  
        16#00#, 16#6F#, 16#80#, 16#00#, 16#00#, 16#1E#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#3B#, 16#33#, 16#A4#, 16#C0#,  
        16#42#, 16#A9#, 16#35#, 16#20#, 16#32#, 16#A9#, 16#35#, 16#00#,  
        16#0B#, 16#31#, 16#2D#, 16#60#, 16#0A#, 16#31#, 16#2D#, 16#20#,  
        16#72#, 16#2B#, 16#A4#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Spring_40x48_Width  : constant := 40;  
    Spring_40x48_Height : constant := 48;  

    Spring_40x48_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#F0#,  
        16#00#, 16#00#, 16#00#, 16#06#, 16#4C#, 16#00#, 16#00#, 16#00#,  
        16#0A#, 16#4A#, 16#00#, 16#00#, 16#00#, 16#11#, 16#F1#, 16#00#,  
        16#00#, 16#00#, 16#1A#, 16#0B#, 16#00#, 16#00#, 16#00#, 16#24#,  
        16#44#, 16#80#, 16#00#, 16#00#, 16#24#, 16#64#, 16#80#, 16#00#,  
        16#00#, 16#3D#, 16#F7#, 16#80#, 16#00#, 16#00#, 16#24#, 16#C4#,  
        16#80#, 16#00#, 16#00#, 16#64#, 16#44#, 16#80#, 16#00#, 16#00#,  
        16#DA#, 16#0B#, 16#00#, 16#00#, 16#00#, 16#F1#, 16#F1#, 16#00#,  
        16#00#, 16#01#, 16#8A#, 16#4A#, 16#00#, 16#00#, 16#01#, 16#86#,  
        16#4C#, 16#00#, 16#00#, 16#03#, 16#01#, 16#F0#, 16#00#, 16#00#,  
        16#03#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#03#, 16#01#, 16#C0#, 16#00#, 16#00#, 16#03#,  
        16#03#, 16#40#, 16#00#, 16#00#, 16#01#, 16#82#, 16#60#, 16#00#,  
        16#00#, 16#01#, 16#84#, 16#A0#, 16#00#, 16#00#, 16#E1#, 16#84#,  
        16#A0#, 16#00#, 16#01#, 16#BE#, 16#C5#, 16#A0#, 16#00#, 16#01#,  
        16#03#, 16#65#, 16#20#, 16#00#, 16#00#, 16#81#, 16#E5#, 16#20#,  
        16#00#, 16#00#, 16#98#, 16#F4#, 16#60#, 16#00#, 16#00#, 16#44#,  
        16#33#, 16#40#, 16#00#, 16#00#, 16#23#, 16#3D#, 16#C0#, 16#00#,  
        16#00#, 16#10#, 16#8F#, 16#80#, 16#00#, 16#00#, 16#0C#, 16#67#,  
        16#00#, 16#00#, 16#00#, 16#02#, 16#0E#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#F8#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#1E#, 16#F3#, 16#CF#, 16#A2#, 16#70#, 16#20#, 16#8A#, 16#22#,  
        16#32#, 16#88#, 16#20#, 16#8A#, 16#22#, 16#32#, 16#80#, 16#1C#,  
        16#F3#, 16#C2#, 16#2A#, 16#80#, 16#02#, 16#82#, 16#42#, 16#2A#,  
        16#98#, 16#02#, 16#82#, 16#22#, 16#26#, 16#88#, 16#3C#, 16#82#,  
        16#2F#, 16#A6#, 16#70#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Spring_56x64_Width  : constant := 56;  
    Spring_56x64_Height : constant := 64;  

    Spring_56x64_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#F0#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#06#, 16#4C#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#0A#, 16#4A#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#11#, 16#F1#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#1A#, 16#0B#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#24#,  
        16#44#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#24#, 16#64#,  
        16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#3D#, 16#F7#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#24#, 16#C4#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#64#, 16#44#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#DA#, 16#0B#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#F1#, 16#F1#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#,  
        16#8A#, 16#4A#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#86#,  
        16#4C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#01#, 16#F0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#03#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#03#, 16#01#, 16#C0#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#03#, 16#03#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#82#, 16#60#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#,  
        16#84#, 16#A0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#E1#, 16#84#,  
        16#A0#, 16#00#, 16#00#, 16#00#, 16#01#, 16#BE#, 16#C5#, 16#A0#,  
        16#00#, 16#00#, 16#00#, 16#01#, 16#03#, 16#65#, 16#20#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#81#, 16#E5#, 16#20#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#98#, 16#F4#, 16#60#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#44#, 16#33#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#23#, 16#3D#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#10#,  
        16#8F#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0C#, 16#67#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#0E#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#F8#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#01#, 16#F7#, 16#C7#, 16#C7#, 16#F6#, 16#11#,  
        16#C0#, 16#02#, 16#04#, 16#24#, 16#20#, 16#86#, 16#12#, 16#20#,  
        16#04#, 16#04#, 16#14#, 16#10#, 16#85#, 16#14#, 16#10#, 16#04#,  
        16#04#, 16#14#, 16#10#, 16#85#, 16#14#, 16#00#, 16#02#, 16#04#,  
        16#24#, 16#20#, 16#85#, 16#14#, 16#00#, 16#01#, 16#C7#, 16#C7#,  
        16#C0#, 16#84#, 16#94#, 16#00#, 16#00#, 16#24#, 16#04#, 16#40#,  
        16#84#, 16#94#, 16#70#, 16#00#, 16#14#, 16#04#, 16#40#, 16#84#,  
        16#54#, 16#10#, 16#00#, 16#14#, 16#04#, 16#20#, 16#84#, 16#54#,  
        16#10#, 16#00#, 16#24#, 16#04#, 16#20#, 16#84#, 16#32#, 16#20#,  
        16#07#, 16#C4#, 16#04#, 16#17#, 16#F4#, 16#31#, 16#C0#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Spring_68x80_Width  : constant := 68;  
    Spring_68x80_Height : constant := 80;  

    Spring_68x80_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#,  
        16#E0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#0C#, 16#98#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#14#, 16#94#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#23#, 16#E2#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#34#, 16#16#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#48#, 16#89#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#48#, 16#C9#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#7B#, 16#EF#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#49#,  
        16#89#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#C8#, 16#89#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#B4#, 16#16#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#01#, 16#E3#, 16#E2#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#03#, 16#14#, 16#94#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#03#, 16#0C#, 16#98#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#03#, 16#E0#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#06#,  
        16#03#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#06#, 16#06#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#03#, 16#04#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#03#, 16#09#, 16#40#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#01#, 16#C3#, 16#09#, 16#40#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#03#, 16#7D#, 16#8B#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#06#, 16#CA#, 16#40#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#03#, 16#CA#,  
        16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#31#,  
        16#E8#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#88#, 16#66#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#46#, 16#7B#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#21#, 16#1F#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#18#, 16#CE#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#04#, 16#1C#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#F0#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#0F#, 16#E7#, 16#F0#, 16#FE#, 16#1F#, 16#F3#, 16#02#, 16#1E#,  
        16#00#, 16#10#, 16#04#, 16#08#, 16#81#, 16#01#, 16#03#, 16#02#,  
        16#21#, 16#00#, 16#20#, 16#04#, 16#04#, 16#80#, 16#81#, 16#02#,  
        16#82#, 16#40#, 16#80#, 16#20#, 16#04#, 16#04#, 16#80#, 16#81#,  
        16#02#, 16#82#, 16#40#, 16#00#, 16#20#, 16#04#, 16#04#, 16#80#,  
        16#81#, 16#02#, 16#42#, 16#40#, 16#00#, 16#10#, 16#04#, 16#08#,  
        16#81#, 16#01#, 16#02#, 16#42#, 16#40#, 16#00#, 16#0F#, 16#87#,  
        16#F0#, 16#FE#, 16#01#, 16#02#, 16#22#, 16#47#, 16#80#, 16#00#,  
        16#44#, 16#00#, 16#82#, 16#01#, 16#02#, 16#22#, 16#40#, 16#80#,  
        16#00#, 16#24#, 16#00#, 16#82#, 16#01#, 16#02#, 16#12#, 16#40#,  
        16#80#, 16#00#, 16#24#, 16#00#, 16#81#, 16#01#, 16#02#, 16#12#,  
        16#40#, 16#80#, 16#00#, 16#24#, 16#00#, 16#81#, 16#01#, 16#02#,  
        16#0A#, 16#40#, 16#80#, 16#00#, 16#44#, 16#00#, 16#80#, 16#81#,  
        16#02#, 16#0A#, 16#21#, 16#00#, 16#3F#, 16#84#, 16#00#, 16#80#,  
        16#9F#, 16#F2#, 16#06#, 16#1E#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Spring_80x96_Width  : constant := 80;  
    Spring_80x96_Height : constant := 96;  

    Spring_80x96_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#0F#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#32#, 16#60#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#52#,  
        16#50#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#8F#, 16#88#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#D0#, 16#58#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#22#, 16#24#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#23#,  
        16#24#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#EF#, 16#BC#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#01#, 16#26#, 16#24#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#22#, 16#24#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#06#, 16#D0#,  
        16#58#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#07#, 16#8F#, 16#88#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#0C#, 16#52#, 16#50#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#0C#, 16#32#, 16#60#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#18#, 16#0F#,  
        16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#18#, 16#0E#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#18#, 16#1A#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#0C#, 16#13#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#0C#, 16#25#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#07#, 16#0C#, 16#25#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0D#, 16#F6#, 16#2D#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#,  
        16#1B#, 16#29#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#04#, 16#0F#, 16#29#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#04#, 16#C7#, 16#A3#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#21#, 16#9A#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#,  
        16#19#, 16#EE#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#84#, 16#7C#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#63#, 16#38#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#10#, 16#70#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#0F#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#FC#,  
        16#7F#, 16#03#, 16#F8#, 16#3F#, 16#E3#, 16#02#, 16#0F#, 16#00#,  
        16#02#, 16#00#, 16#40#, 16#82#, 16#04#, 16#02#, 16#03#, 16#02#,  
        16#10#, 16#80#, 16#04#, 16#00#, 16#40#, 16#42#, 16#02#, 16#02#,  
        16#02#, 16#82#, 16#20#, 16#40#, 16#04#, 16#00#, 16#40#, 16#42#,  
        16#02#, 16#02#, 16#02#, 16#82#, 16#20#, 16#00#, 16#04#, 16#00#,  
        16#40#, 16#42#, 16#02#, 16#02#, 16#02#, 16#42#, 16#20#, 16#00#,  
        16#02#, 16#00#, 16#40#, 16#82#, 16#04#, 16#02#, 16#02#, 16#42#,  
        16#20#, 16#00#, 16#01#, 16#F0#, 16#7F#, 16#03#, 16#F8#, 16#02#,  
        16#02#, 16#22#, 16#23#, 16#C0#, 16#00#, 16#08#, 16#40#, 16#02#,  
        16#08#, 16#02#, 16#02#, 16#22#, 16#20#, 16#40#, 16#00#, 16#04#,  
        16#40#, 16#02#, 16#08#, 16#02#, 16#02#, 16#12#, 16#20#, 16#40#,  
        16#00#, 16#04#, 16#40#, 16#02#, 16#04#, 16#02#, 16#02#, 16#12#,  
        16#20#, 16#40#, 16#00#, 16#04#, 16#40#, 16#02#, 16#04#, 16#02#,  
        16#02#, 16#0A#, 16#20#, 16#40#, 16#00#, 16#08#, 16#40#, 16#02#,  
        16#02#, 16#02#, 16#02#, 16#0A#, 16#10#, 16#80#, 16#07#, 16#F0#,  
        16#40#, 16#02#, 16#02#, 16#3F#, 16#E2#, 16#06#, 16#0F#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

end Tile_Data_Spring;  

