with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  

package Tile_Data_Summer is  
------------------------------------------------------------------------------  
-- Dragon - a version of Mah-Jongg for X Windows  
--  
-- Author: Gary E. Barnes        March 1989  
--  
-- tile_data_summer - Mah-Jongg tile bitmap data.  
------------------------------------------------------------------------------  
-- 05/30/90 GEB  - Translate to Ada  
------------------------------------------------------------------------------  

-- *****************************************************************************  
-- Tile Sizes  
--  
-- For each size of playing surface (there are 5 sizes) we have a different  
-- size for tiles and for what goes on them.  The tile sizes are in the Tile  
-- column.  
--  
-- Many tiles (eg. Summer) are a single image.  However, some times (eg. Crak's)  
-- are made up of composite images.  These image pieces have sizes as shown  
-- in the Objs column.  
--  
-- Many tiles (eg. Summer) have writing upon them.  The sizes of lettering  
-- on these tiles is given in the Char column.  The lettering on these tiles  
-- is positioned upwards from the bottom of the tile by the number of pixels  
-- indicated in the Up column.  The letters are spaced apart by the number of  
-- pixels indicated in the Apart column.  
--  
--       Tile    Objs    Char    Up      Apart  
--  
--       28x32   7x8     4x6     1       1  
--       40x48   10x12   5x7     1       1  
--       56x64   14x16   7x11    7       1  
--       68x80   17x20   9x13    11      2  
--       80x96   20x24   9x13    16      3  
--  
-- ****************************************************************************  
--
------------------------------------------------------------------------------  
----SEASONS  
------------------------------------------------------------------------------  

    Summer_28x32_Width  : constant := 28;  
    Summer_28x32_Height : constant := 32;  

    Summer_28x32_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#04#, 16#00#, 16#00#, 16#00#, 16#04#, 16#00#, 16#00#,  
        16#04#, 16#0E#, 16#04#, 16#00#, 16#03#, 16#3F#, 16#98#, 16#00#,  
        16#03#, 16#C4#, 16#78#, 16#00#, 16#01#, 16#04#, 16#10#, 16#00#,  
        16#01#, 16#44#, 16#50#, 16#00#, 16#02#, 16#24#, 16#88#, 16#00#,  
        16#02#, 16#15#, 16#08#, 16#00#, 16#06#, 16#00#, 16#0C#, 16#00#,  
        16#1F#, 16#F1#, 16#FF#, 16#00#, 16#06#, 16#00#, 16#0C#, 16#00#,  
        16#02#, 16#15#, 16#08#, 16#00#, 16#02#, 16#24#, 16#88#, 16#00#,  
        16#01#, 16#44#, 16#50#, 16#00#, 16#01#, 16#04#, 16#10#, 16#00#,  
        16#03#, 16#C4#, 16#78#, 16#00#, 16#03#, 16#3F#, 16#98#, 16#00#,  
        16#04#, 16#0E#, 16#04#, 16#00#, 16#00#, 16#04#, 16#00#, 16#00#,  
        16#00#, 16#04#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#3A#, 16#55#, 16#5D#, 16#C0#,  
        16#42#, 16#5D#, 16#D1#, 16#20#, 16#42#, 16#5D#, 16#D1#, 16#20#,  
        16#32#, 16#55#, 16#59#, 16#C0#, 16#0A#, 16#55#, 16#51#, 16#20#,  
        16#71#, 16#95#, 16#5D#, 16#20#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Summer_40x48_Width  : constant := 40;  
    Summer_40x48_Height : constant := 48;  

    Summer_40x48_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#10#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#10#, 16#00#, 16#00#, 16#01#, 16#00#, 16#38#, 16#01#, 16#00#,  
        16#00#, 16#C0#, 16#38#, 16#06#, 16#00#, 16#00#, 16#F0#, 16#38#,  
        16#1E#, 16#00#, 16#00#, 16#6C#, 16#54#, 16#2C#, 16#00#, 16#00#,  
        16#56#, 16#7C#, 16#D4#, 16#00#, 16#00#, 16#29#, 16#93#, 16#28#,  
        16#00#, 16#00#, 16#36#, 16#10#, 16#D0#, 16#00#, 16#00#, 16#14#,  
        16#10#, 16#50#, 16#00#, 16#00#, 16#08#, 16#10#, 16#20#, 16#00#,  
        16#00#, 16#08#, 16#10#, 16#20#, 16#00#, 16#00#, 16#30#, 16#00#,  
        16#18#, 16#00#, 16#01#, 16#D0#, 16#00#, 16#17#, 16#00#, 16#07#,  
        16#FF#, 16#83#, 16#FF#, 16#C0#, 16#01#, 16#D0#, 16#00#, 16#17#,  
        16#00#, 16#00#, 16#30#, 16#00#, 16#18#, 16#00#, 16#00#, 16#08#,  
        16#10#, 16#20#, 16#00#, 16#00#, 16#08#, 16#10#, 16#20#, 16#00#,  
        16#00#, 16#14#, 16#10#, 16#50#, 16#00#, 16#00#, 16#16#, 16#10#,  
        16#D0#, 16#00#, 16#00#, 16#29#, 16#93#, 16#28#, 16#00#, 16#00#,  
        16#56#, 16#7C#, 16#D4#, 16#00#, 16#00#, 16#68#, 16#54#, 16#2C#,  
        16#00#, 16#00#, 16#F0#, 16#38#, 16#1E#, 16#00#, 16#00#, 16#C0#,  
        16#38#, 16#06#, 16#00#, 16#01#, 16#00#, 16#38#, 16#01#, 16#00#,  
        16#00#, 16#00#, 16#10#, 16#00#, 16#00#, 16#00#, 16#00#, 16#10#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#1E#, 16#8A#, 16#28#, 16#BE#, 16#F0#, 16#20#, 16#8B#, 16#6D#,  
        16#A0#, 16#88#, 16#20#, 16#8B#, 16#6D#, 16#A0#, 16#88#, 16#1C#,  
        16#8A#, 16#AA#, 16#B8#, 16#F0#, 16#02#, 16#8A#, 16#EA#, 16#A0#,  
        16#90#, 16#02#, 16#8A#, 16#28#, 16#A0#, 16#88#, 16#3C#, 16#72#,  
        16#28#, 16#BE#, 16#88#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Summer_56x64_Width  : constant := 56;  
    Summer_56x64_Height : constant := 64;  

    Summer_56x64_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#10#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#10#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#01#, 16#00#, 16#38#, 16#01#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#C0#, 16#38#, 16#06#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#F0#, 16#38#, 16#1E#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#6C#, 16#54#, 16#2C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#56#,  
        16#7C#, 16#D4#, 16#00#, 16#00#, 16#00#, 16#00#, 16#29#, 16#93#,  
        16#28#, 16#00#, 16#00#, 16#00#, 16#00#, 16#36#, 16#10#, 16#D0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#14#, 16#10#, 16#50#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#08#, 16#10#, 16#20#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#08#, 16#10#, 16#20#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#30#, 16#00#, 16#18#, 16#00#, 16#00#, 16#00#, 16#01#,  
        16#D0#, 16#00#, 16#17#, 16#00#, 16#00#, 16#00#, 16#07#, 16#FF#,  
        16#83#, 16#FF#, 16#C0#, 16#00#, 16#00#, 16#01#, 16#D0#, 16#00#,  
        16#17#, 16#00#, 16#00#, 16#00#, 16#00#, 16#30#, 16#00#, 16#18#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#10#, 16#20#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#08#, 16#10#, 16#20#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#14#, 16#10#, 16#50#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#16#, 16#10#, 16#D0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#29#, 16#93#, 16#28#, 16#00#, 16#00#, 16#00#, 16#00#, 16#56#,  
        16#7C#, 16#D4#, 16#00#, 16#00#, 16#00#, 16#00#, 16#68#, 16#54#,  
        16#2C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#F0#, 16#38#, 16#1E#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#C0#, 16#38#, 16#06#, 16#00#,  
        16#00#, 16#00#, 16#01#, 16#00#, 16#38#, 16#01#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#10#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#10#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#07#, 16#D0#, 16#50#, 16#48#,  
        16#2F#, 16#E7#, 16#C0#, 16#08#, 16#10#, 16#58#, 16#CC#, 16#68#,  
        16#04#, 16#20#, 16#10#, 16#10#, 16#58#, 16#CC#, 16#68#, 16#04#,  
        16#10#, 16#10#, 16#10#, 16#58#, 16#CC#, 16#68#, 16#04#, 16#10#,  
        16#10#, 16#10#, 16#55#, 16#4A#, 16#A8#, 16#04#, 16#10#, 16#08#,  
        16#10#, 16#55#, 16#4A#, 16#A8#, 16#04#, 16#10#, 16#07#, 16#10#,  
        16#55#, 16#4A#, 16#AF#, 16#04#, 16#20#, 16#00#, 16#90#, 16#52#,  
        16#49#, 16#28#, 16#07#, 16#C0#, 16#00#, 16#50#, 16#52#, 16#49#,  
        16#28#, 16#04#, 16#40#, 16#00#, 16#50#, 16#52#, 16#49#, 16#28#,  
        16#04#, 16#20#, 16#00#, 16#50#, 16#50#, 16#48#, 16#28#, 16#04#,  
        16#20#, 16#00#, 16#88#, 16#90#, 16#48#, 16#28#, 16#04#, 16#10#,  
        16#1F#, 16#07#, 16#10#, 16#48#, 16#2F#, 16#E4#, 16#10#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Summer_68x80_Width  : constant := 68;  
    Summer_68x80_Height : constant := 80;  

    Summer_68x80_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#04#, 16#00#, 16#E0#, 16#04#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#03#, 16#00#, 16#E0#, 16#18#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#03#, 16#C0#, 16#E0#, 16#78#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#B1#, 16#50#, 16#B0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#59#, 16#F3#,  
        16#50#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#A6#,  
        16#4C#, 16#A0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#D8#, 16#43#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#50#, 16#41#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#20#, 16#40#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#20#, 16#40#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#C0#, 16#00#, 16#60#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#07#, 16#40#, 16#00#, 16#5C#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#1F#, 16#FE#, 16#0F#,  
        16#FF#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#07#, 16#40#,  
        16#00#, 16#5C#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#C0#, 16#00#, 16#60#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#20#, 16#40#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#20#, 16#40#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#50#, 16#41#, 16#40#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#58#, 16#43#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#A6#, 16#4C#, 16#A0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#59#, 16#F3#,  
        16#50#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#A1#,  
        16#50#, 16#B0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#,  
        16#C0#, 16#E0#, 16#78#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#03#, 16#00#, 16#E0#, 16#18#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#04#, 16#00#, 16#E0#, 16#04#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#0F#, 16#E4#, 16#04#, 16#80#, 16#90#, 16#13#, 16#FE#, 16#7F#,  
        16#00#, 16#10#, 16#04#, 16#04#, 16#C1#, 16#98#, 16#32#, 16#00#,  
        16#40#, 16#80#, 16#20#, 16#04#, 16#04#, 16#C1#, 16#98#, 16#32#,  
        16#00#, 16#40#, 16#40#, 16#20#, 16#04#, 16#04#, 16#C1#, 16#98#,  
        16#32#, 16#00#, 16#40#, 16#40#, 16#20#, 16#04#, 16#04#, 16#A2#,  
        16#94#, 16#52#, 16#00#, 16#40#, 16#40#, 16#10#, 16#04#, 16#04#,  
        16#A2#, 16#94#, 16#52#, 16#00#, 16#40#, 16#40#, 16#0F#, 16#84#,  
        16#04#, 16#A2#, 16#94#, 16#53#, 16#E0#, 16#40#, 16#80#, 16#00#,  
        16#44#, 16#04#, 16#94#, 16#92#, 16#92#, 16#00#, 16#7F#, 16#00#,  
        16#00#, 16#24#, 16#04#, 16#94#, 16#92#, 16#92#, 16#00#, 16#41#,  
        16#00#, 16#00#, 16#24#, 16#04#, 16#94#, 16#92#, 16#92#, 16#00#,  
        16#40#, 16#80#, 16#00#, 16#24#, 16#04#, 16#88#, 16#91#, 16#12#,  
        16#00#, 16#40#, 16#80#, 16#00#, 16#42#, 16#08#, 16#88#, 16#91#,  
        16#12#, 16#00#, 16#40#, 16#40#, 16#3F#, 16#81#, 16#F0#, 16#88#,  
        16#91#, 16#13#, 16#FE#, 16#40#, 16#40#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    Summer_80x96_Width  : constant := 80;  
    Summer_80x96_Height : constant := 96;  

    Summer_80x96_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#01#, 16#C0#,  
        16#08#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#06#,  
        16#01#, 16#C0#, 16#30#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#07#, 16#81#, 16#C0#, 16#F0#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#03#, 16#62#, 16#A1#, 16#60#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#B3#, 16#E6#,  
        16#A0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#,  
        16#4C#, 16#99#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#01#, 16#B0#, 16#86#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#A0#, 16#82#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#81#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#40#, 16#81#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#01#, 16#80#, 16#00#, 16#C0#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#0E#, 16#80#, 16#00#, 16#B8#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#3F#, 16#FC#, 16#1F#,  
        16#FE#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#0E#,  
        16#80#, 16#00#, 16#B8#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#01#, 16#80#, 16#00#, 16#C0#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#81#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#81#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#A0#, 16#82#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#B0#, 16#86#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#01#, 16#4C#, 16#99#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#B3#, 16#E6#,  
        16#A0#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#03#,  
        16#42#, 16#A1#, 16#60#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#07#, 16#81#, 16#C0#, 16#F0#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#06#, 16#01#, 16#C0#, 16#30#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#01#, 16#C0#,  
        16#08#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#FC#,  
        16#40#, 16#44#, 16#04#, 16#40#, 16#47#, 16#FC#, 16#7F#, 16#00#,  
        16#02#, 16#00#, 16#40#, 16#46#, 16#0C#, 16#60#, 16#C4#, 16#00#,  
        16#40#, 16#80#, 16#04#, 16#00#, 16#40#, 16#46#, 16#0C#, 16#60#,  
        16#C4#, 16#00#, 16#40#, 16#40#, 16#04#, 16#00#, 16#40#, 16#46#,  
        16#0C#, 16#60#, 16#C4#, 16#00#, 16#40#, 16#40#, 16#04#, 16#00#,  
        16#40#, 16#45#, 16#14#, 16#51#, 16#44#, 16#00#, 16#40#, 16#40#,  
        16#02#, 16#00#, 16#40#, 16#45#, 16#14#, 16#51#, 16#44#, 16#00#,  
        16#40#, 16#40#, 16#01#, 16#F0#, 16#40#, 16#45#, 16#14#, 16#51#,  
        16#47#, 16#C0#, 16#40#, 16#80#, 16#00#, 16#08#, 16#40#, 16#44#,  
        16#A4#, 16#4A#, 16#44#, 16#00#, 16#7F#, 16#00#, 16#00#, 16#04#,  
        16#40#, 16#44#, 16#A4#, 16#4A#, 16#44#, 16#00#, 16#41#, 16#00#,  
        16#00#, 16#04#, 16#40#, 16#44#, 16#A4#, 16#4A#, 16#44#, 16#00#,  
        16#40#, 16#80#, 16#00#, 16#04#, 16#40#, 16#44#, 16#44#, 16#44#,  
        16#44#, 16#00#, 16#40#, 16#80#, 16#00#, 16#08#, 16#20#, 16#84#,  
        16#44#, 16#44#, 16#44#, 16#00#, 16#40#, 16#40#, 16#07#, 16#F0#,  
        16#1F#, 16#04#, 16#44#, 16#44#, 16#47#, 16#FC#, 16#40#, 16#40#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

end Tile_Data_Summer;  

