with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  

package Tile_Data_West is  
------------------------------------------------------------------------------  
-- Dragon - a version of Mah-Jongg for X Windows  
--  
-- Author: Gary E. Barnes        March 1989  
--  
-- tile_data_west - Mah-Jongg tile bitmap data.  
------------------------------------------------------------------------------  
-- 05/30/90 GEB  - Translate to Ada  
------------------------------------------------------------------------------  

-- *****************************************************************************  
-- Tile Sizes  
--  
-- For each size of playing surface (there are 5 sizes) we have a different  
-- size for tiles and for what goes on them.  The tile sizes are in the Tile  
-- column.  
--  
-- Many tiles (eg. Summer) are a single image.  However, some times (eg. Crak's)  
-- are made up of composite images.  These image pieces have sizes as shown  
-- in the Objs column.  
--  
-- Many tiles (eg. Summer) have writing upon them.  The sizes of lettering  
-- on these tiles is given in the Char column.  The lettering on these tiles  
-- is positioned upwards from the bottom of the tile by the number of pixels  
-- indicated in the Up column.  The letters are spaced apart by the number of  
-- pixels indicated in the Apart column.  
--  
--       Tile    Objs    Char    Up      Apart  
--  
--       28x32   7x8     4x6     1       1  
--       40x48   10x12   5x7     1       1  
--       56x64   14x16   7x11    7       1  
--       68x80   17x20   9x13    11      2  
--       80x96   20x24   9x13    16      3  
--  
-- ****************************************************************************  
--
------------------------------------------------------------------------------  
----WINDS  
------------------------------------------------------------------------------  

    West_28x32_Width  : constant := 28;  
    West_28x32_Height : constant := 32;  

    West_28x32_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#48#, 16#00#, 16#00#, 16#00#,  
        16#48#, 16#00#, 16#00#, 16#00#, 16#48#, 16#00#, 16#00#, 16#00#,  
        16#78#, 16#00#, 16#20#, 16#00#, 16#78#, 16#00#, 16#70#, 16#00#,  
        16#48#, 16#01#, 16#AC#, 16#00#, 16#00#, 16#06#, 16#03#, 16#00#,  
        16#78#, 16#02#, 16#AA#, 16#00#, 16#40#, 16#02#, 16#02#, 16#00#,  
        16#40#, 16#01#, 16#24#, 16#00#, 16#70#, 16#00#, 16#88#, 16#00#,  
        16#40#, 16#01#, 16#A8#, 16#00#, 16#78#, 16#00#, 16#50#, 16#00#,  
        16#00#, 16#04#, 16#20#, 16#00#, 16#38#, 16#00#, 16#20#, 16#00#,  
        16#40#, 16#10#, 16#10#, 16#00#, 16#40#, 16#00#, 16#10#, 16#00#,  
        16#30#, 16#80#, 16#08#, 16#00#, 16#08#, 16#00#, 16#04#, 16#00#,  
        16#70#, 16#00#, 16#02#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#,  
        16#70#, 16#00#, 16#01#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#,  
        16#20#, 16#00#, 16#00#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#,  
        16#20#, 16#00#, 16#00#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    West_40x48_Width  : constant := 40;  
    West_40x48_Height : constant := 48;  

    West_40x48_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#44#, 16#00#, 16#00#,  
        16#20#, 16#00#, 16#44#, 16#00#, 16#00#, 16#70#, 16#00#, 16#54#,  
        16#00#, 16#01#, 16#8C#, 16#00#, 16#54#, 16#00#, 16#06#, 16#23#,  
        16#00#, 16#6C#, 16#00#, 16#18#, 16#00#, 16#C0#, 16#6C#, 16#00#,  
        16#0A#, 16#AA#, 16#80#, 16#44#, 16#00#, 16#08#, 16#00#, 16#80#,  
        16#00#, 16#00#, 16#04#, 16#A1#, 16#00#, 16#7C#, 16#00#, 16#02#,  
        16#02#, 16#00#, 16#40#, 16#00#, 16#02#, 16#22#, 16#00#, 16#40#,  
        16#00#, 16#01#, 16#04#, 16#00#, 16#70#, 16#00#, 16#03#, 16#24#,  
        16#00#, 16#40#, 16#00#, 16#00#, 16#88#, 16#00#, 16#40#, 16#00#,  
        16#04#, 16#50#, 16#00#, 16#7C#, 16#00#, 16#00#, 16#50#, 16#00#,  
        16#00#, 16#00#, 16#10#, 16#20#, 16#00#, 16#3C#, 16#00#, 16#00#,  
        16#20#, 16#00#, 16#40#, 16#00#, 16#40#, 16#20#, 16#00#, 16#40#,  
        16#00#, 16#00#, 16#20#, 16#00#, 16#38#, 16#01#, 16#00#, 16#10#,  
        16#00#, 16#04#, 16#00#, 16#00#, 16#18#, 16#00#, 16#04#, 16#04#,  
        16#00#, 16#04#, 16#00#, 16#78#, 16#00#, 16#00#, 16#04#, 16#00#,  
        16#00#, 16#10#, 16#00#, 16#02#, 16#00#, 16#7C#, 16#00#, 16#00#,  
        16#01#, 16#00#, 16#10#, 16#40#, 16#00#, 16#01#, 16#00#, 16#10#,  
        16#00#, 16#00#, 16#00#, 16#80#, 16#11#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#10#, 16#00#, 16#00#, 16#00#, 16#00#, 16#10#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#10#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    West_56x64_Width  : constant := 56;  
    West_56x64_Height : constant := 64;  

    West_56x64_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#49#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#49#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#49#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#49#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#55#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#55#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#55#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#,  
        16#00#, 16#22#, 16#00#, 16#00#, 16#00#, 16#07#, 16#00#, 16#00#,  
        16#22#, 16#00#, 16#00#, 16#00#, 16#18#, 16#C0#, 16#00#, 16#22#,  
        16#00#, 16#00#, 16#00#, 16#62#, 16#30#, 16#00#, 16#22#, 16#00#,  
        16#00#, 16#01#, 16#80#, 16#0C#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#AA#, 16#A8#, 16#00#, 16#7F#, 16#00#, 16#00#, 16#00#,  
        16#80#, 16#08#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#4A#,  
        16#10#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#20#, 16#20#,  
        16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#22#, 16#20#, 16#00#,  
        16#40#, 16#00#, 16#00#, 16#00#, 16#10#, 16#40#, 16#00#, 16#78#,  
        16#00#, 16#00#, 16#00#, 16#32#, 16#40#, 16#00#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#08#, 16#80#, 16#00#, 16#40#, 16#00#, 16#00#,  
        16#00#, 16#45#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#,  
        16#05#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#, 16#01#, 16#02#,  
        16#00#, 16#00#, 16#7F#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#04#, 16#02#, 16#00#, 16#00#,  
        16#1F#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#00#, 16#20#,  
        16#00#, 16#00#, 16#10#, 16#01#, 16#00#, 16#00#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#01#, 16#80#, 16#00#, 16#40#, 16#00#, 16#00#,  
        16#40#, 16#00#, 16#40#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#40#, 16#00#, 16#1C#, 16#00#, 16#01#, 16#00#, 16#00#,  
        16#20#, 16#00#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#10#,  
        16#00#, 16#01#, 16#00#, 16#04#, 16#00#, 16#00#, 16#10#, 16#00#,  
        16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#00#, 16#02#,  
        16#00#, 16#10#, 16#00#, 16#00#, 16#00#, 16#00#, 16#7C#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#7F#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#08#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#08#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#08#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#08#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#08#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#08#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    West_68x80_Width  : constant := 68;  
    West_68x80_Height : constant := 80;  

    West_68x80_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#22#, 16#20#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#22#, 16#20#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#22#, 16#20#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#22#, 16#20#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#15#,  
        16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#15#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#15#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#15#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#08#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#80#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#08#,  
        16#80#, 16#00#, 16#00#, 16#00#, 16#E0#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#03#, 16#18#, 16#00#, 16#00#,  
        16#00#, 16#3F#, 16#E0#, 16#00#, 16#00#, 16#0C#, 16#46#, 16#00#,  
        16#00#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#30#, 16#01#,  
        16#80#, 16#00#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#15#,  
        16#55#, 16#00#, 16#00#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#,  
        16#10#, 16#01#, 16#00#, 16#00#, 16#00#, 16#20#, 16#00#, 16#00#,  
        16#00#, 16#09#, 16#42#, 16#00#, 16#00#, 16#00#, 16#20#, 16#00#,  
        16#00#, 16#00#, 16#04#, 16#04#, 16#00#, 16#00#, 16#00#, 16#3E#,  
        16#00#, 16#00#, 16#00#, 16#04#, 16#44#, 16#00#, 16#00#, 16#00#,  
        16#20#, 16#00#, 16#00#, 16#00#, 16#02#, 16#08#, 16#00#, 16#00#,  
        16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#06#, 16#48#, 16#00#,  
        16#00#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#01#, 16#10#,  
        16#00#, 16#00#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#08#,  
        16#A0#, 16#00#, 16#00#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#A0#, 16#00#, 16#00#, 16#00#, 16#3F#, 16#E0#, 16#00#,  
        16#00#, 16#20#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#0F#,  
        16#E0#, 16#00#, 16#00#, 16#80#, 16#40#, 16#00#, 16#00#, 16#00#,  
        16#10#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#,  
        16#00#, 16#20#, 16#00#, 16#00#, 16#02#, 16#00#, 16#20#, 16#00#,  
        16#00#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#00#, 16#30#,  
        16#00#, 16#00#, 16#00#, 16#20#, 16#00#, 16#00#, 16#08#, 16#00#,  
        16#08#, 16#00#, 16#00#, 16#00#, 16#10#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#08#, 16#00#, 16#00#, 16#00#, 16#0F#, 16#80#, 16#00#,  
        16#20#, 16#00#, 16#04#, 16#00#, 16#00#, 16#00#, 16#00#, 16#40#,  
        16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#20#, 16#00#, 16#80#, 16#00#, 16#02#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#01#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#20#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#3F#, 16#80#, 16#10#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#3F#, 16#E0#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

    West_80x96_Width  : constant := 80;  
    West_80x96_Height : constant := 96;  

    West_80x96_Bits : constant U_Char_Array :=  
       (16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#22#, 16#20#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#22#, 16#20#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#22#, 16#20#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#22#, 16#20#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#22#, 16#20#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#15#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#15#, 16#40#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#15#, 16#40#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#15#, 16#40#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#08#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#08#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#80#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#08#, 16#80#,  
        16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#07#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#3F#, 16#E0#, 16#00#, 16#00#, 16#00#, 16#18#,  
        16#C0#, 16#00#, 16#00#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#62#, 16#30#, 16#00#, 16#00#, 16#00#, 16#20#, 16#00#,  
        16#00#, 16#00#, 16#01#, 16#80#, 16#0C#, 16#00#, 16#00#, 16#00#,  
        16#20#, 16#00#, 16#00#, 16#00#, 16#00#, 16#AA#, 16#A8#, 16#00#,  
        16#00#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#00#, 16#80#,  
        16#08#, 16#00#, 16#00#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#4A#, 16#10#, 16#00#, 16#00#, 16#00#, 16#3E#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#20#, 16#20#, 16#00#, 16#00#, 16#00#,  
        16#20#, 16#00#, 16#00#, 16#00#, 16#00#, 16#22#, 16#20#, 16#00#,  
        16#00#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#00#, 16#10#,  
        16#40#, 16#00#, 16#00#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#32#, 16#40#, 16#00#, 16#00#, 16#00#, 16#20#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#08#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#20#, 16#00#, 16#00#, 16#00#, 16#00#, 16#45#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#3F#, 16#E0#, 16#00#, 16#00#, 16#00#, 16#05#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#, 16#07#, 16#F0#,  
        16#00#, 16#00#, 16#00#, 16#02#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#08#, 16#00#, 16#00#, 16#00#, 16#04#, 16#02#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#10#, 16#00#, 16#00#, 16#00#, 16#00#, 16#02#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#10#, 16#00#, 16#00#, 16#00#,  
        16#10#, 16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#10#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#01#, 16#80#, 16#00#, 16#00#, 16#00#,  
        16#08#, 16#00#, 16#00#, 16#00#, 16#40#, 16#00#, 16#40#, 16#00#,  
        16#00#, 16#00#, 16#07#, 16#C0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#40#, 16#00#, 16#00#, 16#00#, 16#00#, 16#20#, 16#00#, 16#01#,  
        16#00#, 16#00#, 16#20#, 16#00#, 16#00#, 16#00#, 16#00#, 16#10#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#10#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#10#, 16#00#, 16#04#, 16#00#, 16#00#, 16#10#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#10#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#08#, 16#00#, 16#00#, 16#00#, 16#00#, 16#20#, 16#00#, 16#10#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#1F#, 16#C0#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#80#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#1F#, 16#F0#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#01#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#01#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#,  
        16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#, 16#00#);  

end Tile_Data_West;  

