with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  

package Ran1_Package is  
------------------------------------------------------------------------------  
-- Ran1 returns a uniform random deviate between 0.0 and 1.0.  Uses linear  
-- congruential methods using three portable not-as-good random number  
-- generators.  Near infinite period with no sensible sequential correlation.  
--  
-- Based upon:  Ran1, pg 196;  
-- Numerical Recipes: The Art Of Scientific Computing  
-- W. H. Press, B. P. Flannery, S. A. Teukolsky, W. T. Vetterling  
-- Cambridge University Press, 1986  
------------------------------------------------------------------------------  

    type Ran1_Data is private;  

    Null_Ran1_Data : constant Ran1_Data;  

    function Ran1_Initialize (Iseed : S_Natural) return Ran1_Data;  
------------------------------------------------------------------------------  
--  Iseed   - Specifies the initial seed value to be passed to the RAN()  
--              during our initialization.  
-- Called to create and initialize a Ran1_Data area for use with Ran1.  
------------------------------------------------------------------------------  

    procedure Ran1_Free (Idum : in out Ran1_Data);  
------------------------------------------------------------------------------  
--  Idum        - Specifies the data area to free.  
-- Called to free up the Ran1_Data area allocated by Ran1_Initialize.  It is  
-- OK to call this with a null.  
------------------------------------------------------------------------------  

    function Ran1 (Idum : Ran1_Data) return Float;  
------------------------------------------------------------------------------  
--  Idum    - Specifies the Ran1_Data to use in calculating the next random  
--              value.  
-- Called to calculate the next uniform random deviate between 0.0 and 1.0.  
------------------------------------------------------------------------------  

private  

    type Ran1_Data_Rec;  
    type Ran1_Data is access Ran1_Data_Rec;  
--/ if R1000 then  
    pragma Enable_Deallocation (Ran1_Data);  
  
--/ end if;  

    Null_Ran1_Data : constant Ran1_Data := null;  

end Ran1_Package;  

