with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  
with Xlbt_String;  
use Xlbt_String;  

package Xlbmp_Environment is  
------------------------------------------------------------------------------  
-- X Library Machine Dependent Environment Support  
--  
-- Xlbmp_Environment - Machine-Dependent support for environmental inquiry.  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  
--
    function X_Env_Concat_Dir_File (Dir  : X_String;  
                                    File : X_String) return X_String;  
------------------------------------------------------------------------------  
--  Dir     - Specifies a "path" to some "place" in a directory structure  
--  File    - Specifies a "file" in the directory  
--  
-- Called to attach a file name to a directory name.  
-- e.g.  
--      R1000       "!Foo.Bar.Gorp" and "Blat"   => "!Foo.Bar.Gorp.Blat"  
--      Unix        "/foo/bar/gorp" and "blat"   => "/foo/bar/gorp/blat"  
--      Vms         "Foo:[Bar.Gorp]" and "Bl.at" => "Foo:[Bar.Gorp]Bl.at"  
--  
-- The Dir does not have any "separator" syntax after the name of the  
-- path/directory.  The File does not have any "separator" syntax at the front.  
-- This routine supplies any "glue" syntax required between the Dir and the  
-- File.  
------------------------------------------------------------------------------  

    function X_Env_Get_Environment_Variable  
                (Var : X_String) return X_String_Pointer;  
------------------------------------------------------------------------------  
--  Var     - Specifies the name of the variable  
--  
-- Returns None_X_String_Pointer if the variable does not exist or if  
-- this environment/operating-system/run-time does not support any such  
-- concept as environment variables.  
--  
-- Don't forget to free the string result with Free_X_String_Pointer;  
------------------------------------------------------------------------------  

    function X_Env_Get_Home_Dir return X_String;  
------------------------------------------------------------------------------  
-- Called to obtain a string containing the user's home directory.  It does  
-- *not* have any trailing separator syntax.  i.e. It returns something  
-- like "/usr/home/foobar" and not "/usr/home/foobar/" or else  
-- "!Users.Foobar" rather than "!Users.Foobar.".  
------------------------------------------------------------------------------  

    function X_Env_Get_Host_Name return X_String;  
------------------------------------------------------------------------------  
-- Called to obtain the name of the machine we are running upon.  
------------------------------------------------------------------------------  

    function X_Env_Strip_Program_Name (Program : X_String) return X_String;  
------------------------------------------------------------------------------  
--  Program - Specifies the full/complex/whatever program name  
--  
-- Called with a string that contains the program name.  On a Unix system this  
-- is a string that contains the program name complete with possible path  
-- information.  The string came from the command line.  On VMS this would  
-- just be the name of the command verb that ran the program.  On an R1000 this  
-- is whatever string the user wanted to provide.  
--  
-- The purpose of this routine is to do whatever operating-system specific  
-- actions may be necessary to transform the string into a simple name for  
-- the purposes of resource lookup.  We always lower-case the result on  
-- systems that insist on changing the case of file/program names, e.g. VMS  
-- always makes them upper case and the R1000 always capitalizes.  On systems  
-- where case is significant, e.g. Unix, we leave the case alone.  
-- e.g.  
--      R1000       "!Foo.Bar.Gorp'V(3"     => "gorp"  
--      Unix        "/foo/bar/gorp"         => "gorp"  
--      Vms         "Foo:[Bar]Gorp.Exe"     => "gorp"  
------------------------------------------------------------------------------  

--
end Xlbmp_Environment;  

