--/ if R1000 then  
with Transport;  
with Transport_Defs;  
--/ end if;  

with Xlbmt_Network_Types;  
use Xlbmt_Network_Types;  

package Xlbmp_Network_Interface is  
------------------------------------------------------------------------------  
-- X Library Machine Network Interfaces  
--  
-- Xlbmp_Network_Interface - Machine/Compiler dependent network interfaces.  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the name of Rational not be used in  
-- advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- Rational disclaims all warranties with regard to this software, including  
-- all implied warranties of merchantability and fitness, in no event shall  
-- Rational be liable for any special, indirect or consequential damages or  
-- any damages whatsoever resulting from loss of use, data or profits, whether  
-- in an action of contract, negligence or other tortious action, arising out  
-- of or in connection with the use or performance of this software.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- Network Reception - Receive data  
------------------------------------------------------------------------------  

    procedure Receive_Maybe (Connection :        Connection_Id;  
                             Status     : out    Status_Code;  
                             Data       : in out X_Raw_Data_Array;  
                             Count      : out    X_Raw_Data_Index);  
------------------------------------------------------------------------------  
--  Connection  - Specifies the connection to use for reading  
--  Status      - Receives the final status of the operation  
--  Data        - Receives the data  
--  Count       - Receives the length of the data  
--                  isn't any available  
--  
-- Called when we are ready to receive something from the network connection  
-- but we won't be upset if there isn't anything to receive.  
------------------------------------------------------------------------------  

    procedure Receive_Must (Connection :        Connection_Id;  
                            Status     : out    Status_Code;  
                            Data       : in out X_Raw_Data_Array;  
                            Count      : out    X_Raw_Data_Index);  
------------------------------------------------------------------------------  
--  Connection  - Specifies the connection to use for reading  
--  Status      - Receives the final status of the operation  
--  Data        - Receives the data  
--  Count       - Receives the length of the data  
--  
-- Called when we are ready to receive some specific number of bytes from the  
-- network connection and we will wait forever if necessary in order to get  
-- exactly that many bytes.  
------------------------------------------------------------------------------  

--/ if R1000 and not R1000_Xlib_Only then  
  
    procedure Receive_Queue (Connection :        Connection_Id;  
                             Status     : out    Status_Code;  
                             Data       : in out X_Raw_Data_Array;  
                             Count      : out    X_Raw_Data_Index;  
                             Notify     :        Notify_Complete);  
------------------------------------------------------------------------------  
--  Connection  - Specifies the connection to use for reading  
--  Status      - Receives the final status of the operation  
--  Data        - Receives the data  
--  Count       - Receives the length of the data  
--                  isn't any available  
--  Notify      - Specifies the completion task to use when queuing a read  
--  
-- This interface is for the use of the Xt library.  Applications should not  
-- use it unless they are very very very certain that they understand it  
-- and they are very very very certain that their use will not bother Xt's  
-- use of it for a given connection.  
--  
-- Called when we are ready to receive something from the network connection  
-- but we won't be upset if there isn't anything to receive.  In addition,  
-- we tell the Connection_Reader task to queue a read and to let us know  
-- via the Notify_Complete task when the read has occured.  
------------------------------------------------------------------------------  
  
--/ end if; -- not R1000_Xlib_Only  

--
------------------------------------------------------------------------------  
-- Network Transmission - Transmit data  
------------------------------------------------------------------------------  

    procedure Transmit_Maybe (Connection :     Connection_Id;  
                              Status     : out Status_Code;  
                              Data       :     X_Raw_Data_Array;  
                              Count      : out X_Raw_Data_Index);  
------------------------------------------------------------------------------  
--  Connection  - Specifies the connection to use for writing  
--  Status      - Receives the final status of  operation  
--  Data        - Specifies the data  
--  Count       - Receives the length of the data actually sent  
--  
-- Called when we want to transmit something on the network and we have other  
-- things we could be doing if the transmit would block for some reason.  We  
-- send out as much as possible and return.  
------------------------------------------------------------------------------  

--
end Xlbmp_Network_Interface;  

