with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  

package Xlbp_U_Char_Generics is  
------------------------------------------------------------------------------  
-- X Library Machine Conversions  
--  
-- Xlbp_U_Char_Generics - Machine/Compiler dependent conversions to/from  
--  the U_Char_Array data type.  
------------------------------------------------------------------------------  
-- Copyright 1990 - 1991 by Rational, Santa Clara, California.  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the name of Rational not be used in  
-- advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- Rational disclaims all warranties with regard to this software, including  
-- all implied warranties of merchantability and fitness, in no event shall  
-- Rational be liable for any special, indirect or consequential damages or  
-- any damages whatsoever resulting from loss of use, data or profits, whether  
-- in an action of contract, negligence or other tortious action, arising out  
-- of or in connection with the use or performance of this software.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- Generic converters to the U_Char_Array type.  
--  
-- These take the compiler/machine specific steps that are necessary in order  
-- to convert a value of some time into an instance of the U_Char_Array  
-- type.  E.g. with array types we make sure that only the data portion of  
-- the array is converted and not, for instance, the dope/array-index  
-- information.  
------------------------------------------------------------------------------  

    generic  
        type Original is (<>);  
    procedure Convert_Discrete_To_Uca (Uca  : out U_Char_Array;  
                                       Orig :     Original);  

    generic  
        type Original is private;  
    procedure Convert_Private_To_Uca (Uca  : out U_Char_Array;  
                                      Orig :     Original);  

    generic  
        type Original       is private;  
        type Index          is (<>);  
        type Original_Array is array (Index range <>) of Original;  
    procedure Convert_Array_To_Uca (Uca  : out U_Char_Array;  
                                    Orig :     Original_Array);  

    generic  
        type Original       is private;  
        type Index1         is (<>);  
        type Index2         is (<>);  
        type Original_Array is array (Index1 range <>,  
                                      Index2 range <>) of Original;  
    procedure Convert_2d_Array_To_Uca (Uca  : out U_Char_Array;  
                                       Orig :     Original_Array);  


------------------------------------------------------------------------------  
-- Generic converters from the U_Char_Array type.  
--  
-- These take the compiler/machine specific steps that are necessary in order  
-- to convert an U_Char_Array value into an instance of the Original  
-- type.  E.g. with array types we make sure that the converted information  
-- occupies only the data portion of the array after conversion and not, for  
-- instance, the dope/array-index portion.  
------------------------------------------------------------------------------  

    generic  
        type Original is (<>);  
    procedure Convert_Uca_To_Discrete (Orig : out Original;  
                                       Uca  :     U_Char_Array);  

    generic  
        type Original is private;  
    procedure Convert_Uca_To_Private (Orig : out Original;  
                                      Uca  :     U_Char_Array);  

    generic  
        type Original       is private;  
        type Index          is (<>);  
        type Original_Array is array (Index range <>) of Original;  
    procedure Convert_Uca_To_Array (Orig : out Original_Array;  
                                    Uca  :     U_Char_Array);  
    ----Convert U_Char data into an array.  

    generic  
        type Original       is private;  
        type Index1         is (<>);  
        type Index2         is (<>);  
        type Original_Array is array (Index1 range <>,  
                                      Index2 range <>) of Original;  
    procedure Convert_Uca_To_2d_Array (Orig : out Original_Array;  
                                       Uca  :     U_Char_Array);  

end Xlbp_U_Char_Generics;  

