with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  
with Xlbt_Basic;  
use Xlbt_Basic;  
with Xlbt_Bitmap;  
use Xlbt_Bitmap;  

package Xlbp_Bitmap is  
------------------------------------------------------------------------------  
-- X Library Bitmaps  
--  
-- Xlbp_Bitmap - Used to create bitmaps and bitmap files.  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
    function X_Create_Bitmap_From_Data (Display  : X_Display;  
                                        Drawable : X_Drawable;  
                                        Data     : U_Char_Array;  
                                        Width    : U_Short;  
                                        Height   : U_Short) return X_Pixmap;  
------------------------------------------------------------------------------  
--  Display   - Specifies the display to use.  
--  Drawable  - Specifies the drawable (X_Window, X_Pixmap) to use.  
--  Data      - Specifies the data to use in creating the pixmap.  
--  Width     - Specifies the width of the bitmap.  
--  Height    - Specifies the height of the bitmap.  
--  
-- Creates a pixmap from data available in memory.  
------------------------------------------------------------------------------  

    function X_Create_Pixmap_From_Bitmap_Data  
                (Display    : X_Display;  
                 Drawable   : X_Drawable;  
                 Data       : U_Char_Array;  
                 Width      : U_Short;  
                 Height     : U_Short;  
                 Foreground : X_Pixel;  
                 Background : X_Pixel;  
                 Depth      : U_Char) return X_Pixmap;  
------------------------------------------------------------------------------  
--  Display   - Specifies the display to use.  
--  Drawable  - Specifies the drawable (Window, Pixmap) to use.  
--  Data      - Specifies the data to use in creating the pixmap.  
--  Width     - Receives the width of the bitmap read.  
--  Height    - Receives the height of the bitmap read.  
--  Foreground- Specifies the pixel to use for the foreground color;  
--  Background- Specifies the pixel to use for the background color;  
--  Depth     - Specifies the depth of the pixmap.  
--  
-- Creates a pixmap of the given depth and then does a bitmap-format  
-- X_Put_Image of the data into it.  
------------------------------------------------------------------------------  

    function X_Write_Bitmap_File (Display   : X_Display;  
                                  Filename  : String;  
                                  Bitmap    : X_Pixmap;  
                                  Width     : U_Short;  
                                  Height    : U_Short;  
                                  X_Hotspot : S_Short;  
                                  Y_Hotspot : S_Short) return X_Bitmap_Returns;  
------------------------------------------------------------------------------  
--  Display   - Specifies the display to use.  
--  Filename  - Specifies the name of the file to read.  The format of this  
--              file is operating system dependent.  
--  Bitmap    - Specifies the bitmap ID to be written.  
--  Width     - Specifies the width of the bitmap to be written.  
--  Height    - Specifies the height of the bitmap to be written.  
--  X_Hotspot - Specifies the hotspot X coordinate.  
--  Y_Hotspot - Specifies the hotspot Y coordinate.  
--  
-- Writes a bitmap file based upon an existing Pixmap.  Returns a success  
-- status indicating success or else the failure class.  
------------------------------------------------------------------------------  

    procedure X_Read_Bitmap_File (Display   :     X_Display;  
                                  Drawable  :     X_Drawable;  
                                  Filename  :     String;  
                                  Width     : out U_Short;  
                                  Height    : out U_Short;  
                                  Bitmap    : out X_Pixmap;  
                                  X_Hotspot : out S_Short;  
                                  Y_Hotspot : out S_Short;  
                                  Status    : out X_Bitmap_Returns);  
------------------------------------------------------------------------------  
--  Display   - Specifies the display to use.  
--  Drawable  - Specifies the drawable (Window, Pixmap) to use.  
--  Filename  - Specifies the name of the file to read.  The format of this  
--              file is operating system dependent.  
--  Width     - Receives the width of the bitmap read.  
--  Height    - Receives the height of the bitmap read.  
--  Bitmap    - Receives the bitmap ID that was created.  
--  X_Hotspot - Receives the hotspot X coordinate.  
--  Y_Hotspot - Receives the hotspot Y coordinate.  
--  Status    - Receives the success/failure/other status.  
--  
-- Read a bitmap file and create a pixmap of the appropriate size.  Use  
-- X_Free_Pixmap when done with the new pixmap.  
------------------------------------------------------------------------------  

--
end Xlbp_Bitmap;  

