with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  
with Xlbt_Basic;  
use Xlbt_Basic;  
with Xlbt_Cut_Buffer;  
use Xlbt_Cut_Buffer;  

package Xlbp_Cut_Buffer is  
------------------------------------------------------------------------------  
-- X Library Cut and Paste Buffers  
--  
-- Xlbp_Cut_Buffer - Used to handle cut-and-paste buffers.  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
    function X_Fetch_Buffer (Display : X_Display;  
                             Buffer  : X_Cut_Buffer_Number) return U_Char_List;  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Buffer  - Specifies the cut buffer to use.  
--  
-- Returns the data bytes stored in the specified cut buffer.  If there is no  
-- data in the cut buffer then a null array is returned.  
------------------------------------------------------------------------------  

    function X_Fetch_Bytes (Display : X_Display) return U_Char_List;  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  
-- Returns the data bytes stored in cut buffer 0.  If there is no data in the  
-- cut buffer then a null array is returned.  
------------------------------------------------------------------------------  

    procedure X_Rotate_Buffers (Display : X_Display;  
                                Rotate  : S_Short);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Rotate  - Specifies the rotation amount.  
--  
-- Rotates the cut buffers such that cut buffer 0 becomes cut buffer N, cut  
-- buffer 1 becomes N+1, etc. modulo the number of buffers.  Will generate  
-- a protocol error if not all 8 cut buffers have been created.  
------------------------------------------------------------------------------  

    procedure X_Store_Buffer (Display : X_Display;  
                              Bytes   : U_Char_Array;  
                              Buffer  : X_Cut_Buffer_Number);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Bytes   - Specifies the bytes to store.  The entire array is stored.  
--  Buffer  - Specifies the cut buffer to use.  
--  
-- Stores data into the cut buffer specified.  
------------------------------------------------------------------------------  

    procedure X_Store_Bytes (Display : X_Display;  
                             Bytes   : U_Char_Array);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Bytes   - Specifies the bytes to store.  The entire array is stored.  
--  
-- Stores data into cut buffer 0.  
------------------------------------------------------------------------------  

end Xlbp_Cut_Buffer;  

