with Xlbt_Display3;  
use Xlbt_Display3;  
with Xlbt_Error;  
use Xlbt_Error;  
with Xlbt_String;  
use Xlbt_String;  

with Xlbp_Proc_Var;  
use Xlbp_Proc_Var;  

package Xlbp_Error is  
------------------------------------------------------------------------------  
-- X Library Error Control  
--  
-- Xlbp_Error - Provides control over error responses within the X Library.  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1987 - 1989 by Digital Equipment Corporation, Maynard, Mass.  
-- Copyright 1987 - 1989 by Massachusetts Institute of Technology,  
--                          Cambridge, Massachusetts.  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of Digital, MIT, or Rational  
-- not be used in advertising or publicity pertaining to distribution of  
-- the software without specific, written prior permission.  
--  
-- Digital, MIT, and Rational disclaim all warranties with regard to this  
-- software, including all implied warranties of merchantability and fitness,  
-- in no event shall Digital, MIT, or Rational be liable for any special,  
-- indirect or consequential damages or any damages whatsoever resulting from  
-- loss of use, data or profits, whether in an action of contract, negligence  
-- or other tortious action, arising out of or in connection with the use or  
-- performance of this software.  
------------------------------------------------------------------------------  

--
    function X_Get_Error_Database_Text (Display : X_Display;  
                                        Name    : X_String;  
                                        Message : X_String;  
                                        Default : X_String) return X_String;  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Name    - Specifies the name of the application.  
--  Message - Specifies the type of the error message.  
--  Default - Specifies the default to use if the database has no entry.  
--  
-- Returns a message string from the error message database.  
-- These predefined Names exist:  
-- "XProtoError"    - protocol error code names  
-- "XlibMessage"    - message strings used internally by Xlib  
-- "XRequestMajor"  - protocol request code names  
------------------------------------------------------------------------------  

    function X_Get_Error_Text (Display : X_Display;  
                               Kind    : X_Error_Code) return X_String;  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Kind    - Specifies the error code to look up.  
--  
-- Returns an Ada string whose contents describe the specific error code.  
-- Equivalent to a call on X_Get_Error_Database_Text(Display,"XProtoError",  
-- X_Error_Code'Image(Kind),...) if there are no extensions present.  This  
-- routine also queries each active extension to see if they have anything to  
-- add to the error description.  
------------------------------------------------------------------------------  

    function X_Get_Error_String (Name1   : X_String;  
                                 Name2   : X_String;  
                                 Default : X_String) return X_String;  
------------------------------------------------------------------------------  
--  Name1   - Specifies the first part of the error name  
--  Name2   - Specifies the second part of the error name  
--  Default - Specifies a default error message string  
--  
-- Called to obtain Name1.Name2 or else Name1.Name2.1 & Name1.Name2.2 & ...  
-- from Xlib's system-wide error message database.  A message may appear in the  
-- database in either of two ways.  This way:  
--  
--      Name1.Name2 : Message Text  
--  
-- or else in a multipart fashion like this:  
--  
--      Name1.Name2.1 : First line of message  
--      Name1.Name2.2 : Second line of message  
--      Name1.Name2.3 : Third line of message  
--          ...and so on...  
--  
-- If we can't find either type of entry then we return the Default value.  
------------------------------------------------------------------------------  

    function X_Set_Error_Handler (Handler : Proc_Var_X_Error_Function.Pv)  
                                 return Proc_Var_X_Error_Function.Pv;  
    function X_Set_Error_Handler (Display : X_Display;  
                                  Handler : Proc_Var_X_Error_Function.Pv)  
                                 return Proc_Var_X_Error_Function.Pv;  
------------------------------------------------------------------------------  
--  Display - Specifies which display to use.  
--  Handler - Specifies the new error handler to use.  
--  
-- Changes the current error handler.  Changes either the global default  
-- for all displays or else changes it just for one display.  
-- Returns the previous global or display handler routine.  The handler routine  
-- is allowed to return.  It must not perform any action that would change the  
-- state of any X_Display.  
------------------------------------------------------------------------------  

    function X_Set_Io_Error_Handler (Handler : Proc_Var_X_Io_Error_Function.Pv)  
                                    return Proc_Var_X_Io_Error_Function.Pv;  
    function X_Set_Io_Error_Handler (Display : X_Display;  
                                     Handler : Proc_Var_X_Io_Error_Function.Pv)  
                                    return Proc_Var_X_Io_Error_Function.Pv;  
------------------------------------------------------------------------------  
--  Display - Specifies which display to use.  
--  Handler - Specifies the new I/O error handler to use.  
--  
-- Changes the current I/O error handler.  Changes either the global default  
-- for all displays or else changes it just for one display.  
-- Returns the previous handler routine.  The handler must not return; it  
-- may raise an exception if it so desires, the exception will be allowed to  
-- propagate out to the enclosing application code.  
-- It must not perform any action that would change the state of any  
-- X_Display.  The X_Display involved in the I/O operation can be presumed to  
-- be unusable at this point; it should still be closed with X_Close_Display.  
-- Other X_Displays still existing in Xlib will continue to  
-- function (unless the basic problem is that the X server is dead).  
------------------------------------------------------------------------------  

    function X_Set_Report_Handler (Handler : Proc_Var_X_Report_Error.Pv)  
                                  return Proc_Var_X_Report_Error.Pv;  
    function X_Set_Report_Handler (Display : X_Display;  
                                   Handler : Proc_Var_X_Report_Error.Pv)  
                                  return Proc_Var_X_Report_Error.Pv;  
------------------------------------------------------------------------------  
--  Display - Specifies which display to use.  
--  Handler - Specifies the new error handler to use.  
--  
-- Changes the current error report handler.  Changes either the global default  
-- for all displays or else changes it just for one display.  
-- Returns the previous handler routine.  The handler is epected to return.  
-- This routine must not perform any action that would change the state of any  
-- X_Display.  
------------------------------------------------------------------------------  

end Xlbp_Error;  

