with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  
with Xlbt_Basic;  
use Xlbt_Basic;  
with Xlbt_Font;  
use Xlbt_Font;  
with Xlbt_String;  
use Xlbt_String;  

package Xlbp_Font is  
------------------------------------------------------------------------------  
-- X Library Fonts  
--  
-- Xlbp_Font - Loading and using fonts.  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
    function X_Load_Query_Font (Display : X_Display;  
                                Name    : X_String) return X_Font_Struct;  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Name    - Specifies the name of the font to load.  
--  
-- Opens and load the font.  Returns None_X_Font_Struct if the font does not  
-- exist or could not be loaded.  Call X_Free_Font to free the storage  
-- associated with the X_Font_Struct when done.  
------------------------------------------------------------------------------  

    procedure X_Free_Font (Display :        X_Display;  
                           Font    : in out X_Font_Struct);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Font    - Specifies the font structure to be freed.  
--  
-- Frees the font structure and sets the pointer to None_X_Font_Struct.  
------------------------------------------------------------------------------  

    function X_Query_Font (Display : X_Display;  
                           Font    : X_Font) return X_Font_Struct;  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Font    - Specifies the ID of the font or graphics context.  
--  
-- Returns None_X_Font_Struct on errors.  Fetches the font information  
-- associated with a particular font ID.  Call X_Free_Font to free the storage  
-- associated with the X_Font_Struct when done.  
------------------------------------------------------------------------------  

    function X_Load_Font (Display : X_Display;  
                          Name    : X_String) return X_Font;  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Name    - Specifies the name of the font to load.  
--  
-- Load the specified font and returns its associated ID.  If the function was  
-- unsuccessful at loading the specified font it generates a Bad_Name error.  
------------------------------------------------------------------------------  

    procedure X_Unload_Font (Display : X_Display;  
                             Font    : X_Font);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Font    - Specifies the font to unload.  
--  
-- Deletes the association between the font ID and the loaded font.  The font  
-- is unloaded only after the last reference to it is deleted.  
------------------------------------------------------------------------------  

    procedure X_Get_Font_Property (Font   :     X_Font_Struct;  
                                   Name   :     X_Atom;  
                                   Value  : out S_Long;  
                                   Status : out X_Status);  
------------------------------------------------------------------------------  
--  Font    - Specifies the font to query.  
--  Name    - Specifies the name of the property to query.  
--  Value   - Receives the value of the property.  
--  Status  - Receives Successful if the property is present and Failed if  
--            not.  
--  
-- Returns the value of the property for this font.  
------------------------------------------------------------------------------  

end Xlbp_Font;  

