with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  
with Xlbt_Basic;  
use Xlbt_Basic;  
with Xlbt_Gc;  
use Xlbt_Gc;  
with Xlbt_Graphics;  
use Xlbt_Graphics;  

package Xlbp_Gc is  
------------------------------------------------------------------------------  
-- X Library Graphic Context  
--  
-- Xlbp_Gc - Create and maintain Graphic Contexts  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
    procedure X_Change_Gc (Display     : X_Display;  
                           Gc          : X_Gc;  
                           Values_Mask : X_Gc_Components;  
                           Values      : X_Gc_Values);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Gc          - Specifies which graphic context to change.  
--  Values_Mask - Specifies which components of Values to set in Gc.  
--  Values      - Specifies the new component values for Gc.  
--  
-- Changes those components in Gc as specified by Values_Mask to match the  
-- corresponding Values components.  
------------------------------------------------------------------------------  

    procedure X_Copy_Gc (Display        : X_Display;  
                         Source_Gc      : X_Gc;  
                         Values_Mask    : X_Gc_Components;  
                         Destination_Gc : X_Gc);  
------------------------------------------------------------------------------  
--  Display        - Specifies the display to use.  
--  Source_Gc      - Specifies the source graphic context.  
--  Values_Mask    - Specifies which components of the Source_Gc to copy.  
--  Destination_Gc - Specifies and returns the Gc which is to receive the  
--                   components.  
--  
-- Copies the components of Source_Gc specified by Mask into Destination_Gc.  
------------------------------------------------------------------------------  

    function X_Create_Gc (Display     : X_Display;  
                          Drawable    : X_Drawable;  
                          Values_Mask : X_Gc_Components;  
                          Values      : X_Gc_Values) return X_Gc;  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Drawable    - Specifies the drawable to use.  
--  Values_Mask - Specifies which components of Values to use in the creation.  
--  Values      - Specifies various values to be used in creating the X_Gc.  
--  
-- Creates a new graphic context and returns the X_Gc that corresponds to it.  
------------------------------------------------------------------------------  

    procedure X_Free_Gc (Display :        X_Display;  
                         Gc      : in out X_Gc);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Gc      - Specifies the GC that is to be destroyed.  
--  
-- Destroys the specified graphics context.  
------------------------------------------------------------------------------  

    function X_G_Context_From_Gc (Gc : X_Gc) return X_G_Context;  
------------------------------------------------------------------------------  
--  Gc      - Specifies the graphics context to use.  
--  
-- Returns the graphics context ID associated with the X_Gc.  
------------------------------------------------------------------------------  

    procedure X_Get_Gc_Values (Display     :     X_Display;  
                               Gc          :     X_Gc;  
                               Values_Mask :     X_Gc_Components;  
                               Values      : out X_Gc_Values;  
                               Status      : out X_Status);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Gc          - Specifies the graphics context to use.  
--  Values_Mask - Specifies what values to return  
--  Values      - Receives the values returned  
--  Status      - Receives Successful if all values were returned, Failed if  
--                  some values were not returned  
--  
-- Called to obtain some values from those that make up a GC.  Can return all  
-- GC fields except Gc_Clip_Mask and Gc_Dash_List.  
------------------------------------------------------------------------------  

    procedure X_Query_Best_Size (Display     :     X_Display;  
                                 Class       :     X_Best_Size_Class;  
                                 Drawable    :     X_Drawable;  
                                 Width       :     U_Short;  
                                 Height      :     U_Short;  
                                 Best_Width  : out U_Short;  
                                 Best_Height : out U_Short;  
                                 Status      : out X_Status);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Class       - Specifies the class of object whose sizes we want.  
--  Drawable    - Specifies (indirectly) the screen being queried.  
--  Width       - Specifies the width, in pixels, which we would like.  
--  Height      - Specifies the height, in pixels, which we would like.  
--  Best_Width  - Receives the best or closest width, in pixels, to Width.  
--  Best_Height - Receives the best or closest heigh, in pixels, to Height.  
--  Status      - Receives Successful or Failed.  
--  
-- Queries the server to discover the best or closest size to some desired  
-- size of some class of drawable object.  The Best_Width/Height returned  
-- indicates the size most supported by the server that is the best or closest  
-- fit to the desired size.  
------------------------------------------------------------------------------  

    procedure X_Query_Best_Stipple (Display     :     X_Display;  
                                    Drawable    :     X_Drawable;  
                                    Width       :     U_Short;  
                                    Height      :     U_Short;  
                                    Best_Width  : out U_Short;  
                                    Best_Height : out U_Short;  
                                    Status      : out X_Status);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Drawable    - Specifies (indirectly) the screen being queried.  
--  Width       - Specifies the width, in pixels, which we would like.  
--  Height      - Specifies the height, in pixels, which we would like.  
--  Best_Width  - Receives the best or closest width, in pixels, to Width.  
--  Best_Height - Receives the best or closest heigh, in pixels, to Height.  
--  Status      - Receives Successful or Failed.  
--  
-- Queries the server to discover the best or closest size to some desired  
-- size of stipple.  The Best_Width/Height returned indicates the size most  
-- supported by the server that is the best or closest fit to the desired size.  
------------------------------------------------------------------------------  

    procedure X_Query_Best_Tile (Display     :     X_Display;  
                                 Drawable    :     X_Drawable;  
                                 Width       :     U_Short;  
                                 Height      :     U_Short;  
                                 Best_Width  : out U_Short;  
                                 Best_Height : out U_Short;  
                                 Status      : out X_Status);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Drawable    - Specifies (indirectly) the screen being queried.  
--  Width       - Specifies the width, in pixels, which we would like.  
--  Height      - Specifies the height, in pixels, which we would like.  
--  Best_Width  - Receives the best or closest width, in pixels, to Width.  
--  Best_Height - Receives the best or closest heigh, in pixels, to Height.  
--  Status      - Receives Successful or Failed.  
--  
-- Queries the server to discover the best or closest size to some desired  
-- size of tile.  The Best_Width/Height returned indicates the size most  
-- supported by the server that is the best or closest fit to the desired size.  
------------------------------------------------------------------------------  

    procedure X_Set_Arc_Mode (Display  : X_Display;  
                              Gc       : X_Gc;  
                              Arc_Mode : X_Arc_Mode);  
------------------------------------------------------------------------------  
--  Display   - Specifies the display to use.  
--  Gc        - Specifies the graphics context to use.  
--  Arc_Mode  - Specifies the arc mode to use.  
--  
-- Set the arc mode to use in the graphics context.  
------------------------------------------------------------------------------  

    procedure X_Set_Background (Display    : X_Display;  
                                Gc         : X_Gc;  
                                Background : X_Pixel);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Gc          - Specifies the graphics context to use.  
--  Background  - Specifies the new background pixel to use.  
--  
-- Sets the pixel value to use in future graphics operations using this  
-- graphics context.  
------------------------------------------------------------------------------  

    procedure X_Set_Clip_Mask (Display : X_Display;  
                               Gc      : X_Gc;  
                               Mask    : X_Pixmap);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Gc      - Specifies the graphics context to use.  
--  Mask    - Specifies the clip mask to use.  
--  
-- Sets the clip mask for the graphics context.  
------------------------------------------------------------------------------  

    procedure X_Set_Clip_Origin (Display : X_Display;  
                                 Gc      : X_Gc;  
                                 X       : S_Short;  
                                 Y       : S_Short);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Gc      - Specifies the graphics context to use.  
--  X       - Specifies the X coordinate for the clip origin.  
--  Y       - Specifies the Y coordinate for the clip origin.  
--  
-- Sets the clip origin for the graphics context.  
------------------------------------------------------------------------------  

    procedure X_Set_Clip_Rectangles (Display    : X_Display;  
                                     Gc         : X_Gc;  
                                     X          : S_Short;  
                                     Y          : S_Short;  
                                     Rectangles : X_Rectangle_Array;  
                                     Ordering   : X_Clip_Ordering);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Gc          - Specifies the graphics context to use.  
--  X           - Specifies the X coordinate for the clip origin.  
--  Y           - Specifies the Y coordinate for the clip origin.  
--  Rectangles  - Specifies the clipping rectangles.  
--  Ordering    - Specifies the clip ordering to use.  
--  
-- Sets the clip mask for the graphics context.  
------------------------------------------------------------------------------  

    procedure X_Set_Dashes (Display     : X_Display;  
                            Gc          : X_Gc;  
                            Dash_Offset : U_Short;  
                            Dash_List   : U_Char_Array);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Gc          - Specifies the graphics context to use.  
--  Dash_Offset - Specifies the phase of the pattern for the dashed line style  
--                being set.  
--  Dash_List   - Specifies the dash list for the dashed line style.  
--  
-- Sets the Dash_Offset and the Dash_List  
------------------------------------------------------------------------------  

    procedure X_Set_Fill_Rule (Display   : X_Display;  
                               Gc        : X_Gc;  
                               Fill_Rule : X_Fill_Rule);  
------------------------------------------------------------------------------  
--  Display   - Specifies the display to use.  
--  Gc        - Specifies the graphics context to use.  
--  Fill_Rule - Specifies the fill rule to use.  
--  
-- Sets the fill rule in use with the graphics context.  
------------------------------------------------------------------------------  

    procedure X_Set_Fill_Style (Display    : X_Display;  
                                Gc         : X_Gc;  
                                Fill_Style : X_Fill_Style);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Gc          - Specifies the graphics context to use.  
--  Fill_Style  - Specifies the fill style to use.  
--  
-- Sets the fill style in use with the graphics context.  
------------------------------------------------------------------------------  

    procedure X_Set_Font (Display : X_Display;  
                          Gc      : X_Gc;  
                          Font    : X_Font);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Gc      - Specifies the graphics context to use.  
--  Font    - Specifies the font to use.  
--  
-- Sets the font in use in a graphics context.  
------------------------------------------------------------------------------  

    procedure X_Set_Foreground (Display    : X_Display;  
                                Gc         : X_Gc;  
                                Foreground : X_Pixel);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Gc          - Specifies the graphics context to use.  
--  Foreground  - Specifies the new foreground pixel to use.  
--  
-- Sets the pixel value to use in future graphics operations using this  
-- graphics context.  
------------------------------------------------------------------------------  

    procedure X_Set_Function (Display : X_Display;  
                              Gc      : X_Gc;  
                              Funct   : X_Graphic_Function);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Gc      - Specifies the graphics context to use.  
--  Funct       - Specifies the new graphic function to use.  
--  
-- Sets the graphic function to use in future graphics operations using this  
-- graphics context.  
------------------------------------------------------------------------------  

    procedure X_Set_Graphics_Exposures  
                 (Display            : X_Display;  
                  Gc                 : X_Gc;  
                  Graphics_Exposures : Boolean);  
------------------------------------------------------------------------------  
--  Display             - Specifies the display to use.  
--  Gc                  - Specifies the graphics context to use.  
--  Graphics_Exposures  - Specifies True if Graphics_Expose events are desired.  
--  
-- Sets the Graphics_Exposures flag in the graphics context.  
------------------------------------------------------------------------------  

    procedure X_Set_Line_Attributes (Display    : X_Display;  
                                     Gc         : X_Gc;  
                                     Line_Width : U_Short;  
                                     Line_Style : X_Line_Style;  
                                     Cap_Style  : X_Cap_Style;  
                                     Join_Style : X_Join_Style);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Gc          - Specifies the graphics context to use.  
--  Line_Width  - Specifies the line width to use.  
--  Line_Style  - Specifies the line style to use.  
--  Cap_Style   - Specifies the cap style to use when terminating lines.  
--  Join_Style  - Specifies the join style to use when joining line ends.  
--  
-- Sets all of these things in the graphics context.  These values will take  
-- effect in all successive uses of the X_Gc.  
------------------------------------------------------------------------------  

    procedure X_Set_Plane_Mask (Display    : X_Display;  
                                Gc         : X_Gc;  
                                Plane_Mask : X_Plane_Mask);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Gc          - Specifies the graphics context to use.  
--  Plane_Mask  - Specifies the new plane mask to use.  
--  
-- Sets the plane mask to use in future graphics operations using this  
-- graphics context.  
------------------------------------------------------------------------------  

    procedure X_Set_State (Display    : X_Display;  
                           Gc         : X_Gc;  
                           Foreground : X_Pixel;  
                           Background : X_Pixel;  
                           Funct      : X_Graphic_Function;  
                           Plane_Mask : X_Plane_Mask);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Gc          - Specifies the graphics context to use.  
--  Foreground  - Specifies the new foreground pixel to use.  
--  Background  - Specifies the new background pixel to use.  
--  Funct       - Specifies the new graphic function to use.  
--  Plane_Mask  - Specifies the new plane mask to use.  
--  
-- Sets all of these things in the graphics context.  These values will take  
-- effect in all successive uses of the X_Gc.  
------------------------------------------------------------------------------  

    procedure X_Set_Stipple (Display : X_Display;  
                             Gc      : X_Gc;  
                             Stipple : X_Pixmap);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Gc      - Specifies the graphics context to use.  
--  Stipple - Specifies the pixmap to use for stippling.  
--  
-- Set the stippling for the graphics context.  
------------------------------------------------------------------------------  

    procedure X_Set_Subwindow_Mode (Display        : X_Display;  
                                    Gc             : X_Gc;  
                                    Subwindow_Mode : X_Subwindow_Mode);  
------------------------------------------------------------------------------  
--  Display         - Specifies the display to use.  
--  Gc              - Specifies the graphics context to use.  
--  Subwindow_Mode  - Specifies the subwindow clip mode to use.  
--  
-- Specifies the subwindow clip mode to use in the graphics context.  
------------------------------------------------------------------------------  

    procedure X_Set_Tile (Display : X_Display;  
                          Gc      : X_Gc;  
                          Tile    : X_Pixmap);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Gc      - Specifies the graphics context to use.  
--  Tile    - Specifies the pixmap to use for fill tiling.  
--  
-- Set the fill tile for the graphics context.  
------------------------------------------------------------------------------  

    procedure X_Set_Ts_Origin (Display : X_Display;  
                               Gc      : X_Gc;  
                               X       : S_Short;  
                               Y       : S_Short);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Gc      - Specifies the graphics context to use.  
--  X       - Specifies the X coordinate for tiling/stippling.  
--  Y       - Specifies the Y coordinate for tiling/stippling.  
--  
-- Set the tile/stipple origin for the graphics context.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- Special interfaces for X Library Extension builders.  
------------------------------------------------------------------------------  

    procedure Private_X_Flush_Gc (Display : X_Display;  
                                  Gc      : X_Gc);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Gc      - Specifies the GC to flush.  
--  
-- Flush any pending GC changes to the display.  
-- **Only call this when the display is locked.**  
------------------------------------------------------------------------------  

    procedure Private_X_Flush_Gc_Cache  
                 (Display : X_Display;  
                  Gc      : X_Gc) renames Private_X_Flush_Gc;  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Gc      - Specifies the GC to flush.  
--  
-- Flush any pending GC changes to the display.  
-- **Only call this when the display is locked.**  
------------------------------------------------------------------------------  

--
end Xlbp_Gc;  

