with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  
with Xlbt_Basic;  
use Xlbt_Basic;  
with Xlbt_Geometry;  
use Xlbt_Geometry;  
with Xlbt_Hint;  
use Xlbt_Hint;  
with Xlbt_String;  
use Xlbt_String;  
with Xlbt_Window;  
use Xlbt_Window;  

package Xlbp_Geometry is  
------------------------------------------------------------------------------  
-- X Library Window Geometries  
--  
-- Xlbp_Geometry - Used to parse string representations of window geometries  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
    procedure X_Parse_Geometry  
                 (Geometry :        X_String;  
                  X        : in out S_Short;  
                  Y        : in out S_Short;  
                  Width    : in out U_Short;  
                  Height   : in out U_Short;  
                  Flags    : out    X_Parse_Geometry_Flags);  
------------------------------------------------------------------------------  
--  Geometry- Specifies the geometry string to parse.  
--              {=}<width>x<height>{+|-}<X_Offset>{+|-}<Y_Offset>  
--  X       - Receives the X offset, or 0 if not set.  
--  Y       - Receives the Y offset, or 0 if not set.  
--  Width   - Receives the width, or 0 if not set.  
--  Height  - Receives the height, or 0 if not set.  
--  Flags   - Receives the bits X_Value, Y_Value, Width_Value, Height_Value,  
--            X_Negative, and Y_Negative indicating which of the output  
--            parameters are set and which are really negative.  
--  
-- Parses the geometry string passed in and returns the components.  
------------------------------------------------------------------------------  

    procedure X_Geometry (Display      :     X_Display;  
                          Screen       :     X_Screen_Number;  
                          Geometry     :     X_String;  
                          Default      :     X_String;  
                          Border_Width :     U_Short;  
                          Font_Width   :     U_Short;  
                          Font_Height  :     U_Short;  
                          X_Additional :     S_Short;  
                          Y_Additional :     S_Short;  
                          X            : out S_Short;  
                          Y            : out S_Short;  
                          Width        : out U_Short;  
                          Height       : out U_Short;  
                          Flags        : out X_Parse_Geometry_Flags);  
------------------------------------------------------------------------------  
--  Display      - Specifies the display to use.  
--  Screen       - Specifies the screen to use for the computation.  
--  Geometry     - Specifies the geometry string to parse.  
--                {=}<width>x<height>{+|-}<X_Offset>{+|-}<Y_Offset>  
--  Default      - Specifies the default geometry spec for the window  
--  Border_Width - Specifies the border width.  
--  Font_Width   - Specifies the font height in pixels (increment size)  
--  Font_Height  - Specifies the font width in pixels (increment size)  
--  X_Additional - Specifies the additional X interior padding needed in window  
--  Y_Additional - Specifies the additional Y interior padding needed in window  
--  X            - Receives the X offset, or 0 if not set.  
--  Y            - Receives the Y offset, or 0 if not set.  
--  Width        - Receives the width, or 0 if not set.  
--  Height       - Receives the height, or 0 if not set.  
--  Flags        - Receives the bits X_Value, Y_Value, Width_Value,  
--                 Height_Value, X_Negative, and Y_Negative indicating which of  
--                 the output parameters are set and which are really negative.  
------------------------------------------------------------------------------  

    procedure X_Wm_Geometry (Display          :     X_Display;  
                             Screen           :     X_Screen_Number;  
                             User_Geometry    :     X_String;  
                             Default_Geometry :     X_String;  
                             Border_Width     :     U_Short;  
                             Hints            :     X_Size_Hints;  
                             X                : out S_Short;  
                             Y                : out S_Short;  
                             Width            : out U_Short;  
                             Height           : out U_Short;  
                             Gravity          : out X_Window_Gravity;  
                             Flags            : out X_Parse_Geometry_Flags);  
------------------------------------------------------------------------------  
--  Display          - Specifies the display to use.  
--  Screen           - Specifies the screen to use.  
--  User_Geometry    - Specifies the user-specified geometry or "".  
--  Default_Geometry - Specifies the application's default geometry or "".  
--  Border_Width     - Specifies the border width.  
--  Hints            - Specifies the size hints for the window in the normal state.  
--  X                - Receives the X offset.  
--  Y                - Receives the Y offset.  
--  Width            - Receives the width.  
--  Height           - Receives the height.  
--  Gravity          - Receives the window gravity.  
--  Flags            - Receives the output flags.  
--  
-- Combines any geometry information specified by the user and by the  
-- application with the size hints (usually the ones that were stored into  
-- the application shell's Xa_Wm_Size_Hints) and returns the position, size,  
-- gravity that describe the window.  
--  
-- If the base size is not set in the X_Size_Hints then the minimum size is  
-- used if set.  Otherwise a base size of 0 is assumed.  
--  
-- A mask that describes which values came from the user specification and  
-- whether or not the position coordinates are relative to the right and bottom  
-- edges is returned.  The X/Y values returned have already been adjusted  
-- to account for edge-relativity.  
--  
-- Note that an invalid geometry specification can cause a width or height of  
-- 0 to be returned.  
------------------------------------------------------------------------------  

--
end Xlbp_Geometry;  

