with Xlbt_Basic;  
use Xlbt_Basic;  
with Xlbt_Key;  
use Xlbt_Key;  
with Xlbt_Grab;  
use Xlbt_Grab;  
with Xlbt_Pointer;  
use Xlbt_Pointer;  

package Xlbp_Grab is  
------------------------------------------------------------------------------  
-- X Library Grabs - Keyboard, Pointer, Server  
--  
-- Xlbp_Grab - Controls for "grabs"  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
    procedure X_Allow_Events (Display : X_Display;  
                              Mode    : X_Allow_Event_Mode;  
                              Time    : X_Time);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to be used.  
--  Mode    - Specifies the event mode to use.  
--  Time    - Specifies the time, in milliseconds, to be used.  
--  
-- Release some queued events if the client has caused some device to freeze.  
-- The function has no effect if the specified time is earlier than the last  
-- grab-time or if the time is later than the current X server time.  
------------------------------------------------------------------------------  

    procedure X_Change_Active_Pointer_Grab (Display    : X_Display;  
                                            Event_Mask : X_Event_Mask;  
                                            Cursor     : X_Cursor;  
                                            Time       : X_Time);  
------------------------------------------------------------------------------  
--  Display    - Specifies the display to be used.  
--  Event_Mask - Specifies which pointer events are reported.  
--  Cursor     - Specifies the cursor to be used during the grab.  
--  Time       - Specifies the time, in milliseconds, to be used; or  
--               Current_Time.  
--  
-- Changes the specified dynamic parameters if the pointer is actively  
-- grabbed by the client and if the time is no earlier than the last-pointer  
-- grab and no later than the current server time.  
------------------------------------------------------------------------------  

    procedure X_Grab_Button (Display       : X_Display;  
                             Button        : X_Button_Name;  
                             Modifiers     : X_Key_Button_Mask;  
                             Grab_Window   : X_Window;  
                             Owner_Events  : Boolean;  
                             Event_Mask    : X_Event_Mask;  
                             Pointer_Mode  : X_Grab_Mode;  
                             Keyboard_Mode : X_Grab_Mode;  
                             Confine_To    : X_Window;  
                             Cursor        : X_Cursor);  
------------------------------------------------------------------------------  
--  Display       - Specifies the display to be used.  
--  Button        - Specifies the pointer button that is to be grabbed  
--                  when the specified modifiers are down; or Any_Button.  
--  Modifiers     - Specifies the modifiers that cause the grab; or  
--                  Any_Modifier.  
--  Grab_Window   - Specifies the window relative to which events are  
--                  reported while it is grabbed.  
--  Owner_Events  - Specifies if the pointer events are to be reported  
--                  normal (True) or with respect to the grab window  
--                  if selected by the event mask (False).  
--  Event_Mask    - Specifies which pointer events are reported to the client.  
--  Pointer_Mode  - Controls further processing of pointer events.  
--                  Grab_Mode_Sync or Grab_Mode_Async.  
--  Keyboard_Mode - Controls further processing of keyboard events.  
--                  Grab_Mode_Sync or Grab_Mode_Async.  
--  Confine_To    - Specifies a window to confine the pointer to; or  
--                  None_X_Window.  
--  Cursor        - Specifies the cursor to use during the grab.  
--  
-- Establishes a passive grab for a particular (set of) pointer button chord(s).  
------------------------------------------------------------------------------  

    function X_Grab_Keyboard (Display       : X_Display;  
                              Grab_Window   : X_Window;  
                              Owner_Events  : Boolean;  
                              Pointer_Mode  : X_Grab_Mode;  
                              Keyboard_Mode : X_Grab_Mode;  
                              Time          : X_Time) return X_Grab_Status;  
------------------------------------------------------------------------------  
--  Display       - Specifies the display to be used.  
--  Grab_Window   - Specifies the window relative to which events are  
--                  reported while it is grabbed.  
--  Owner_Events  - Specifies if the pointer events are to be reported  
--                  normal (True) or with respect to the grab window  
--                  if selected by the event mask (False).  
--  Event_Mask     - Specifies which pointer events are reported to the client.  
--  Pointer_Mode  - Controls further processing of pointer events.  
--                  Grab_Mode_Sync or Grab_Mode_Async.  
--  Keyboard_Mode - Controls further processing of keyboard events.  
--                  Grab_Mode_Sync or Grab_Mode_Async.  
--  Time          - Specifies the time, in milliseconds, to be used.  
--  
-- Grabs control of the main keyboard and returns Grab_Success if it succeeded.  
------------------------------------------------------------------------------  

    procedure X_Grab_Key (Display       : X_Display;  
                          Key           : X_Key_Code;  
                          Modifiers     : X_Key_Button_Mask;  
                          Grab_Window   : X_Window;  
                          Owner_Events  : Boolean;  
                          Pointer_Mode  : X_Grab_Mode;  
                          Keyboard_Mode : X_Grab_Mode);  
------------------------------------------------------------------------------  
--  Display       - Specifies the display to be used.  
--  Key           - Specifies the keyboard key that is to be grabbed  
--                  when the specified modifiers are down; or Any_Key.  
--  Modifiers     - Specifies the modifiers that cause the grab; or  
--                  Any_Modifier.  
--  Grab_Window   - Specifies the window relative to which events are  
--                  reported while it is grabbed.  
--  Owner_Events  - Specifies if the pointer events are to be reported  
--                  normal (True) or with respect to the grab window  
--                  if selected by the event mask (False).  
--  Event_Mask    - Specifies which pointer events are reported to the client.  
--  Pointer_Mode  - Controls further processing of pointer events.  
--                  Grab_Mode_Sync or Grab_Mode_Async.  
--  Keyboard_Mode - Controls further processing of keyboard events.  
--                  Grab_Mode_Sync or Grab_Mode_Async.  
--  
-- Establishes a passive grab for a particular (set of) key chord(s).  
------------------------------------------------------------------------------  

    function X_Grab_Pointer (Display       : X_Display;  
                             Grab_Window   : X_Window;  
                             Owner_Events  : Boolean;  
                             Event_Mask    : X_Event_Mask;  
                             Pointer_Mode  : X_Grab_Mode;  
                             Keyboard_Mode : X_Grab_Mode;  
                             Confine_To    : X_Window;  
                             Cursor        : X_Cursor;  
                             Time          : X_Time) return X_Grab_Status;  
------------------------------------------------------------------------------  
--  Display       - Specifies the display to be used.  
--  Grab_Window   - Specifies the window relative to which events are  
--                  reported while it is grabbed.  
--  Owner_Events  - Specifies if the pointer events are to be reported  
--                  normal (True) or with respect to the grab window  
--                  if selected by the event mask (False).  
--  Event_Mask    - Specifies which pointer events are reported to the client.  
--  Pointer_Mode  - Controls further processing of pointer events.  
--                  Grab_Mode_Sync or Grab_Mode_Async.  
--  Keyboard_Mode - Controls further processing of keyboard events.  
--                  Grab_Mode_Sync or Grab_Mode_Async.  
--  Confine_To    - Specifies a window to confine the pointer to; or  
--                  None_X_Window.  
--  Cursor        - Specifies the cursor to use during the grab.  
--  Time          - Specifies the time, in milliseconds, to be used; or  
--                  Current_Time.  
--  
-- Grabs control of the pointer and returns Grab_Success if successful.  
------------------------------------------------------------------------------  

    procedure X_Grab_Server (Display : X_Display);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to be used.  
--  
-- Grabs the server and effectively shuts it down except for this connection.  
------------------------------------------------------------------------------  

    procedure X_Ungrab_Button (Display     : X_Display;  
                               Button      : X_Button_Name;  
                               Modifiers   : X_Key_Button_Mask;  
                               Grab_Window : X_Window);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to be used.  
--  Button      - Specifies the pointer button that is to be ungrabbed  
--                when the specified modifiers are down; or Any_Button.  
--  Modifiers   - Specifies the modifiers that caused the grab; or  
--                Any_Modifier.  
--  Grab_Window - Specifies the window relative to which events were  
--                reported while it was grabbed.  
--  
-- Release the button and any queued events if this client has actively  
-- grabbed the button.  The function does not release the button if the  
-- specified time is earlier than the last-pointer-grab time or is later than  
-- the current X server time.  
------------------------------------------------------------------------------  

    procedure X_Ungrab_Keyboard (Display : X_Display;  
                                 Time    : X_Time);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to be used.  
--  Time    - Specifies the time, in milliseconds, to be used.  
--  
-- Release the keyboard and any queued events if this client has actively  
-- grabbed the keyboard.  The function does not release the keyboard if the  
-- specified time is earlier than the last-pointer-grab time or is later than  
-- the current X server time.  
------------------------------------------------------------------------------  

    procedure X_Ungrab_Key (Display     : X_Display;  
                            Key         : X_Key_Code;  
                            Modifiers   : X_Key_Button_Mask;  
                            Grab_Window : X_Window);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to be used.  
--  Key         - Specifies the keyboard key that is to be ungrabbed  
--                when the specified modifiers are down; or Any_Key.  
--  Modifiers   - Specifies the modifiers that caused the grab; or  
--                Any_Modifier.  
--  Grab_Window - Specifies the window relative to which events were  
--                reported while it was grabbed.  
--  
-- Release the key and any queued events if this client has actively  
-- grabbed the key.  The function does not release the key if the  
-- specified time is earlier than the last-pointer-grab time or is later than  
-- the current X server time.  
------------------------------------------------------------------------------  

    procedure X_Ungrab_Pointer (Display : X_Display;  
                                Time    : X_Time);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to be used.  
--  Time    - Specifies the time, in milliseconds, to be used.  
--  
-- Release the pointer and any queued events if this client has actively  
-- grabbed the pointer from X_Grab_Pointer or X_Grab_Button or from a normal  
-- button press.  The function does not release the pointer if the specified  
-- time is earlier than the last-pointer-grab time or is later than the  
-- current X server time.  
------------------------------------------------------------------------------  

    procedure X_Ungrab_Server (Display : X_Display);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to be used.  
--  
-- Ungrabs the server.  
------------------------------------------------------------------------------  

end Xlbp_Grab;  

