with Xlbt_Basic;  
use Xlbt_Basic;  
with Xlbt_Host;  
use Xlbt_Host;  

package Xlbp_Host is  
------------------------------------------------------------------------------  
-- X Library Host Access Control  
--  
-- Xlbp_Host - Control access to the X server on a host-name basis.  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- New Host/Client Controls  
------------------------------------------------------------------------------  

    procedure X_Add_Host (Display : X_Display;  
                          Host    : X_Host_Address);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Host    - Specifies the host address.  
--  
-- Add the host to the access list.  
------------------------------------------------------------------------------  

    procedure X_Add_Hosts (Display : X_Display;  
                           Hosts   : X_Host_Address_Array);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Hosts   - Specifies several host addresses.  
--  
-- Add the hosts to the access list.  
------------------------------------------------------------------------------  

    function X_List_Hosts (Display : X_Display) return X_Host_List;  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  
-- Returns a list of allowed hosts and a flag indicating whether or not  
-- the list is enabled.  Free the list after use with Free_X_Host_List.  
------------------------------------------------------------------------------  

    procedure X_Remove_Host (Display : X_Display;  
                             Host    : X_Host_Address);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Host    - Specifies the host address.  
--  
-- Remove the host from the access list.  
------------------------------------------------------------------------------  

    procedure X_Remove_Hosts (Display : X_Display;  
                              Hosts   : X_Host_Address_Array);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Hosts   - Specifies several host addresses.  
--  
-- Remove the hosts from the access list.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- Access Control  
------------------------------------------------------------------------------  

    procedure X_Disable_Access_Control (Display : X_Display);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  
-- Turns off access control for the X server.  
------------------------------------------------------------------------------  

    procedure X_Enable_Access_Control (Display : X_Display);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  
-- Turns on access control for the X server.  
------------------------------------------------------------------------------  

    procedure X_Set_Access_Control (Display : X_Display;  
                                    Mode    : X_Change_Access);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Mode    - Specifies the new access control mode.  
--  
-- Sets access control for the X server.  
------------------------------------------------------------------------------  

--
------------------------------------------------------------------------------  
-- Existing Client Controls  
------------------------------------------------------------------------------  

    procedure X_Kill_Client (Display  : X_Display;  
                             Resource : X_Id);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Resource    - Specifies an ID than can be used to deduce a client.  
--  
-- Forces a close-down of the client that create the resource if a valid  
-- resource is specified.  
------------------------------------------------------------------------------  

    procedure X_Set_Close_Down_Mode (Display : X_Display;  
                                     Mode    : X_Close_Down_Mode);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Mode    - Specifies the new mode.  
--  
-- Determines what will happen to the client's resources at connection close.  
-- Connection starts in Destroy_All mode.  
------------------------------------------------------------------------------  

    procedure X_Add_To_Save_Set (Display : X_Display;  
                                 Window  : X_Window);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Window  - Specifies the window to use.  
--  
-- Same as X_Change_Save_Set with Set_Mode_Insert.  
------------------------------------------------------------------------------  

    procedure X_Change_Save_Set (Display : X_Display;  
                                 Window  : X_Window;  
                                 Mode    : X_Change_Save_Set_Mode);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Window  - Specifies the window to use.  
--  Mode    - Specifies the save-set mode to use.  
--  
-- Either adds or removes a subwindow from a client's save-set.  The specified  
-- window must have been created by another client.  
------------------------------------------------------------------------------  

    procedure X_Remove_From_Save_Set (Display : X_Display;  
                                      Window  : X_Window);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Window  - Specifies the window to use.  
--  
-- Same as X_Change_Save_Set with Set_Mode_Delete.  
------------------------------------------------------------------------------  

end Xlbp_Host;  

