with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  
with Xlbt_Basic;  
use Xlbt_Basic;  
with Xlbt_Input_Focus;  
use Xlbt_Input_Focus;  

package Xlbp_Input_Focus is  
------------------------------------------------------------------------------  
-- X Library Input Focus Control  
--  
-- Xlbp_Input_Focus - Controls for setting and changing the server's input focus  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
    procedure X_Get_Input_Focus (Display   :     X_Display;  
                                 Focus     : out X_Window;  
                                 Revert_To : out X_Revert_Focus_Mode);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Focus       - Receives the window which has the focus.  
--  Revert_To   - Receives the window to which the focus reverts if this  
--                window becomes not viewable.  
--  
-- Returns the current focus window and the reversion mode.  
------------------------------------------------------------------------------  

    procedure X_Set_Input_Focus (Display   : X_Display;  
                                 Focus     : X_Window;  
                                 Revert_To : X_Revert_Focus_Mode;  
                                 Time      : X_Time);  
------------------------------------------------------------------------------  
--  Display     - Specifies the display to use.  
--  Focus       - Specifies the window to receive the focus.  
--  Revert_To   - Specifies the window to which the focus reverts if this  
--                window becomes not viewable.  
--  Time        - Current_Time or a timestamp in milliseconds controlling  
--                when this takes effect.  
--  
-- Changes the input focus and the last-focus-change-time for the X server.  
------------------------------------------------------------------------------  

    procedure X_Warp_Pointer (Display       : X_Display;  
                              Source        : X_Window;  
                              Destination   : X_Window;  
                              Source_X      : S_Short;  
                              Source_Y      : S_Short;  
                              Source_Width  : U_Short;  
                              Source_Height : U_Short;  
                              Destination_X : S_Short;  
                              Destination_Y : S_Short);  
------------------------------------------------------------------------------  
--  Display        - Specifies the display to use.  
--  Source         - Specifies the source window ID; or None_X_Window.  
--  Destination    - Specifies the destination window ID; or None_X_Window.  
--  Source_X       - Specifies the X coordinate of the source rectangle.  
--  Source_Y       - Specifies the Y coordinate of the source rectangle.  
--  Source_Width   - Specifies the width, in pixels, of the source  
--                   rectangle.  
--  Source_Height  - Specifies the height, in pixels, of the source  
--                   rectangle.  
--  Destination_X  - Specifies the destination X coordinate.  
--  Destination_Y  - Specifies the destination Y coordinate.  
--  
-- If Destination_Window is None_X_Window then the move will use Destination_X/Y  
-- as displacements from the current pointer position.  
-- If Source_Window is None_X_Window then the pointer is moved unconditionally.  
-- If Source_Window is not None_X_Window then if the pointer is within the  
-- rectangle described by X/Y/Width/Height then it is warped.  If either  
-- Width/Height is zero then the current full window Width/Height minus the X/Y  
-- position is used (means go from the X/Y to the Right/Bottom size of the  
-- window).  
------------------------------------------------------------------------------  

end Xlbp_Input_Focus;  

