with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  
with Xlbt_Basic;  
use Xlbt_Basic;  
with Xlbt_Request;  
use Xlbt_Request;  

with Xlbp_Extension;  
use Xlbp_Extension;  

with Xlbip_Internal;  
use Xlbip_Internal;  
with Xlbip_Put_Request;  
use Xlbip_Put_Request;  

package body Xlbp_Pixmap is  
------------------------------------------------------------------------------  
-- X Library Pixmaps  
--  
-- Xlbp_Pixmap - Create and destroy Pixmaps  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
    function X_Create_Pixmap (Display  : X_Display;  
                              Drawable : X_Drawable;  
                              Width    : U_Short_Positive;  
                              Height   : U_Short_Positive;  
                              Depth    : U_Char) return X_Pixmap is  
        Pid : X_Pixmap;  
    begin  

----Lock the display.  

        Lock_Display (Display);  
        begin  

            Pid := X_Pixmap'(Drawable =>  
                                (Id => Xlbp_Extension.X_Alloc_Id (Display)));  
            Put_X_Create_Pixmap_Request  
               (Display, (Kind     => Create_Pixmap,  
                          Length   => X_Create_Pixmap_Request'Size / 32,  
                          Drawable => Drawable,  
                          Width    => Width,  
                          Height   => Height,  
                          Depth    => Depth,  
                          Pixmap   => Pid));  

----Catch exceptions.  

        exception  
            when others =>  
                Unlock_Display (Display);  
                raise;  
        end;  

----Unlock; sync; return.  

        Unlock_Display (Display);  
        Sync_Handle (Display);  
        return Pid;  

    end X_Create_Pixmap;  

--
    procedure X_Free_Pixmap (Display : X_Display;  
                             Pixmap  : X_Pixmap) is  
    begin  

----Lock the display.  

        Lock_Display (Display);  
        begin  

            Put_X_Free_Pixmap_Request  
               (Display, (Kind   => Free_Pixmap,  
                          Length => X_Free_Pixmap_Request'Size / 32,  
                          Pad    => 0,  
                          Id     => Pixmap));  

----Catch exceptions.  

        exception  
            when others =>  
                Unlock_Display (Display);  
                raise;  
        end;  

----Unlock; sync; return.  

        Unlock_Display (Display);  
        Sync_Handle (Display);  

    end X_Free_Pixmap;  

--
end Xlbp_Pixmap;  

