with Xlbt_Arithmetic;  
use Xlbt_Arithmetic;  
with Xlbt_Basic;  
use Xlbt_Basic;  
with Xlbt_Key;  
use Xlbt_Key;  
with Xlbt_Window;  
use Xlbt_Window;  

package Xlbp_Window_Information is  
------------------------------------------------------------------------------  
-- X Library Miscellaneous Window Information  
--  
-- Xlbp_Window_Information - Provide useful miscellaneous information  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
    procedure X_Get_Geometry (Display      :     X_Display;  
                              Drawable     :     X_Drawable;  
                              Root         : out X_Window;  
                              X            : out S_Short;  
                              Y            : out S_Short;  
                              Width        : out U_Short;  
                              Height       : out U_Short;  
                              Border_Width : out U_Short;  
                              Depth        : out U_Char;  
                              Status       : out X_Status);  
------------------------------------------------------------------------------  
--  Display       - Specifies the display to use.  
--  Drawable      - Specifies the drawable (Window, Pixmap) to use.  
--  Root          - Receives the root window ID for the specified window.  
--  X             - Receives the X coordinate of the drawable.  
--  Y             - Receives the Y coordinate of the drawable.  These specify  
--                  the upper left outer corner relative to its parent's origin.  
--                  For pixmaps, these coordinates are always zero.  
--  Width         - Receives the width of the drawable.  
--  Height        - Receives the height of the drawable.  These are the  
--                  drawable's dimensions.  For windows, these specify the  
--                  inside size, not including the border.  
--  Border_Width  - Receives the border width in pixels.  For pixmaps, this  
--                  is always zero.  
--  Depth         - Receives the depth of the pixmap (bits per pixel) for the  
--                  object.  
--  Status        - Receives Successful or Failed.  
------------------------------------------------------------------------------  

    procedure X_Get_Window_Attributes (Display :        X_Display;  
                                       Window  :        X_Window;  
                                       Values  : in out X_Window_Attributes;  
                                       Status  : out    X_Status);  
------------------------------------------------------------------------------  
--  Display - Specifies the display to use.  
--  Window  - Specifies the window to use.  
--  Values  - Receives the attributes of the specified window.  
--  Status  - Receives Successful or Failed.  
------------------------------------------------------------------------------  

    procedure X_Query_Pointer (Display     :     X_Display;  
                               Window      :     X_Window;  
                               Root        : out X_Window;  
                               Child       : out X_Window;  
                               Root_X      : out U_Short;  
                               Root_Y      : out U_Short;  
                               Window_X    : out S_Short;  
                               Window_Y    : out S_Short;  
                               Mask        : out X_Key_Button_Mask;  
                               Same_Screen : out Boolean);  
------------------------------------------------------------------------------  
--  Display       - Specifies the display to use.  
--  Window        - Specifies the window to use.  
--  Root          - Receives the root window ID for the specified window.  
--                  Identifies the root window the pointer is currently on.  
--  Child         - Receives the child window ID that the pointer is on, if any.  
--  Root_X        - Receives the X coordinate relative to the root origin.  
--  Root_Y        - Receives the Y coordinate relative to the root origin.  
--  Window_X      - Receives the X coordinate relative to the specified window's  
--                  origin.  
--  Window_Y      - Receives the Y coordinate relative to the specified window's  
--                  origin.  
--  Mask          - Receives the current state of the modifier keys and pointer  
--                  buttons.  
--  Same_Screen   - Receives True if the pointer is on the same screen as the  
--                  window associated with the window specified.  Receives  
--                  False if not; in this case Child = None_X_Window and  
--                  Window_X/Y are both zero.  
--  
-- Returns the root window the pointer is logically on and the pointer  
-- coordinates relative to the root window's origin.  
------------------------------------------------------------------------------  

    procedure X_Query_Tree (Display  :     X_Display;  
                            Window   :     X_Window;  
                            Root     : out X_Window;  
                            Parent   : out X_Window;  
                            Children : out X_Window_List;  
                            Status   : out X_Status);  
------------------------------------------------------------------------------  
--  Display   - Specifies the display to use.  
--  Window    - Specifies the window to use.  
--  Root      - Receives the root window.  
--  Parent    - Receives the parent window.  
--  Children  - Receives the child list.  
--  Status    - Receives Successful or Failed.  Children = None_X_Window_List  
--              if this is Failed.  
--  
-- Returns the root, the parent, and a list of the children of the specified  
-- window.  The Successful/Failed status of the query is part of the return  
-- value.  Use Free_X_Window_List to deallocate the Children value when done.  
------------------------------------------------------------------------------  

    procedure X_Translate_Coordinates (Display       :     X_Display;  
                                       Source        :     X_Window;  
                                       Destination   :     X_Window;  
                                       Source_X      :     S_Short;  
                                       Source_Y      :     S_Short;  
                                       Destination_X : out S_Short;  
                                       Destination_Y : out S_Short;  
                                       Child         : out X_Window;  
                                       Same_Screen   : out Boolean);  
------------------------------------------------------------------------------  
--  Display        - Specifies the display to use.  
--  Source         - Specifies the source window.  
--  Destination    - Specifies the destination window.  
--  Source_X       - Specifies the source X coordinate.  
--  Source_Y       - Specifies the source Y coordinate.  
--  Destination_X  - Receives the destination X coordinate.  
--  Destination_Y  - Receives the destination Y coordinate.  
--  Child          - Receives the child window if the coordinates are  
--                   contained in a mapped child of the destination window.  
--  Same_Screen    - Receives True if Source_Window and Destination_Window  
--                   are on the same screen.  Receives False if not; in  
--                   this case then Child = None_X_Window and  
--                   Destination_X/Y are both zero.  
------------------------------------------------------------------------------  

end Xlbp_Window_Information;  

