with Xlbt_Error3;  

package Xlbt_Error is  
------------------------------------------------------------------------------  
-- X Library Protocol Error Codes and Packets - Normal User Interface  
--  
-- Xlbt_Error - Record definitions of the protocol error packets  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
    package E3 renames Xlbt_Error3;  

------------------------------------------------------------------------------  
-- X_Error_Code  
--  
--      Success       - everything's okay  
--      Bad_Request   - bad request code  
--      Bad_Value     - int parameter out of range  
--      Bad_Window    - parameter not a Window  
--      Bad_Pixmap    - parameter not a Pixmap  
--      Bad_Atom      - parameter not an Atom  
--      Bad_Cursor    - parameter not a Cursor  
--      Bad_Font      - parameter not a Font  
--      Bad_Match     - parameter mismatch  
--      Bad_Drawable  - parameter not a Pixmap or Window  
--      Bad_Access    - depending on context: key/button already grabbed,  
--                      attempt to free an illegal colormap entry,  
--                      attempt to store into a read-only color map entry,  
--                      attempt to modify the access control list from other  
--                      than the local host.  
--      Bad_Alloc     - insufficient resources  
--      Bad_Color     - no such colormap  
--      Bad_Gc        - parameter not a GC  
--      Bad_Id_Choice - choice not in range or already used  
--      Bad_Name      - font or color name doesn't exist  
--      Bad_Length    - Request length incorrect  
--  
--      Bad_Implementation - server defective  
--  
--      First_Extension_Error  
--      Last_Extension_Error  
------------------------------------------------------------------------------  

    subtype X_Error_Code is E3.X_Error_Code;  

    Success            : constant X_Error_Code := E3.Success;  
    Bad_Request        : constant X_Error_Code := E3.Bad_Request;  
    Bad_Value          : constant X_Error_Code := E3.Bad_Value;  
    Bad_Window         : constant X_Error_Code := E3.Bad_Window;  
    Bad_Pixmap         : constant X_Error_Code := E3.Bad_Pixmap;  
    Bad_Atom           : constant X_Error_Code := E3.Bad_Atom;  
    Bad_Cursor         : constant X_Error_Code := E3.Bad_Cursor;  
    Bad_Font           : constant X_Error_Code := E3.Bad_Font;  
    Bad_Match          : constant X_Error_Code := E3.Bad_Match;  
    Bad_Drawable       : constant X_Error_Code := E3.Bad_Drawable;  
    Bad_Access         : constant X_Error_Code := E3.Bad_Access;  
    Bad_Alloc          : constant X_Error_Code := E3.Bad_Alloc;  
    Bad_Color          : constant X_Error_Code := E3.Bad_Color;  
    Bad_Gc             : constant X_Error_Code := E3.Bad_Gc;  
    Bad_Id_Choice      : constant X_Error_Code := E3.Bad_Id_Choice;  
    Bad_Name           : constant X_Error_Code := E3.Bad_Name;  
    Bad_Length         : constant X_Error_Code := E3.Bad_Length;  
    Bad_Implementation : constant X_Error_Code := E3.Bad_Implementation;  

    First_Extension_Error : constant X_Error_Code := E3.First_Extension_Error;  
    Last_Extension_Error  : constant X_Error_Code := E3.Last_Extension_Error;  

    function "=" (A, B : X_Error_Code) return Boolean renames E3."=";  

    None_X_Error_Code : constant X_Error_Code := E3.None_X_Error_Code;  

------------------------------------------------------------------------------  
-- Error Events are differentiated using this record type.  
------------------------------------------------------------------------------  

    subtype X_Error_Contents is E3.X_Error_Contents;  

------------------------------------------------------------------------------  
-- X Protocol Error Packets  
------------------------------------------------------------------------------  

    subtype X_Error_With_Atom     is E3.X_Error_With_Atom;  
    subtype X_Error_With_Nothing  is E3.X_Error_With_Nothing;  
    subtype X_Error_With_Resource is E3.X_Error_With_Resource;  
    subtype X_Error_With_Value    is E3.X_Error_With_Value;  

end Xlbt_Error;  

