with Xlbt_Extension3;  
with Xlbt_Extension4;  

package Xlbt_Extension is  
------------------------------------------------------------------------------  
-- X Library Extensions - Normal User Interface  
--  
-- Xlbt_Extension - Types dealing with extensions to the X Library and to  
-- the X Protocol.  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1985 - 1989 by the Massachusetts Institute of Technology  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of MIT or Rational not be  
-- used in advertising or publicity pertaining to distribution of the software  
-- without specific, written prior permission.  
--  
-- MIT and Rational disclaim all warranties with regard to this software,  
-- including all implied warranties of merchantability and fitness, in no  
-- event shall MIT or Rational be liable for any special, indirect or  
-- consequential damages or any damages whatsoever resulting from loss of use,  
-- data or profits, whether in an action of contract, negligence or other  
-- tortious action, arising out of or in connection with the use or performance  
-- of this software.  
------------------------------------------------------------------------------  

--
    package E3 renames Xlbt_Extension3;  
    package E4 renames Xlbt_Extension4;  

------------------------------------------------------------------------------  
-- X_Extension_Number - each extension gets a unique (to this program) number  
------------------------------------------------------------------------------  

    subtype X_Extension_Number is E4.X_Extension_Number;  

    subtype Xen is X_Extension_Number;  

    function "=" (A, B : Xen) return Boolean renames E4."=";  
    function "+" (A, B : Xen) return Xen     renames E4."+";  
    function "-" (A, B : Xen) return Xen     renames E4."-";  
    function "-" (A : Xen)    return Xen     renames E4."-";  

------------------------------------------------------------------------------  
-- X_Ext_Data - Extensions need a way to hang private data on some structures.  
------------------------------------------------------------------------------  

    subtype X_Ext_Data is E4.X_Ext_Data;  

    None_X_Ext_Data : X_Ext_Data renames E4.None_X_Ext_Data;  

    function "=" (A, B : X_Ext_Data) return Boolean renames E4."=";  

    procedure Free_X_Ext_Data (Ext_Data : in out X_Ext_Data)  
        renames E4.Free_X_Ext_Data;  
    ----Free a single X_Ext_Data record.  

    procedure Free_X_Ext_Data_List (List : in out X_Ext_Data)  
        renames E4.Free_X_Ext_Data_List;  
    ----Free the linked list of X_Ext_Data records.  

------------------------------------------------------------------------------  
-- X_Ext_Codes - Extension Codes contains structures used by the extension  
--               mechanism.  
------------------------------------------------------------------------------  

    subtype X_Ext_Codes is E3.X_Ext_Codes;  

    None_X_Ext_Codes : X_Ext_Codes renames E3.None_X_Ext_Codes;  

    function "=" (A, B : X_Ext_Codes) return Boolean renames E3."=";  

    procedure Free_X_Ext_Codes (Ext : in out X_Ext_Codes)  
        renames E3.Free_X_Ext_Codes;  

------------------------------------------------------------------------------  
-- X_Extension - This structure is private to the library.  
------------------------------------------------------------------------------  

    subtype X_Extension is E3.X_Extension;  

    None_X_Extension : X_Extension renames E3.None_X_Extension;  

    function "=" (A, B : X_Extension) return Boolean renames E3."=";  

    procedure Free_X_Extension (Ext : in out X_Extension)  
        renames E3.Free_X_Extension;  
    ----Free a single X_Extension record.  

    procedure Free_X_Extension_List (List : in out X_Extension)  
        renames E3.Free_X_Extension_List;  
    ----Free a linked list of X_Extension records.  

end Xlbt_Extension;  

