with Xlbt_Visual3;  

package Xlbt_Visual is  
------------------------------------------------------------------------------  
-- X Library Visuals - Normal User Interface  
--  
-- Xlbt_Visual - Types dealing with Visual Display Types supported by a  
-- display.  
------------------------------------------------------------------------------  
-- Copyright 1989 - 1991 by Rational, Santa Clara, California.  
-- Copyright 1987 - 1989 by Digital Equipment Corporation, Maynard, Mass.  
-- Copyright 1987 - 1989 by Massachusetts Institute of Technology,  
--                          Cambridge, Massachusetts.  
--  
--                  All Rights Reserved.  
--  
-- Permission to use, copy, modify, and distribute this software and its  
-- documentation for any purpose and without fee is hereby granted,  
-- provided that the above copyright notice(s) appear in all copies and that  
-- both that copyright notice(s) and this permission notice appear in  
-- supporting documentation, and that the names of Digital, MIT, or Rational  
-- not be used in advertising or publicity pertaining to distribution of  
-- the software without specific, written prior permission.  
--  
-- Digital, MIT, and Rational disclaim all warranties with regard to this  
-- software, including all implied warranties of merchantability and fitness,  
-- in no event shall Digital, MIT, or Rational be liable for any special,  
-- indirect or consequential damages or any damages whatsoever resulting from  
-- loss of use, data or profits, whether in an action of contract, negligence  
-- or other tortious action, arising out of or in connection with the use or  
-- performance of this software.  
------------------------------------------------------------------------------  

--
    package V3 renames Xlbt_Visual3;  


------------------------------------------------------------------------------  
-- X_Display_Class - used in opening the connection  
-- Note that the statically allocated ones are even numbered and the  
-- dynamically changeable ones are odd numbered  
------------------------------------------------------------------------------  

    subtype X_Display_Class is V3.X_Display_Class;  

    Static_Gray  : constant X_Display_Class := V3.Static_Gray;  
    Grayscale    : constant X_Display_Class := V3.Grayscale;  
    Static_Color : constant X_Display_Class := V3.Static_Color;  
    Pseudo_Color : constant X_Display_Class := V3.Pseudo_Color;  
    True_Color   : constant X_Display_Class := V3.True_Color;  
    Direct_Color : constant X_Display_Class := V3.Direct_Color;  

    None_X_Display_Class : X_Display_Class renames V3.None_X_Display_Class;  

    function "=" (A, B : X_Display_Class) return Boolean renames V3."=";  

------------------------------------------------------------------------------  
-- Visual structure; contains information about colormapping possibilities.  
--  
--  Copy_From_Parent_Visual - Special X_Visual passed to Create_Window to  
--                          indicate "copy it from the parent window"  
------------------------------------------------------------------------------  

    subtype X_Visual      is V3.X_Visual;  
    subtype X_Visual_List is V3.X_Visual_List;  

    None_X_Visual           : X_Visual      renames V3.None_X_Visual;  
    None_X_Visual_List      : X_Visual_List renames V3.None_X_Visual_List;  
    Copy_From_Parent_Visual : X_Visual      renames V3.Copy_From_Parent_Visual;  

    function "=" (A, B : X_Visual)      return Boolean renames V3."=";  
    function "=" (A, B : X_Visual_List) return Boolean renames V3."=";  

    procedure Free_X_Visual (Visual : in out X_Visual) renames V3.Free_X_Visual;  
    procedure Free_X_Visual_List (List : in out X_Visual_List)  
        renames V3.Free_X_Visual_List;  

------------------------------------------------------------------------------  
-- X_Visual_Info - Visual Information  
--  
-- Information used by the visual utility routines to find desired visual  
-- type from the many visuals a display may support.  
------------------------------------------------------------------------------  

    subtype X_Visual_Info      is V3.X_Visual_Info;  
    subtype X_Visual_Info_List is V3.X_Visual_Info_List;  

    None_X_Visual_Info      : X_Visual_Info renames V3.None_X_Visual_Info;  
    None_X_Visual_Info_List : X_Visual_Info_List  
        renames V3.None_X_Visual_Info_List;  

    procedure Free_X_Visual_Info_List (List : in out X_Visual_Info_List)  
        renames V3.Free_X_Visual_Info_List;  

------------------------------------------------------------------------------  
-- Visual Information Flags  
--  
--      Visual_Bits_Per_Rgb_Mask  
--      Visual_Colormap_Size_Mask  
--      Visual_Blue_Mask_Mask  
--      Visual_Green_Mask_Mask  
--      Visual_Red_Mask_Mask  
--      Visual_Class_Mask  
--      Visual_Depth_Mask  
--      Visual_Screen_Mask  
--      Visual_Id_Mask  
------------------------------------------------------------------------------  

    subtype X_Visual_Info_Flags_Index is V3.X_Visual_Info_Flags_Index;  
    subtype X_Visual_Info_Flags       is V3.X_Visual_Info_Flags;  

    subtype Xvifi is X_Visual_Info_Flags_Index;  

    Visual_Bits_Per_Rgb_Mask  : constant Xvifi := V3.Visual_Bits_Per_Rgb_Mask;  
    Visual_Colormap_Size_Mask : constant Xvifi := V3.Visual_Colormap_Size_Mask;  
    Visual_Blue_Mask_Mask     : constant Xvifi := V3.Visual_Blue_Mask_Mask;  
    Visual_Green_Mask_Mask    : constant Xvifi := V3.Visual_Green_Mask_Mask;  
    Visual_Red_Mask_Mask      : constant Xvifi := V3.Visual_Red_Mask_Mask;  
    Visual_Class_Mask         : constant Xvifi := V3.Visual_Class_Mask;  
    Visual_Depth_Mask         : constant Xvifi := V3.Visual_Depth_Mask;  
    Visual_Screen_Mask        : constant Xvifi := V3.Visual_Screen_Mask;  
    Visual_Id_Mask            : constant Xvifi := V3.Visual_Id_Mask;  

    ----These flags are currently unused by the X Library.  

    Xvifi31 : constant Xvifi := V3.Xvifi31;  
    Xvifi30 : constant Xvifi := V3.Xvifi30;  
    Xvifi29 : constant Xvifi := V3.Xvifi29;  
    Xvifi28 : constant Xvifi := V3.Xvifi28;  
    Xvifi27 : constant Xvifi := V3.Xvifi27;  
    Xvifi26 : constant Xvifi := V3.Xvifi26;  
    Xvifi25 : constant Xvifi := V3.Xvifi25;  
    Xvifi24 : constant Xvifi := V3.Xvifi24;  
    Xvifi23 : constant Xvifi := V3.Xvifi23;  
    Xvifi22 : constant Xvifi := V3.Xvifi22;  
    Xvifi21 : constant Xvifi := V3.Xvifi21;  
    Xvifi20 : constant Xvifi := V3.Xvifi20;  
    Xvifi19 : constant Xvifi := V3.Xvifi19;  
    Xvifi18 : constant Xvifi := V3.Xvifi18;  
    Xvifi17 : constant Xvifi := V3.Xvifi17;  
    Xvifi16 : constant Xvifi := V3.Xvifi16;  
    Xvifi15 : constant Xvifi := V3.Xvifi15;  
    Xvifi14 : constant Xvifi := V3.Xvifi14;  
    Xvifi13 : constant Xvifi := V3.Xvifi13;  
    Xvifi12 : constant Xvifi := V3.Xvifi12;  
    Xvifi11 : constant Xvifi := V3.Xvifi11;  
    Xvifi10 : constant Xvifi := V3.Xvifi10;  
    Xvifi09 : constant Xvifi := V3.Xvifi09;  

    Visual_No_Mask : X_Visual_Info_Flags renames V3.Visual_No_Mask;  

    Visual_All_Mask : X_Visual_Info_Flags renames V3.Visual_All_Mask;  

    function "=" (A, B : Xvifi)               return Boolean renames V3."=";  
    function "=" (A, B : X_Visual_Info_Flags) return Boolean renames V3."=";  

end Xlbt_Visual;  

